searchData={"items":[{"type":"behaviour","title":"Jido","doc":"自動 (Jido) - A foundational framework for building autonomous, distributed agent systems in Elixir.\n\nThis module provides the main interface for interacting with Jido components, including:\n- Managing and interacting with Agents through a high-level API\n- Listing and retrieving Actions, Sensors, and Domains\n- Filtering and paginating results\n- Generating unique slugs for components","ref":"Jido.html"},{"type":"behaviour","title":"Agent Interaction Examples - Jido","doc":"# Find and act on an agent\n    \"agent-id\"\n    |> Jido.get_agent_by_id()\n    |> Jido.act(:command, %{param: \"value\"})\n\n    # Act asynchronously\n    {:ok, agent} = Jido.get_agent_by_id(\"agent-id\")\n    Jido.act_async(agent, :command)\n\n    # Send management commands\n    {:ok, agent} = Jido.get_agent_by_id(\"agent-id\")\n    Jido.manage(agent, :pause)","ref":"Jido.html#module-agent-interaction-examples"},{"type":"function","title":"Jido.clone_agent/3","doc":"Clones an existing agent with a new ID.","ref":"Jido.html#clone_agent/3"},{"type":"function","title":"Parameters - Jido.clone_agent/3","doc":"- `source_id`: ID of the agent to clone\n- `new_id`: ID for the new cloned agent\n- `opts`: Optional keyword list of options to override for the new agent","ref":"Jido.html#clone_agent/3-parameters"},{"type":"function","title":"Returns - Jido.clone_agent/3","doc":"- `{:ok, pid}` with the new agent's process ID\n- `{:error, reason}` if cloning fails","ref":"Jido.html#clone_agent/3-returns"},{"type":"function","title":"Examples - Jido.clone_agent/3","doc":"iex> {:ok, new_pid} = Jido.clone_agent(\"source-agent\", \"cloned-agent\")\n    {:ok, #PID<0.125.0>}","ref":"Jido.html#clone_agent/3-examples"},{"type":"type","title":"Jido.component_metadata/0","doc":"","ref":"Jido.html#t:component_metadata/0"},{"type":"callback","title":"Jido.config/0","doc":"","ref":"Jido.html#c:config/0"},{"type":"function","title":"Jido.ensure_started/1","doc":"Callback used by the generated `start_link/0` function.\nThis is where we actually call Jido.Supervisor.start_link.","ref":"Jido.html#ensure_started/1"},{"type":"function","title":"Jido.get_action_by_slug/1","doc":"","ref":"Jido.html#get_action_by_slug/1"},{"type":"function","title":"Jido.get_agent/2","doc":"Retrieves a running Agent by its ID.","ref":"Jido.html#get_agent/2"},{"type":"function","title":"Parameters - Jido.get_agent/2","doc":"- `id`: String or atom ID of the agent to retrieve\n- `opts`: Optional keyword list of options:\n  - `:registry`: Override the default agent registry","ref":"Jido.html#get_agent/2-parameters"},{"type":"function","title":"Returns - Jido.get_agent/2","doc":"- `{:ok, pid}` if agent is found and running\n- `{:error, :not_found}` if agent doesn't exist","ref":"Jido.html#get_agent/2-returns"},{"type":"function","title":"Examples - Jido.get_agent/2","doc":"iex> {:ok, agent} = Jido.get_agent(\"my-agent\")\n    {:ok, #PID<0.123.0>}\n\n    # Using a custom registry\n    iex> {:ok, agent} = Jido.get_agent(\"my-agent\", registry: MyApp.Registry)\n    {:ok, #PID<0.123.0>}","ref":"Jido.html#get_agent/2-examples"},{"type":"function","title":"Jido.get_agent!/2","doc":"Pipe-friendly version of get_agent that raises on errors.","ref":"Jido.html#get_agent!/2"},{"type":"function","title":"Parameters - Jido.get_agent!/2","doc":"- `id`: String or atom ID of the agent to retrieve\n- `opts`: Optional keyword list of options:\n  - `:registry`: Override the default agent registry","ref":"Jido.html#get_agent!/2-parameters"},{"type":"function","title":"Returns - Jido.get_agent!/2","doc":"- `pid` if agent is found\n- Raises `RuntimeError` if agent not found","ref":"Jido.html#get_agent!/2-returns"},{"type":"function","title":"Examples - Jido.get_agent!/2","doc":"iex> \"my-agent\" |> Jido.get_agent!() |> Jido.cmd(:command)\n    :ok","ref":"Jido.html#get_agent!/2-examples"},{"type":"function","title":"Jido.get_agent_by_slug/1","doc":"","ref":"Jido.html#get_agent_by_slug/1"},{"type":"function","title":"Jido.get_agent_state/1","doc":"Gets the current state of an agent.","ref":"Jido.html#get_agent_state/1"},{"type":"function","title":"Parameters - Jido.get_agent_state/1","doc":"- `agent_or_id`: Agent pid, ID, or return value from get_agent","ref":"Jido.html#get_agent_state/1-parameters"},{"type":"function","title":"Returns - Jido.get_agent_state/1","doc":"- `{:ok, state}` with the agent's current state\n- `{:error, reason}` if state couldn't be retrieved","ref":"Jido.html#get_agent_state/1-returns"},{"type":"function","title":"Examples - Jido.get_agent_state/1","doc":"iex> {:ok, state} = Jido.get_agent_state(\"my-agent\")\n    {:ok, %Jido.Agent.Server.State{...}}","ref":"Jido.html#get_agent_state/1-examples"},{"type":"function","title":"Jido.get_demo_by_slug/1","doc":"","ref":"Jido.html#get_demo_by_slug/1"},{"type":"function","title":"Jido.get_sensor_by_slug/1","doc":"","ref":"Jido.html#get_sensor_by_slug/1"},{"type":"function","title":"Jido.get_skill_by_slug/1","doc":"","ref":"Jido.html#get_skill_by_slug/1"},{"type":"function","title":"Jido.list_actions/1","doc":"","ref":"Jido.html#list_actions/1"},{"type":"function","title":"Jido.list_agents/1","doc":"","ref":"Jido.html#list_agents/1"},{"type":"function","title":"Jido.list_demos/1","doc":"","ref":"Jido.html#list_demos/1"},{"type":"function","title":"Jido.list_sensors/1","doc":"","ref":"Jido.html#list_sensors/1"},{"type":"function","title":"Jido.list_skills/1","doc":"","ref":"Jido.html#list_skills/1"},{"type":"function","title":"Jido.prompt/1","doc":"Retrieves a prompt file from the priv/prompts directory by its name.","ref":"Jido.html#prompt/1"},{"type":"function","title":"Parameters - Jido.prompt/1","doc":"- `name`: An atom representing the name of the prompt file (without .txt extension)","ref":"Jido.html#prompt/1-parameters"},{"type":"function","title":"Returns - Jido.prompt/1","doc":"The contents of the prompt file as a string if found, otherwise raises an error.","ref":"Jido.html#prompt/1-returns"},{"type":"function","title":"Examples - Jido.prompt/1","doc":"iex> Jido.prompt(:system)\n    \"You are a helpful AI assistant...\"\n\n    iex> Jido.prompt(:nonexistent)\n    ** (File.Error) could not read file priv/prompts/nonexistent.txt","ref":"Jido.html#prompt/1-examples"},{"type":"function","title":"Jido.resolve_pid/1","doc":"","ref":"Jido.html#resolve_pid/1"},{"type":"type","title":"Jido.server/0","doc":"","ref":"Jido.html#t:server/0"},{"type":"behaviour","title":"Jido.Action","doc":"Defines a discrete, composable unit of functionality within the Jido system.\n\nEach Action represents a delayed computation that can be composed with others\nto build complex workflows and workflows. Actions are defined at compile-time\nand provide a consistent interface for validating inputs, executing workflows,\nand handling results.","ref":"Jido.Action.html"},{"type":"behaviour","title":"Features - Jido.Action","doc":"- Compile-time configuration validation\n- Runtime input parameter validation\n- Consistent error handling and formatting\n- Extensible lifecycle hooks\n- JSON serialization support","ref":"Jido.Action.html#module-features"},{"type":"behaviour","title":"Usage - Jido.Action","doc":"To define a new Action, use the `Jido.Action` behavior in your module:\n\n    defmodule MyAction do\n      use Jido.Action,\n        name: \"my_action\",\n        description: \"Performs a specific workflow\",\n        category: \"processing\",\n        tags: [\"example\", \"demo\"],\n        vsn: \"1.0.0\",\n        schema: [\n          input: [type: :string, required: true]\n        ]\n\n      @impl true\n      def run(params, _context) do\n        # Your action logic here\n        {:ok, %{result: String.upcase(params.input)}}\n      end\n    end","ref":"Jido.Action.html#module-usage"},{"type":"behaviour","title":"Callbacks - Jido.Action","doc":"Implementing modules must define the following callback:\n\n- `c:run/2`: Executes the main logic of the Action.\n\nOptional callbacks for custom behavior:\n\n- `c:on_before_validate_params/1`: Called before parameter validation.\n- `c:on_after_validate_params/1`: Called after parameter validation.\n- `c:on_after_run/1`: Called after the Action's main logic has executed.","ref":"Jido.Action.html#module-callbacks"},{"type":"behaviour","title":"Error Handling - Jido.Action","doc":"Actions use the `OK` monad for consistent error handling. Errors are wrapped\nin `Jido.Error` structs for uniform error reporting across the system.","ref":"Jido.Action.html#module-error-handling"},{"type":"behaviour","title":"AI Tool Example - Jido.Action","doc":"Actions can be converted to LLM compatible tools for use with LLM-based systems. This is particularly\nuseful when building AI agents that need to interact with your system's capabilities:\n\n    # Define a weather action\n    iex> defmodule WeatherAction do\n    ...>   use Jido.Action,\n    ...>     name: \"get_weather\",\n    ...>     description: \"Gets the current weather for a location\",\n    ...>     category: \"weather\",\n    ...>     tags: [\"weather\", \"location\"],\n    ...>     vsn: \"1.0.0\",\n    ...>     schema: [\n    ...>       location: [\n    ...>         type: :string,\n    ...>         required: true,\n    ...>         doc: \"The city or location to get weather for\"\n    ...>       ]\n    ...>     ]\n    ...>\n    ...>   @impl true\n    ...>   def run(params, _context) do\n    ...>     # Weather API logic here\n    ...>     {:ok, %{temperature: 72, conditions: \"sunny\"}}\n    ...>   end\n    ...> end\n    {:module, WeatherAction, ...}\n\n    # Convert to tool format\n    iex> WeatherAction.to_tool()\n    %{\n      \"name\" => \"get_weather\",\n      \"description\" => \"Gets the current weather for a location\",\n      \"parameters\" => %{\n        \"type\" => \"object\",\n        \"properties\" => %{\n          \"location\" => %{\n            \"type\" => \"string\",\n            \"description\" => \"The city or location to get weather for\"\n          }\n        },\n        \"required\" => [\"location\"]\n      }\n    }\n\nThis tool definition can then be used with AI systems like OpenAI's function calling\nor other LLM frameworks that support tool/function specifications. The schema and\nvalidation ensure the AI system receives proper parameter constraints.","ref":"Jido.Action.html#module-ai-tool-example"},{"type":"behaviour","title":"Testing - Jido.Action","doc":"Actions can be tested directly by calling their `run/2` function with test parameters and context:\n\n    defmodule WeatherActionTest do\n      use ExUnit.Case\n\n      test \"gets weather for location\" do\n        params = %{location: \"Portland\"}\n        context = %{}\n\n        assert {:ok, result} = WeatherAction.run(params, context)\n        assert is_map(result)\n        assert result.temperature > 0\n      end\n\n      test \"handles invalid location\" do\n        params = %{location: \"\"}\n        context = %{}\n\n        assert {:error, error} = WeatherAction.run(params, context)\n        assert error.type == :validation_error\n      end\n    end\n\nFor testing Actions in a more complete runtime environment, including signal routing, state\nmanagement, and error handling, use `Jido.Workflow`. This provides a full test harness for\nvalidating Action behavior within workflows:\n\n    test \"weather action in workflow\" do\n      {:ok, result} = Workflow.run(WeatherAction, %{location: \"Seattle\"})\n      assert result.weather_data.temperature > 0\n    end\n\nSee `Jido.Workflow` documentation for more details on workflow-based testing.","ref":"Jido.Action.html#module-testing"},{"type":"behaviour","title":"Parameter Validation - Jido.Action","doc":"> **Note on Validation:** The validation process for Actions is intentionally open.\n> Only fields specified in the schema are validated. Unspecified fields are not\n> validated, allowing for easier Action composition. This approach enables Actions\n> to accept and pass along additional parameters that may be required by other\n> Actions in a chain without causing validation errors.","ref":"Jido.Action.html#module-parameter-validation"},{"type":"macro","title":"Jido.Action.__using__/1","doc":"Defines a new Action module.\n\nThis macro sets up the necessary structure and callbacks for a Action,\nincluding configuration validation and default implementations.","ref":"Jido.Action.html#__using__/1"},{"type":"macro","title":"Options - Jido.Action.__using__/1","doc":"* `:name` - Required. The name of the Action. Must contain only letters, numbers, and underscores.\n\n* `:description` (`t:String.t/0`) - A description of what the Action does.\n\n* `:category` (`t:String.t/0`) - The category of the Action.\n\n* `:tags` (list of `t:String.t/0`) - A list of tags associated with the Action. The default value is `[]`.\n\n* `:vsn` (`t:String.t/0`) - The version of the Action.\n\n* `:compensation` (`t:keyword/0`) - The default value is `[]`.\n\n  * `:enabled` (`t:boolean/0`) - The default value is `false`.\n\n  * `:max_retries` (`t:non_neg_integer/0`) - The default value is `1`.\n\n  * `:timeout` (`t:non_neg_integer/0`) - The default value is `5000`.\n\n* `:schema` (`t:keyword/0`) - A NimbleOptions schema for validating the Action's input parameters. The default value is `[]`.","ref":"Jido.Action.html#__using__/1-options"},{"type":"macro","title":"Examples - Jido.Action.__using__/1","doc":"defmodule MyAction do\n      use Jido.Action,\n        name: \"my_action\",\n        description: \"Performs a specific workflow\",\n        schema: [\n          input: [type: :string, required: true]\n        ]\n\n      @impl true\n      def run(params, _context) do\n        {:ok, %{result: String.upcase(params.input)}}\n      end\n    end","ref":"Jido.Action.html#__using__/1-examples"},{"type":"function","title":"Jido.Action.new/0","doc":"Raises an error indicating that Actions cannot be defined at runtime.\n\nThis function exists to prevent misuse of the Action system, as Actions\nare designed to be defined at compile-time only.","ref":"Jido.Action.html#new/0"},{"type":"function","title":"Returns - Jido.Action.new/0","doc":"Always returns `{:error, reason}` where `reason` is a config error.","ref":"Jido.Action.html#new/0-returns"},{"type":"function","title":"Examples - Jido.Action.new/0","doc":"iex> Jido.Action.new()\n    {:error, %Jido.Error{type: :config_error, message: \"Actions should not be defined at runtime\"}}","ref":"Jido.Action.html#new/0-examples"},{"type":"function","title":"Jido.Action.new/1","doc":"","ref":"Jido.Action.html#new/1"},{"type":"callback","title":"Jido.Action.on_after_run/1","doc":"Called after the Action's main logic has executed.\n\nThis optional callback allows for post-processing of the Action's result\nbefore it is returned to the caller.","ref":"Jido.Action.html#c:on_after_run/1"},{"type":"callback","title":"Parameters - Jido.Action.on_after_run/1","doc":"- `result`: The result map returned by the `run/2` function.","ref":"Jido.Action.html#c:on_after_run/1-parameters"},{"type":"callback","title":"Returns - Jido.Action.on_after_run/1","doc":"- `{:ok, modified_result}` where `modified_result` is a potentially modified result map.\n- `{:error, reason}` if post-processing fails.","ref":"Jido.Action.html#c:on_after_run/1-returns"},{"type":"callback","title":"Jido.Action.on_after_validate_params/1","doc":"Called after parameter validation.\n\nThis optional callback allows for post-processing of validated parameters\nbefore they are passed to the `run/2` function.","ref":"Jido.Action.html#c:on_after_validate_params/1"},{"type":"callback","title":"Parameters - Jido.Action.on_after_validate_params/1","doc":"- `params`: A map of validated input parameters.","ref":"Jido.Action.html#c:on_after_validate_params/1-parameters"},{"type":"callback","title":"Returns - Jido.Action.on_after_validate_params/1","doc":"- `{:ok, modified_params}` where `modified_params` is a map of potentially modified parameters.\n- `{:error, reason}` if post-processing fails.","ref":"Jido.Action.html#c:on_after_validate_params/1-returns"},{"type":"callback","title":"Jido.Action.on_before_validate_params/1","doc":"Called before parameter validation.\n\nThis optional callback allows for pre-processing of input parameters\nbefore they are validated against the Action's schema.","ref":"Jido.Action.html#c:on_before_validate_params/1"},{"type":"callback","title":"Parameters - Jido.Action.on_before_validate_params/1","doc":"- `params`: A map of raw input parameters.","ref":"Jido.Action.html#c:on_before_validate_params/1-parameters"},{"type":"callback","title":"Returns - Jido.Action.on_before_validate_params/1","doc":"- `{:ok, modified_params}` where `modified_params` is a map of potentially modified parameters.\n- `{:error, reason}` if pre-processing fails.","ref":"Jido.Action.html#c:on_before_validate_params/1-returns"},{"type":"callback","title":"Jido.Action.on_error/4","doc":"Handles errors and performs compensation when enabled.\n\nCalled when an error occurs during Action execution if compensation is enabled\nin the Action's configuration.","ref":"Jido.Action.html#c:on_error/4"},{"type":"callback","title":"Parameters - Jido.Action.on_error/4","doc":"- `failed_params`: The parameters that were passed to the failed execution\n- `error`: The Error struct describing what went wrong\n- `context`: The execution context at the time of failure\n- `opts`: Additional options for compensation handling","ref":"Jido.Action.html#c:on_error/4-parameters"},{"type":"callback","title":"Returns - Jido.Action.on_error/4","doc":"- `{:ok, result}` if compensation succeeded\n- `{:error, reason}` if compensation failed","ref":"Jido.Action.html#c:on_error/4-returns"},{"type":"callback","title":"Examples - Jido.Action.on_error/4","doc":"def on_error(params, error, context, opts) do\n      # Perform compensation logic\n      case rollback_changes(params) do\n        :ok -> {:ok, %{compensated: true, original_error: error}}\n        {:error, reason} -> {:error, \"Compensation failed: #{reason}\"}\n      end\n    end","ref":"Jido.Action.html#c:on_error/4-examples"},{"type":"callback","title":"Jido.Action.run/2","doc":"Executes the Action with the given parameters and context.\n\nThis callback must be implemented by modules using `Jido.Action`.","ref":"Jido.Action.html#c:run/2"},{"type":"callback","title":"Parameters - Jido.Action.run/2","doc":"- `params`: A map of validated input parameters.\n- `context`: A map containing any additional context for the workflow.","ref":"Jido.Action.html#c:run/2-parameters"},{"type":"callback","title":"Returns - Jido.Action.run/2","doc":"- `{:ok, result}` where `result` is a map containing the workflow's output.\n- `{:error, reason}` where `reason` describes why the workflow failed.","ref":"Jido.Action.html#c:run/2-returns"},{"type":"type","title":"Jido.Action.t/0","doc":"","ref":"Jido.Action.html#t:t/0"},{"type":"behaviour","title":"Jido.Agent","doc":"Defines an Agent within the Jido system - a compile-time defined entity for managing complex workflows\nthrough a sequence of Actions.","ref":"Jido.Agent.html"},{"type":"behaviour","title":"Overview - Jido.Agent","doc":"An Agent represents a stateful workflow executor that can plan and execute a series of Actions in a\ntype-safe and composable way. Agents provide a consistent interface for orchestrating complex operations\nwhile maintaining state validation, error handling, and extensibility through lifecycle hooks.","ref":"Jido.Agent.html#module-overview"},{"type":"behaviour","title":"Architecture - Jido.Agent","doc":"","ref":"Jido.Agent.html#module-architecture"},{"type":"behaviour","title":"Key Concepts - Jido.Agent","doc":"* Agents are defined at compile-time using the `use Jido.Agent` macro\n* Each Agent instance is created at server with a guaranteed unique ID\n* Agents maintain their own state schema for validation\n* Actions are registered with Agents and executed through a Runner\n* Lifecycle hooks enable customization of validation, planning and execution flows\n* All operations follow a consistent pattern returning `{:ok, result} | {:error, reason}`","ref":"Jido.Agent.html#module-key-concepts"},{"type":"behaviour","title":"Type Safety & Validation - Jido.Agent","doc":"* Configuration validated at compile-time via NimbleOptions\n* Server state changes validated against defined schema\n* Action modules checked for behavior implementation\n* Consistent return type enforcement across operations","ref":"Jido.Agent.html#module-type-safety-validation"},{"type":"behaviour","title":"Features - Jido.Agent","doc":"* Compile-time configuration validation\n* Server parameter validation via NimbleOptions\n* Comprehensive error handling with recovery hooks\n* Extensible lifecycle callbacks for all operations\n* JSON serialization support for persistence\n* Dynamic Action planning and execution\n* State management with validation and dirty tracking","ref":"Jido.Agent.html#module-features"},{"type":"behaviour","title":"Usage - Jido.Agent","doc":"","ref":"Jido.Agent.html#module-usage"},{"type":"behaviour","title":"Basic Example - Jido.Agent","doc":"defmodule MyAgent do\n      use Jido.Agent,\n        name: \"my_agent\",\n        description: \"Performs a complex workflow\",\n        category: \"processing\",\n        tags: [\"example\", \"demo\"],\n        vsn: \"1.0.0\",\n        schema: [\n          input: [type: :string, required: true],\n          status: [type: :atom, values: [:pending, :running, :complete]]\n        ],\n        actions: [MyAction1, MyAction2]\n    end\n\n    # Create and configure agent\n    {:ok, agent} = MyAgent.new()\n    {:ok, agent} = MyAgent.set(agent, input: \"test data\")\n\n    # Plan and execute actions\n    {:ok, agent} = MyAgent.plan(agent, MyAction1, %{value: 1})\n    {:ok, agent} = MyAgent.run(agent)  # Result stored in agent.result","ref":"Jido.Agent.html#module-basic-example"},{"type":"behaviour","title":"Customizing Behavior - Jido.Agent","doc":"defmodule CustomAgent do\n      use Jido.Agent,\n        name: \"custom_agent\",\n        schema: [status: [type: :atom]]\n\n      # Add pre-execution validation\n      def on_before_run(agent) do\n        if agent.state.status == :ready do\n          {:ok, agent}\n        else\n          {:error, \"Agent not ready\"}\n        end\n      end\n\n      # Custom error recovery\n      def on_error(agent, error) do\n        Logger.warning(\"Agent error\", error: error)\n        {:ok, %{agent | state: %{status: :error}}}\n      end\n    end","ref":"Jido.Agent.html#module-customizing-behavior"},{"type":"behaviour","title":"Callbacks - Jido.Agent","doc":"The following optional callbacks can be implemented to customize agent behavior.\nAll callbacks receive the agent struct and return `{:ok, agent} | {:error, reason}`.\n\n* `on_before_validate_state/1` - Pre-validation processing\n* `on_after_validate_state/1`  - Post-validation processing\n* `on_before_plan/3`           - Pre-planning processing with params\n* `on_before_run/1`            - Pre-execution validation/setup\n* `on_after_run/3`             - Post-execution processing\n* `on_error/2`                 - Error handling and recovery\n\nDefault implementations pass through the agent unchanged.","ref":"Jido.Agent.html#module-callbacks"},{"type":"behaviour","title":"Important Notes - Jido.Agent","doc":"Each Agent module defines its own struct type and behavior. Agent functions must be called\non matching agent structs:\n\n  # Correct usage:\n  agent = MyAgent.new()\n  {:ok, agent} = MyAgent.set(agent, attrs)\n\n  # Incorrect usage:\n  agent = MyAgent.new()\n  {:ok, agent} = OtherAgent.set(agent, attrs)  # Will fail - type mismatch","ref":"Jido.Agent.html#module-important-notes"},{"type":"behaviour","title":"Runner Architecture - Jido.Agent","doc":"The Runner executes actions in the agent's pending_instructions queue.\n\n* Default implementation: `Jido.Runner.Simple`\n* Custom runners must implement the `Jido.Runner` behavior\n* Runners handle instruction execution and state management\n* Support for different execution strategies (simple, chain, parallel)","ref":"Jido.Agent.html#module-runner-architecture"},{"type":"behaviour","title":"Error Handling - Jido.Agent","doc":"Errors are returned as tagged tuples: `{:error, reason}`\n\nCommon error types:\n* `:validation_error` - Schema/parameter validation failures\n* `:execution_error`  - Action execution failures\n* `:directive_error`  - Directive application failures\n\nSee `Jido.Action` for implementing compatible Actions.","ref":"Jido.Agent.html#module-error-handling"},{"type":"behaviour","title":"Type Specifications - Jido.Agent","doc":"* `t()` - The Agent struct type\n* `instruction()` - Single action with params\n* `instructions()` - List of instructions\n* `agent_result()` - `{:ok, t()} | {:error, term()}`","ref":"Jido.Agent.html#module-type-specifications"},{"type":"type","title":"Jido.Agent.agent_result/0","doc":"","ref":"Jido.Agent.html#t:agent_result/0"},{"type":"type","title":"Jido.Agent.agent_result_with_directives/0","doc":"","ref":"Jido.Agent.html#t:agent_result_with_directives/0"},{"type":"callback","title":"Jido.Agent.child_spec/1","doc":"","ref":"Jido.Agent.html#c:child_spec/1"},{"type":"function","title":"Jido.Agent.deregister_action/2","doc":"","ref":"Jido.Agent.html#deregister_action/2"},{"type":"callback","title":"Jido.Agent.handle_signal/1","doc":"","ref":"Jido.Agent.html#c:handle_signal/1"},{"type":"type","title":"Jido.Agent.instruction/0","doc":"","ref":"Jido.Agent.html#t:instruction/0"},{"type":"type","title":"Jido.Agent.instructions/0","doc":"","ref":"Jido.Agent.html#t:instructions/0"},{"type":"type","title":"Jido.Agent.map_result/0","doc":"","ref":"Jido.Agent.html#t:map_result/0"},{"type":"callback","title":"Jido.Agent.mount/2","doc":"","ref":"Jido.Agent.html#c:mount/2"},{"type":"function","title":"Jido.Agent.new/0","doc":"Raises an error indicating that Agents cannot be defined at server.\n\nThis function exists to prevent misuse of the Agent system, as Agents\nare designed to be defined at compile-time only.","ref":"Jido.Agent.html#new/0"},{"type":"function","title":"Returns - Jido.Agent.new/0","doc":"Always returns `{:error, reason}` where `reason` is a config error.","ref":"Jido.Agent.html#new/0-returns"},{"type":"function","title":"Examples - Jido.Agent.new/0","doc":"iex> Jido.Agent.new()\n    {:error, %Jido.Error{type: :config_error, message: \"Agents should not be defined at server\"}}","ref":"Jido.Agent.html#new/0-examples"},{"type":"function","title":"Jido.Agent.new/1","doc":"","ref":"Jido.Agent.html#new/1"},{"type":"callback","title":"Jido.Agent.on_after_run/3","doc":"Called after successful action execution.\nAllows post-processing of execution results.","ref":"Jido.Agent.html#c:on_after_run/3"},{"type":"callback","title":"Jido.Agent.on_after_validate_state/1","doc":"Called after state validation but before saving changes.\nAllows post-processing of validated state.","ref":"Jido.Agent.html#c:on_after_validate_state/1"},{"type":"callback","title":"Jido.Agent.on_before_plan/3","doc":"Called before planning actions, allows preprocessing of instructions prior to appending to the agent's queue","ref":"Jido.Agent.html#c:on_before_plan/3"},{"type":"callback","title":"Jido.Agent.on_before_run/1","doc":"Called after action planning but before execution.\nAllows inspection/modification of planned actions.","ref":"Jido.Agent.html#c:on_before_run/1"},{"type":"callback","title":"Jido.Agent.on_before_validate_state/1","doc":"Called before validating any state changes to the Agent.\nAllows custom preprocessing of state attributes.","ref":"Jido.Agent.html#c:on_before_validate_state/1"},{"type":"callback","title":"Jido.Agent.on_error/2","doc":"","ref":"Jido.Agent.html#c:on_error/2"},{"type":"callback","title":"Jido.Agent.process_result/2","doc":"","ref":"Jido.Agent.html#c:process_result/2"},{"type":"function","title":"Jido.Agent.register_action/2","doc":"Registers one or more action modules with the agent at server. Registered actions\ncan be used in planning and execution. Action modules must implement the `Jido.Action`\nbehavior and pass validation checks.","ref":"Jido.Agent.html#register_action/2"},{"type":"function","title":"Action Requirements - Jido.Agent.register_action/2","doc":"* Must be valid Elixir modules implementing `Jido.Action` behavior\n* Must be loaded and available at server\n* Must pass validation via `Jido.Util.validate_actions/1`","ref":"Jido.Agent.html#register_action/2-action-requirements"},{"type":"function","title":"Parameters - Jido.Agent.register_action/2","doc":"* `agent` - The agent struct to update\n* `action_module` - Single action module or list of action modules to register","ref":"Jido.Agent.html#register_action/2-parameters"},{"type":"function","title":"Returns - Jido.Agent.register_action/2","doc":"* `{:ok, updated_agent}` - Agent struct with newly registered actions prepended\n* `{:error, String.t()}` - If action validation fails","ref":"Jido.Agent.html#register_action/2-returns"},{"type":"function","title":"Examples - Jido.Agent.register_action/2","doc":"# Register single action\n    {:ok, agent} = MyAgent.register_action(agent, MyApp.Actions.ProcessFile)\n\n    # Register multiple actions\n    {:ok, agent} = MyAgent.register_action(agent, [Action1, Action2])","ref":"Jido.Agent.html#register_action/2-examples"},{"type":"function","title":"Server Considerations - Jido.Agent.register_action/2","doc":"* Actions persist only for the agent's lifecycle\n* Duplicates are prevented during validation\n* Most recently registered actions take precedence\n\nSee `Jido.Action` for implementing actions and `Jido.Util.validate_actions/1` for validation details.","ref":"Jido.Agent.html#register_action/2-server-considerations"},{"type":"function","title":"Jido.Agent.registered_actions/1","doc":"Returns all action modules registered with the agent in registration order.\nIncludes both compile-time and server-registered actions.","ref":"Jido.Agent.html#registered_actions/1"},{"type":"function","title":"Parameters - Jido.Agent.registered_actions/1","doc":"* `agent` - The agent struct to inspect","ref":"Jido.Agent.html#registered_actions/1-parameters"},{"type":"function","title":"Returns - Jido.Agent.registered_actions/1","doc":"* `[module()]` - Action modules ordered by registration time (newest first)","ref":"Jido.Agent.html#registered_actions/1-returns"},{"type":"function","title":"Examples - Jido.Agent.registered_actions/1","doc":"agent = MyAgent.new()\n    MyAgent.registered_actions(agent) #=> [DefaultAction, CoreAction]\n\n    {:ok, agent} = MyAgent.register_action(agent, CustomAction)\n    MyAgent.registered_actions(agent) #=> [CustomAction, DefaultAction, CoreAction]\n\nSee `register_action/2` for adding actions and `plan/3` for using them.","ref":"Jido.Agent.html#registered_actions/1-examples"},{"type":"callback","title":"Jido.Agent.shutdown/2","doc":"","ref":"Jido.Agent.html#c:shutdown/2"},{"type":"callback","title":"Jido.Agent.start_link/1","doc":"","ref":"Jido.Agent.html#c:start_link/1"},{"type":"type","title":"Jido.Agent.t/0","doc":"","ref":"Jido.Agent.html#t:t/0"},{"type":"module","title":"Jido.Agent.Server","doc":"GenServer implementation for managing agent processes.\n\nThis server handles the lifecycle and runtime execution of agents, including:\n- Agent initialization and startup\n- Signal processing and routing\n- State management and transitions\n- Child process supervision\n- Graceful shutdown\n\nThe server can be started in different modes (`:auto` or `:manual`) and supports\nboth synchronous (call) and asynchronous (cast) signal handling.","ref":"Jido.Agent.Server.html"},{"type":"function","title":"Jido.Agent.Server.call/3","doc":"Sends a synchronous signal to an agent and waits for the response.","ref":"Jido.Agent.Server.html#call/3"},{"type":"function","title":"Jido.Agent.Server.cast/2","doc":"Sends an asynchronous signal to an agent.","ref":"Jido.Agent.Server.html#cast/2"},{"type":"function","title":"Jido.Agent.Server.child_spec/1","doc":"Returns a child specification for starting the server under a supervisor.","ref":"Jido.Agent.Server.html#child_spec/1"},{"type":"macro","title":"Jido.Agent.Server.dbug/2","doc":"","ref":"Jido.Agent.Server.html#dbug/2"},{"type":"macro","title":"Jido.Agent.Server.error/2","doc":"","ref":"Jido.Agent.Server.html#error/2"},{"type":"function","title":"Jido.Agent.Server.start_link/1","doc":"Starts a new agent server process.","ref":"Jido.Agent.Server.html#start_link/1"},{"type":"function","title":"Options - Jido.Agent.Server.start_link/1","doc":"* `:id` - Unique identifier for the agent (auto-generated if not provided)\n  * `:agent` - Agent module or struct to be managed\n  * `:initial_state` - Initial state map for the agent\n  * `:registry` - Registry for process registration\n  * `:mode` - Operation mode (`:auto` or `:manual`)\n  * `:output` - Output destination for agent signals\n  * `:log_level` - Logging level\n  * `:max_queue_size` - Maximum size of pending signals queue","ref":"Jido.Agent.Server.html#start_link/1-options"},{"type":"function","title":"Returns - Jido.Agent.Server.start_link/1","doc":"* `{:ok, pid}` - Successfully started server process\n  * `{:error, reason}` - Failed to start server","ref":"Jido.Agent.Server.html#start_link/1-returns"},{"type":"type","title":"Jido.Agent.Server.start_option/0","doc":"","ref":"Jido.Agent.Server.html#t:start_option/0"},{"type":"function","title":"Jido.Agent.Server.state/1","doc":"Gets the current state of an agent.","ref":"Jido.Agent.Server.html#state/1"},{"type":"function","title":"Jido.Agent.Server.via_tuple/2","doc":"Returns a via tuple for process registration.","ref":"Jido.Agent.Server.html#via_tuple/2"},{"type":"module","title":"Jido.Instruction","doc":"Instructions in Jido represent discrete units of work that can be planned, validated, and executed by agents.\nThink of them as \"work orders\" that tell agents exactly what needs to be done and how to do it.","ref":"Jido.Instruction.html"},{"type":"module","title":"Core Concepts - Jido.Instruction","doc":"An Instruction wraps an Action module with everything it needs to execute:\n- The Action to perform (required)\n- Parameters for the action\n- Execution context\n- Runtime options","ref":"Jido.Instruction.html#module-core-concepts"},{"type":"module","title":"Structure - Jido.Instruction","doc":"Each Instruction contains:\n\n```elixir\n%Instruction{\n  id: \"inst_abc123\",           # Unique identifier\n  action: MyApp.Actions.DoTask, # The action module to execute\n  params: %{value: 42},        # Parameters for the action\n  context: %{user_id: \"123\"},  # Execution context\n  opts: [retry: true],         # Runtime options\n}\n```","ref":"Jido.Instruction.html#module-structure"},{"type":"module","title":"Creating Instructions - Jido.Instruction","doc":"Instructions support multiple creation formats for convenience:","ref":"Jido.Instruction.html#module-creating-instructions"},{"type":"module","title":"1. Full Struct (Most Explicit) - Jido.Instruction","doc":"```elixir\n%Instruction{\n  action: MyApp.Actions.ProcessOrder,\n  params: %{order_id: \"123\"},\n  context: %{tenant_id: \"456\"}\n}\n```","ref":"Jido.Instruction.html#module-1-full-struct-most-explicit"},{"type":"module","title":"2. Action Module Only (Simplest) - Jido.Instruction","doc":"```elixir\nMyApp.Actions.ProcessOrder\n```","ref":"Jido.Instruction.html#module-2-action-module-only-simplest"},{"type":"module","title":"3. Action With Parameters (Common) - Jido.Instruction","doc":"```elixir\n{MyApp.Actions.ProcessOrder, %{order_id: \"123\"}}\n```","ref":"Jido.Instruction.html#module-3-action-with-parameters-common"},{"type":"module","title":"4. Factory Function - Jido.Instruction","doc":"```elixir\nInstruction.new!(%{\n  action: MyApp.Actions.ProcessOrder,\n  params: %{order_id: \"123\"},\n  context: %{tenant_id: \"456\"}\n})\n```","ref":"Jido.Instruction.html#module-4-factory-function"},{"type":"module","title":"Working with Instructions - Jido.Instruction","doc":"","ref":"Jido.Instruction.html#module-working-with-instructions"},{"type":"module","title":"Normalization - Jido.Instruction","doc":"Convert various input formats to standard instruction structs:\n\n```elixir\n# Normalize a single instruction\n{:ok, [instruction]} = Instruction.normalize(MyApp.Actions.ProcessOrder)\n\n# Normalize with context\n{:ok, instructions} = Instruction.normalize(\n  [\n    MyApp.Actions.ValidateOrder,\n    {MyApp.Actions.ProcessOrder, %{priority: \"high\"}}\n  ],\n  %{tenant_id: \"123\"}  # Shared context\n)\n```","ref":"Jido.Instruction.html#module-normalization"},{"type":"module","title":"Validation - Jido.Instruction","doc":"Ensure instructions use allowed actions:\n\n```elixir\nallowed_actions = [\n  MyApp.Actions.ValidateOrder,\n  MyApp.Actions.ProcessOrder\n]\n\n:ok = Instruction.validate_allowed_actions(instructions, allowed_actions)\n```","ref":"Jido.Instruction.html#module-validation"},{"type":"module","title":"Common Patterns - Jido.Instruction","doc":"","ref":"Jido.Instruction.html#module-common-patterns"},{"type":"module","title":"1. Workflow Definition - Jido.Instruction","doc":"```elixir\ninstructions = [\n  MyApp.Actions.ValidateInput,\n  {MyApp.Actions.ProcessData, %{format: \"json\"}},\n  MyApp.Actions.SaveResults\n]\n```","ref":"Jido.Instruction.html#module-1-workflow-definition"},{"type":"module","title":"2. Conditional Execution - Jido.Instruction","doc":"```elixir\ninstructions = [\n  MyApp.Actions.ValidateOrder,\n  {MyApp.Actions.CheckInventory, %{strict: true}},\n  # Add fulfillment only if in stock\n  if has_stock? do\n    {MyApp.Actions.FulfillOrder, %{warehouse: \"main\"}}\n  end\n]\n|> Enum.reject(&is_nil/1)\n```","ref":"Jido.Instruction.html#module-2-conditional-execution"},{"type":"module","title":"3. Context Sharing - Jido.Instruction","doc":"```elixir\n# All instructions share common context\n{:ok, instructions} = Instruction.normalize(\n  [ValidateUser, ProcessOrder, NotifyUser],\n  %{\n    request_id: \"req_123\",\n    tenant_id: \"tenant_456\",\n  }\n)\n```","ref":"Jido.Instruction.html#module-3-context-sharing"},{"type":"module","title":"See Also - Jido.Instruction","doc":"- `Jido.Action` - Action behavior and implementation\n- `Jido.Runner` - Instruction execution\n- `Jido.Agent` - Agent-based execution","ref":"Jido.Instruction.html#module-see-also"},{"type":"type","title":"Jido.Instruction.action_module/0","doc":"","ref":"Jido.Instruction.html#t:action_module/0"},{"type":"type","title":"Jido.Instruction.action_params/0","doc":"","ref":"Jido.Instruction.html#t:action_params/0"},{"type":"type","title":"Jido.Instruction.action_tuple/0","doc":"","ref":"Jido.Instruction.html#t:action_tuple/0"},{"type":"type","title":"Jido.Instruction.instruction/0","doc":"","ref":"Jido.Instruction.html#t:instruction/0"},{"type":"type","title":"Jido.Instruction.instruction_list/0","doc":"","ref":"Jido.Instruction.html#t:instruction_list/0"},{"type":"function","title":"Jido.Instruction.new/1","doc":"Creates a new Instruction struct from a map or keyword list of attributes.","ref":"Jido.Instruction.html#new/1"},{"type":"function","title":"Parameters - Jido.Instruction.new/1","doc":"* `attrs` - Map or keyword list containing instruction attributes:\n    * `:action` - Action module (required)\n    * `:params` - Map of parameters (optional, default: %{})\n    * `:context` - Context map (optional, default: %{})\n    * `:opts` - Keyword list of options (optional, default: [])\n    * `:id` - String identifier (optional, defaults to UUID)","ref":"Jido.Instruction.html#new/1-parameters"},{"type":"function","title":"Returns - Jido.Instruction.new/1","doc":"* `{:ok, %Instruction{}}` - Successfully created instruction\n  * `{:error, :missing_action}` - If action is not provided\n  * `{:error, :invalid_action}` - If action is not a module","ref":"Jido.Instruction.html#new/1-returns"},{"type":"function","title":"Examples - Jido.Instruction.new/1","doc":"iex> Instruction.new(%{action: MyAction, params: %{value: 1}})\n    {:ok, %Instruction{action: MyAction, params: %{value: 1}}}\n\n    iex> Instruction.new(action: MyAction)\n    {:ok, %Instruction{action: MyAction}}\n\n    iex> Instruction.new(%{params: %{value: 1}})\n    {:error, :missing_action}","ref":"Jido.Instruction.html#new/1-examples"},{"type":"function","title":"Jido.Instruction.new!/1","doc":"Creates a new Instruction struct from a map or keyword list of attributes.\nReturns the struct directly or raises an error.","ref":"Jido.Instruction.html#new!/1"},{"type":"function","title":"Parameters - Jido.Instruction.new!/1","doc":"* `attrs` - Map or keyword list containing instruction attributes:\n    * `:action` - Action module (required)\n    * `:params` - Map of parameters (optional, default: %{})\n    * `:context` - Context map (optional, default: %{})\n    * `:opts` - Keyword list of options (optional, default: [])","ref":"Jido.Instruction.html#new!/1-parameters"},{"type":"function","title":"Returns - Jido.Instruction.new!/1","doc":"* `%Instruction{}` - Successfully created instruction","ref":"Jido.Instruction.html#new!/1-returns"},{"type":"function","title":"Raises - Jido.Instruction.new!/1","doc":"* `Jido.Error` - If action is missing or invalid","ref":"Jido.Instruction.html#new!/1-raises"},{"type":"function","title":"Examples - Jido.Instruction.new!/1","doc":"iex> Instruction.new!(%{action: MyAction, params: %{value: 1}})\n    %Instruction{action: MyAction, params: %{value: 1}}\n\n    iex> Instruction.new!(action: MyAction)\n    %Instruction{action: MyAction}\n\n    iex> Instruction.new!(%{params: %{value: 1}})\n    ** (Jido.Error) missing action","ref":"Jido.Instruction.html#new!/1-examples"},{"type":"function","title":"Jido.Instruction.normalize/3","doc":"Normalizes instruction shorthand input into instruction structs. Accepts a variety of input formats\nand returns a list of normalized instruction structs.","ref":"Jido.Instruction.html#normalize/3"},{"type":"function","title":"Parameters - Jido.Instruction.normalize/3","doc":"* `input` - One of:\n    * Single instruction struct (%Instruction{})\n    * List of instruction structs\n    * Single action module (MyApp.Actions.ProcessOrder)\n    * Action tuple with params ({MyApp.Actions.ProcessOrder, %{order_id: \"123\"}})\n    * Action tuple with empty params ({MyApp.Actions.ProcessOrder, %{}})\n    * Action tuple with context ({MyApp.Actions.ProcessOrder, %{}, %{tenant_id: \"123\"}})\n    * Action tuple with opts ({MyApp.Actions.ProcessOrder, %{}, %{}, [retry: true]})\n    * List of any combination of the above formats\n  * `context` - Optional context map to merge into all instructions (default: %{})\n  * `opts` - Optional keyword list of options (default: [])","ref":"Jido.Instruction.html#normalize/3-parameters"},{"type":"function","title":"Returns - Jido.Instruction.normalize/3","doc":"* `{:ok, [%Instruction{}]}` - List of normalized instruction structs\n  * `{:error, term()}` - If normalization fails","ref":"Jido.Instruction.html#normalize/3-returns"},{"type":"function","title":"Examples - Jido.Instruction.normalize/3","doc":"iex> Instruction.normalize(MyApp.Actions.ProcessOrder)\n    {:ok, [%Instruction{action: MyApp.Actions.ProcessOrder}]}\n\n    iex> Instruction.normalize({MyApp.Actions.ProcessOrder, %{order_id: \"123\"}})\n    {:ok, [%Instruction{action: MyApp.Actions.ProcessOrder, params: %{order_id: \"123\"}}]}\n\n    iex> Instruction.normalize([\n    ...>   MyApp.Actions.ValidateOrder,\n    ...>   {MyApp.Actions.ProcessOrder, %{priority: \"high\"}},\n    ...>   {MyApp.Actions.NotifyUser, %{}, %{user_id: \"123\"}}\n    ...> ])\n    {:ok, [%Instruction{...}, %Instruction{...}, %Instruction{...}]}","ref":"Jido.Instruction.html#normalize/3-examples"},{"type":"function","title":"Jido.Instruction.normalize!/3","doc":"Same as `normalize/3` but raises on error.","ref":"Jido.Instruction.html#normalize!/3"},{"type":"function","title":"Parameters - Jido.Instruction.normalize!/3","doc":"* `instruction` - Instruction to normalize\n  * `context` - Optional context map to merge\n  * `opts` - Optional options to merge","ref":"Jido.Instruction.html#normalize!/3-parameters"},{"type":"function","title":"Returns - Jido.Instruction.normalize!/3","doc":"* `[t()]` - List of normalized instructions","ref":"Jido.Instruction.html#normalize!/3-returns"},{"type":"function","title":"Raises - Jido.Instruction.normalize!/3","doc":"* `Jido.Error` - If instruction cannot be normalized","ref":"Jido.Instruction.html#normalize!/3-raises"},{"type":"function","title":"Examples - Jido.Instruction.normalize!/3","doc":"iex> Instruction.normalize!({MyAction, %{value: 42}})\n    [%Instruction{action: MyAction, params: %{value: 42}}]\n\n    iex> Instruction.normalize!(MyAction)\n    [%Instruction{action: MyAction, params: %{}}]","ref":"Jido.Instruction.html#normalize!/3-examples"},{"type":"type","title":"Jido.Instruction.t/0","doc":"","ref":"Jido.Instruction.html#t:t/0"},{"type":"function","title":"Jido.Instruction.validate_allowed_actions/2","doc":"Validates that all instructions use allowed actions.","ref":"Jido.Instruction.html#validate_allowed_actions/2"},{"type":"function","title":"Parameters - Jido.Instruction.validate_allowed_actions/2","doc":"* `instructions` - List of instruction structs\n  * `allowed_actions` - List of allowed action modules","ref":"Jido.Instruction.html#validate_allowed_actions/2-parameters"},{"type":"function","title":"Returns - Jido.Instruction.validate_allowed_actions/2","doc":"* `:ok` - All actions are allowed\n  * `{:error, term()}` - If any action is not allowed","ref":"Jido.Instruction.html#validate_allowed_actions/2-returns"},{"type":"function","title":"Examples - Jido.Instruction.validate_allowed_actions/2","doc":"iex> instructions = [%Instruction{action: MyAction}, %Instruction{action: OtherAction}]\n    iex> Instruction.validate_allowed_actions(instructions, [MyAction])\n    {:error, \"Actions not allowed: OtherAction\"}\n\n    iex> instructions = [%Instruction{action: MyAction}]\n    iex> Instruction.validate_allowed_actions(instructions, [MyAction])\n    :ok","ref":"Jido.Instruction.html#validate_allowed_actions/2-examples"},{"type":"behaviour","title":"Jido.Sensor","doc":"Defines the behavior and implementation for Sensors in the Jido system.\n\nA Sensor is a GenServer that emits Signals using a Jido.Bus based on specific events and retains a configurable number of last values.","ref":"Jido.Sensor.html"},{"type":"behaviour","title":"Usage - Jido.Sensor","doc":"To define a new Sensor, use the `Jido.Sensor` behavior in your module:\n\n    defmodule MySensor do\n      use Jido.Sensor,\n        name: \"my_sensor\",\n        description: \"Monitors a specific metric\",\n        category: :monitoring,\n        tags: [:example, :demo],\n        vsn: \"1.0.0\",\n        schema: [\n          metric: [type: :string, required: true]\n        ]\n\n      @impl true\n      def generate_signal(state) do\n        # Your sensor logic here\n        {:ok, Jido.Signal.new(%{\n          source: \"#{state.sensor.name}:#{state.id}\",\n          topic: \"metric_update\",\n          payload: %{value: get_metric_value()},\n          timestamp: DateTime.utc_now()\n        })}\n      end\n    end","ref":"Jido.Sensor.html#module-usage"},{"type":"behaviour","title":"Callbacks - Jido.Sensor","doc":"Implementing modules can override the following callbacks:\n\n- `c:mount/1`: Called when the sensor is initialized.\n- `c:deliver_signal/1`: Generates a signal based on the current state.\n- `c:on_before_deliver/2`: Called before a signal is delivered.\n- `c:shutdown/1`: Called when the sensor is shutting down.","ref":"Jido.Sensor.html#module-callbacks"},{"type":"callback","title":"Jido.Sensor.deliver_signal/1","doc":"","ref":"Jido.Sensor.html#c:deliver_signal/1"},{"type":"callback","title":"Jido.Sensor.mount/1","doc":"","ref":"Jido.Sensor.html#c:mount/1"},{"type":"callback","title":"Jido.Sensor.on_before_deliver/2","doc":"","ref":"Jido.Sensor.html#c:on_before_deliver/2"},{"type":"type","title":"Jido.Sensor.options/0","doc":"","ref":"Jido.Sensor.html#t:options/0"},{"type":"callback","title":"Jido.Sensor.shutdown/1","doc":"","ref":"Jido.Sensor.html#c:shutdown/1"},{"type":"type","title":"Jido.Sensor.t/0","doc":"","ref":"Jido.Sensor.html#t:t/0"},{"type":"module","title":"Jido.Workflow","doc":"Workflow provides a robust set of methods for executing Actions (`Jido.Action`).\n\nThis module offers functionality to:\n- Run actions synchronously or asynchronously\n- Manage timeouts and retries\n- Cancel running actions\n- Normalize and validate input parameters and context\n- Emit telemetry events for monitoring and debugging\n\nWorkflows are defined as modules (Actions) that implement specific callbacks, allowing for\na standardized way of defining and executing complex workflows across a distributed system.","ref":"Jido.Workflow.html"},{"type":"module","title":"Features - Jido.Workflow","doc":"- Synchronous and asynchronous workflow execution\n- Automatic retries with exponential backoff\n- Timeout handling for long-running workflows\n- Parameter and context normalization\n- Comprehensive error handling and reporting\n- Telemetry integration for monitoring and tracing\n- Cancellation of running workflows","ref":"Jido.Workflow.html#module-features"},{"type":"module","title":"Usage - Jido.Workflow","doc":"Workflows are executed using the `run/4` or `run_async/4` functions:\n\n    Jido.Workflow.run(MyAction, %{param1: \"value\"}, %{context_key: \"context_value\"})\n\nSee `Jido.Action` for how to define an Action.\n\nFor asynchronous execution:\n\n    async_ref = Jido.Workflow.run_async(MyAction, params, context)\n    # ... do other work ...\n    result = Jido.Workflow.await(async_ref)","ref":"Jido.Workflow.html#module-usage"},{"type":"type","title":"Jido.Workflow.action/0","doc":"","ref":"Jido.Workflow.html#t:action/0"},{"type":"type","title":"Jido.Workflow.async_ref/0","doc":"","ref":"Jido.Workflow.html#t:async_ref/0"},{"type":"function","title":"Jido.Workflow.await/2","doc":"Waits for the result of an asynchronous Action execution.","ref":"Jido.Workflow.html#await/2"},{"type":"function","title":"Parameters - Jido.Workflow.await/2","doc":"- `async_ref`: The reference returned by `run_async/4`.\n- `timeout`: Maximum time (in ms) to wait for the result (default: 5000).","ref":"Jido.Workflow.html#await/2-parameters"},{"type":"function","title":"Returns - Jido.Workflow.await/2","doc":"- `{:ok, result}` if the Action executes successfully.\n- `{:error, reason}` if an error occurs during execution or if the workflow times out.","ref":"Jido.Workflow.html#await/2-returns"},{"type":"function","title":"Examples - Jido.Workflow.await/2","doc":"iex> async_ref = Jido.Workflow.run_async(MyAction, %{input: \"value\"})\n    iex> Jido.Workflow.await(async_ref, 10_000)\n    {:ok, %{result: \"processed value\"}}\n\n    iex> async_ref = Jido.Workflow.run_async(SlowAction, %{input: \"value\"})\n    iex> Jido.Workflow.await(async_ref, 100)\n    {:error, %Jido.Error{type: :timeout, message: \"Async workflow timed out after 100ms\"}}","ref":"Jido.Workflow.html#await/2-examples"},{"type":"function","title":"Jido.Workflow.cancel/1","doc":"Cancels a running asynchronous Action execution.","ref":"Jido.Workflow.html#cancel/1"},{"type":"function","title":"Parameters - Jido.Workflow.cancel/1","doc":"- `async_ref`: The reference returned by `run_async/4`, or just the PID of the process to cancel.","ref":"Jido.Workflow.html#cancel/1-parameters"},{"type":"function","title":"Returns - Jido.Workflow.cancel/1","doc":"- `:ok` if the cancellation was successful.\n- `{:error, reason}` if the cancellation failed or the input was invalid.","ref":"Jido.Workflow.html#cancel/1-returns"},{"type":"function","title":"Examples - Jido.Workflow.cancel/1","doc":"iex> async_ref = Jido.Workflow.run_async(LongRunningAction, %{input: \"value\"})\n    iex> Jido.Workflow.cancel(async_ref)\n    :ok\n\n    iex> Jido.Workflow.cancel(\"invalid\")\n    {:error, %Jido.Error{type: :invalid_async_ref, message: \"Invalid async ref for cancellation\"}}","ref":"Jido.Workflow.html#cancel/1-examples"},{"type":"type","title":"Jido.Workflow.context/0","doc":"","ref":"Jido.Workflow.html#t:context/0"},{"type":"macro","title":"Jido.Workflow.dbug/2","doc":"","ref":"Jido.Workflow.html#dbug/2"},{"type":"macro","title":"Jido.Workflow.error/2","doc":"","ref":"Jido.Workflow.html#error/2"},{"type":"type","title":"Jido.Workflow.params/0","doc":"","ref":"Jido.Workflow.html#t:params/0"},{"type":"function","title":"Jido.Workflow.run/4","doc":"Executes a Action synchronously with the given parameters and context.","ref":"Jido.Workflow.html#run/4"},{"type":"function","title":"Parameters - Jido.Workflow.run/4","doc":"- `action`: The module implementing the Action behavior.\n- `params`: A map of input parameters for the Action.\n- `context`: A map providing additional context for the Action execution.\n- `opts`: Options controlling the execution:\n  - `:timeout` - Maximum time (in ms) allowed for the Action to complete (default: 5000).\n  - `:max_retries` - Maximum number of retry attempts (default: 1).\n  - `:backoff` - Initial backoff time in milliseconds, doubles with each retry (default: 250).","ref":"Jido.Workflow.html#run/4-parameters"},{"type":"function","title":"Returns - Jido.Workflow.run/4","doc":"- `{:ok, result}` if the Action executes successfully.\n- `{:error, reason}` if an error occurs during execution.","ref":"Jido.Workflow.html#run/4-returns"},{"type":"function","title":"Examples - Jido.Workflow.run/4","doc":"iex> Jido.Workflow.run(MyAction, %{input: \"value\"}, %{user_id: 123})\n    {:ok, %{result: \"processed value\"}}\n\n    iex> Jido.Workflow.run(MyAction, %{invalid: \"input\"}, %{}, timeout: 1000)\n    {:error, %Jido.Error{type: :validation_error, message: \"Invalid input\"}}","ref":"Jido.Workflow.html#run/4-examples"},{"type":"function","title":"Jido.Workflow.run_async/4","doc":"Executes a Action asynchronously with the given parameters and context.\n\nThis function immediately returns a reference that can be used to await the result\nor cancel the workflow.\n\n**Note**: This approach integrates with OTP by spawning tasks under a `Task.Supervisor`.\nMake sure `{Task.Supervisor, name: Jido.Workflow.TaskSupervisor}` is part of your supervision tree.","ref":"Jido.Workflow.html#run_async/4"},{"type":"function","title":"Parameters - Jido.Workflow.run_async/4","doc":"- `action`: The module implementing the Action behavior.\n- `params`: A map of input parameters for the Action.\n- `context`: A map providing additional context for the Action execution.\n- `opts`: Options controlling the execution (same as `run/4`).","ref":"Jido.Workflow.html#run_async/4-parameters"},{"type":"function","title":"Returns - Jido.Workflow.run_async/4","doc":"An `async_ref` map containing:\n- `:ref` - A unique reference for this async workflow.\n- `:pid` - The PID of the process executing the Action.","ref":"Jido.Workflow.html#run_async/4-returns"},{"type":"function","title":"Examples - Jido.Workflow.run_async/4","doc":"iex> async_ref = Jido.Workflow.run_async(MyAction, %{input: \"value\"}, %{user_id: 123})\n    %{ref: #Reference<0.1234.5678>, pid: #PID<0.234.0>}\n\n    iex> result = Jido.Workflow.await(async_ref)\n    {:ok, %{result: \"processed value\"}}","ref":"Jido.Workflow.html#run_async/4-examples"},{"type":"type","title":"Jido.Workflow.run_opts/0","doc":"","ref":"Jido.Workflow.html#t:run_opts/0"},{"type":"behaviour","title":"Jido.Runner","doc":"Behavior for executing planned actions on an Agent.\n\nJido provides two built-in runners:\n\n- `Jido.Runner.Simple` - Executes a single instruction from the agent's queue,\n  handling state updates and directives atomically.\n\n- `Jido.Runner.Chain` - Executes multiple instructions sequentially, with results\n  flowing between steps and support for directive-based flow control.","ref":"Jido.Runner.html"},{"type":"type","title":"Jido.Runner.action/0","doc":"","ref":"Jido.Runner.html#t:action/0"},{"type":"callback","title":"Jido.Runner.run/2","doc":"","ref":"Jido.Runner.html#c:run/2"},{"type":"module","title":"Jido.Runner.Chain","doc":"A runner that executes instructions sequentially with support for result chaining\nand directive-based interruption.","ref":"Jido.Runner.Chain.html"},{"type":"module","title":"Chain Execution - Jido.Runner.Chain","doc":"Instructions are executed in sequence with the output of each instruction\nbecoming the input for the next instruction in the chain. This enables\ndata flow between instructions while maintaining state consistency.","ref":"Jido.Runner.Chain.html#module-chain-execution"},{"type":"module","title":"Directive Handling - Jido.Runner.Chain","doc":"The chain supports directive-based flow control:\n* Directives are accumulated and added to the final queue\n* State changes and directives can be mixed in the chain\n* Directives do not interrupt chain execution","ref":"Jido.Runner.Chain.html#module-directive-handling"},{"type":"module","title":"State Management - Jido.Runner.Chain","doc":"* Initial state flows through the chain\n* Each instruction can modify or extend the state\n* Final state reflects accumulated changes\n* Directives are added to the final queue","ref":"Jido.Runner.Chain.html#module-state-management"},{"type":"type","title":"Jido.Runner.Chain.chain_opts/0","doc":"","ref":"Jido.Runner.Chain.html#t:chain_opts/0"},{"type":"type","title":"Jido.Runner.Chain.chain_result/0","doc":"","ref":"Jido.Runner.Chain.html#t:chain_result/0"},{"type":"function","title":"Jido.Runner.Chain.run/2","doc":"Executes a chain of instructions, handling directives and state transitions.","ref":"Jido.Runner.Chain.html#run/2"},{"type":"function","title":"Execution Flow - Jido.Runner.Chain.run/2","doc":"1. Instructions are executed in sequence\n2. Results from each instruction feed into the next\n3. Directives are accumulated throughout the chain\n4. At the end of the chain:\n   - Agent directives are applied to the agent\n   - Server directives are returned with the result","ref":"Jido.Runner.Chain.html#run/2-execution-flow"},{"type":"function","title":"Parameters - Jido.Runner.Chain.run/2","doc":"- agent: The agent struct containing pending instructions\n  - opts: Optional keyword list of execution options:\n    - :merge_results - boolean, merges map results into subsequent instruction params (default: true)","ref":"Jido.Runner.Chain.html#run/2-parameters"},{"type":"function","title":"Returns - Jido.Runner.Chain.run/2","doc":"- `{:ok, updated_agent, server_directives}` - Chain completed successfully\n    - Updated state map\n    - Queue containing any directives from execution\n    - List of server directives\n  - `{:error, error}` - Chain execution failed\n    - Contains error details","ref":"Jido.Runner.Chain.html#run/2-returns"},{"type":"function","title":"Examples - Jido.Runner.Chain.run/2","doc":"# Basic chain execution\n    {:ok, updated_agent, server_directives} = Chain.run(agent)\n\n    # Handle execution error\n    {:error, error} = Chain.run(agent_with_failing_instruction)","ref":"Jido.Runner.Chain.html#run/2-examples"},{"type":"module","title":"Jido.Runner.Simple","doc":"A simple runner that executes a single instruction from an Agent's instruction queue.","ref":"Jido.Runner.Simple.html"},{"type":"module","title":"Overview - Jido.Runner.Simple","doc":"The Simple Runner follows a sequential execution model:\n1. Dequeues a single instruction from the agent's pending queue\n2. Executes the instruction via its action module\n3. Processes the result (either a state update, directive or both)\n4. Applies state changes if configured\n5. Returns the updated agent with the execution results and server directives","ref":"Jido.Runner.Simple.html#module-overview"},{"type":"module","title":"Features - Jido.Runner.Simple","doc":"* Single instruction execution\n* Support for directives and state results\n* Atomic execution guarantees\n* Comprehensive error handling\n* Debug logging at key execution points\n* Optional state application","ref":"Jido.Runner.Simple.html#module-features"},{"type":"module","title":"Error Handling - Jido.Runner.Simple","doc":"* Invalid instructions are rejected\n* Action execution failures return error results\n* Queue empty condition handled gracefully\n* All errors preserve the original agent state","ref":"Jido.Runner.Simple.html#module-error-handling"},{"type":"function","title":"Jido.Runner.Simple.run/2","doc":"Executes a single instruction from the Agent's pending instructions queue.","ref":"Jido.Runner.Simple.html#run/2"},{"type":"function","title":"Execution Process - Jido.Runner.Simple.run/2","doc":"1. Dequeues the oldest instruction from the agent's queue\n2. Creates a new Result struct to track execution\n3. Executes the instruction through its action module\n4. Processes any directives from the execution\n5. Optionally applies state changes\n6. Returns the updated agent with execution results and server directives","ref":"Jido.Runner.Simple.html#run/2-execution-process"},{"type":"function","title":"Parameters - Jido.Runner.Simple.run/2","doc":"* `agent` - The agent struct containing:\n    * `pending_instructions` - Queue of pending instructions\n    * `state` - Current agent state\n    * `id` - Agent identifier\n  * `opts` - Optional keyword list of execution options:\n    * `apply_state` - Whether to apply state changes (default: true)","ref":"Jido.Runner.Simple.html#run/2-parameters"},{"type":"function","title":"Returns - Jido.Runner.Simple.run/2","doc":"* `{:ok, updated_agent, directives}` - Successful execution with:\n    * Updated state map (for state results)\n    * Updated pending instructions queue\n    * Any server directives from the execution\n  * `{:error, reason}` - Execution failed with:\n    * String error for queue empty condition\n    * Error struct with details for execution failures","ref":"Jido.Runner.Simple.html#run/2-returns"},{"type":"function","title":"Examples - Jido.Runner.Simple.run/2","doc":"# Successful state update\n    {:ok, updated_agent, directives} = Runner.Simple.run(agent_with_state_update)\n\n    # Execute without applying state\n    {:ok, updated_agent, directives} = Runner.Simple.run(agent_with_state_update, apply_state: false)\n\n    # Empty queue - returns agent unchanged\n    {:ok, agent, []} = Runner.Simple.run(agent_with_empty_queue)\n\n    # Execution error\n    {:error, error} = Runner.Simple.run(agent_with_failing_action)","ref":"Jido.Runner.Simple.html#run/2-examples"},{"type":"function","title":"Error Handling - Jido.Runner.Simple.run/2","doc":"* Returns `{:error, \"No pending instructions\"}` for empty queue\n  * Returns `{:error, error}` with error details for execution failures\n  * All errors preserve the original agent state\n  * Failed executions do not affect the remaining queue","ref":"Jido.Runner.Simple.html#run/2-error-handling"},{"type":"function","title":"Logging - Jido.Runner.Simple.run/2","doc":"Debug logs are emitted at key points:\n  * Runner start with agent ID\n  * Instruction dequeue result\n  * Execution setup and workflow invocation\n  * Result processing and categorization","ref":"Jido.Runner.Simple.html#run/2-logging"},{"type":"type","title":"Jido.Runner.Simple.run_opts/0","doc":"","ref":"Jido.Runner.Simple.html#t:run_opts/0"},{"type":"type","title":"Jido.Runner.Simple.run_result/0","doc":"","ref":"Jido.Runner.Simple.html#t:run_result/0"},{"type":"module","title":"Jido.Workflow.Chain","doc":"Provides functionality to chain multiple Jido Workflows together with interruption support.\n\nThis module allows for sequential execution of workflows, where the output\nof one workflow becomes the input for the next workflow in the chain.\nExecution can be interrupted between workflows using an interruption check function.","ref":"Jido.Workflow.Chain.html"},{"type":"module","title":"Examples - Jido.Workflow.Chain","doc":"iex> interrupt_check = fn -> System.monotonic_time(:millisecond) > @deadline end\n    iex> Jido.Workflow.Chain.chain([AddOne, MultiplyByTwo], %{value: 5}, interrupt_check: interrupt_check)\n    {:ok, %{value: 12}}\n\n    # When interrupted:\n    iex> Jido.Workflow.Chain.chain([AddOne, MultiplyByTwo], %{value: 5}, interrupt_check: fn -> true end)\n    {:interrupted, %{value: 6}}","ref":"Jido.Workflow.Chain.html#module-examples"},{"type":"function","title":"Jido.Workflow.Chain.chain/3","doc":"Executes a chain of workflows sequentially with optional interruption support.","ref":"Jido.Workflow.Chain.html#chain/3"},{"type":"function","title":"Parameters - Jido.Workflow.Chain.chain/3","doc":"- `workflows`: A list of workflows to be executed in order. Each workflow\n  can be a module (the workflow module) or a tuple of `{workflow_module, options}`.\n- `initial_params`: A map of initial parameters to be passed to the first workflow.\n- `opts`: Additional options for the chain execution.","ref":"Jido.Workflow.Chain.html#chain/3-parameters"},{"type":"function","title":"Options - Jido.Workflow.Chain.chain/3","doc":"- `:async` - When set to `true`, the chain will be executed asynchronously (default: `false`).\n- `:context` - A map of context data to be passed to each workflow.\n- `:interrupt_check` - A function that returns boolean, called between workflows to check if chain should be interrupted.","ref":"Jido.Workflow.Chain.html#chain/3-options"},{"type":"function","title":"Returns - Jido.Workflow.Chain.chain/3","doc":"- `{:ok, result}` where `result` is the final output of the chain.\n- `{:error, error}` if any workflow in the chain fails.\n- `{:interrupted, result}` if the chain was interrupted, containing the last successful result.\n- `Task.t()` if the `:async` option is set to `true`.","ref":"Jido.Workflow.Chain.html#chain/3-returns"},{"type":"type","title":"Jido.Workflow.Chain.chain_result/0","doc":"","ref":"Jido.Workflow.Chain.html#t:chain_result/0"},{"type":"type","title":"Jido.Workflow.Chain.chain_workflow/0","doc":"","ref":"Jido.Workflow.Chain.html#t:chain_workflow/0"},{"type":"type","title":"Jido.Workflow.Chain.interrupt_check/0","doc":"","ref":"Jido.Workflow.Chain.html#t:interrupt_check/0"},{"type":"module","title":"Jido.Workflow.Closure","doc":"Provides functionality to create closures around Jido Workflows (Actions).\n\nThis module allows for partial application of context and options to actions,\ncreating reusable workflow closures that can be executed with different parameters.","ref":"Jido.Workflow.Closure.html"},{"type":"type","title":"Jido.Workflow.Closure.action/0","doc":"","ref":"Jido.Workflow.Closure.html#t:action/0"},{"type":"function","title":"Jido.Workflow.Closure.async_closure/3","doc":"Creates an async closure around a action with pre-applied context and options.","ref":"Jido.Workflow.Closure.html#async_closure/3"},{"type":"function","title":"Parameters - Jido.Workflow.Closure.async_closure/3","doc":"- `action`: The action module to create an async closure for.\n- `context`: The context to be applied to the action (default: %{}).\n- `opts`: The options to be applied to the action execution (default: []).","ref":"Jido.Workflow.Closure.html#async_closure/3-parameters"},{"type":"function","title":"Returns - Jido.Workflow.Closure.async_closure/3","doc":"A function that takes params and returns an async reference.","ref":"Jido.Workflow.Closure.html#async_closure/3-returns"},{"type":"function","title":"Examples - Jido.Workflow.Closure.async_closure/3","doc":"iex> async_closure = Jido.Workflow.Closure.async_closure(MyAction, %{user_id: 123}, [timeout: 10_000])\n    iex> async_ref = async_closure.(%{input: \"test\"})\n    iex> Jido.Workflow.await(async_ref)\n    {:ok, %{result: \"processed test\"}}","ref":"Jido.Workflow.Closure.html#async_closure/3-examples"},{"type":"type","title":"Jido.Workflow.Closure.closure/0","doc":"","ref":"Jido.Workflow.Closure.html#t:closure/0"},{"type":"function","title":"Jido.Workflow.Closure.closure/3","doc":"Creates a closure around a action with pre-applied context and options.","ref":"Jido.Workflow.Closure.html#closure/3"},{"type":"function","title":"Parameters - Jido.Workflow.Closure.closure/3","doc":"- `action`: The action module to create a closure for.\n- `context`: The context to be applied to the action (default: %{}).\n- `opts`: The options to be applied to the action execution (default: []).","ref":"Jido.Workflow.Closure.html#closure/3-parameters"},{"type":"function","title":"Returns - Jido.Workflow.Closure.closure/3","doc":"A function that takes params and returns the result of running the action.","ref":"Jido.Workflow.Closure.html#closure/3-returns"},{"type":"function","title":"Examples - Jido.Workflow.Closure.closure/3","doc":"iex> closure = Jido.Workflow.Closure.closure(MyAction, %{user_id: 123}, [timeout: 10_000])\n    iex> closure.(%{input: \"test\"})\n    {:ok, %{result: \"processed test\"}}","ref":"Jido.Workflow.Closure.html#closure/3-examples"},{"type":"type","title":"Jido.Workflow.Closure.context/0","doc":"","ref":"Jido.Workflow.Closure.html#t:context/0"},{"type":"type","title":"Jido.Workflow.Closure.params/0","doc":"","ref":"Jido.Workflow.Closure.html#t:params/0"},{"type":"type","title":"Jido.Workflow.Closure.run_opts/0","doc":"","ref":"Jido.Workflow.Closure.html#t:run_opts/0"},{"type":"module","title":"Jido.Actions.Directives","doc":"A collection of actions that are used to control the behavior of the agent.\n\nThis module provides a set of simple, reusable actions:\n- EnqueueAction: Enqueues another action based on params\n- RegisterAction: Registers a new action module\n- DeregisterAction: Deregisters an existing action module\n\nEach action is implemented as a separate submodule and follows the Jido.Action behavior.","ref":"Jido.Actions.Directives.html"},{"type":"module","title":"Jido.Agent.Directive","doc":"Provides a type-safe way to modify agent state through discrete, validated directives.","ref":"Jido.Agent.Directive.html"},{"type":"module","title":"Overview - Jido.Agent.Directive","doc":"Directives are immutable instructions that can be applied to an agent to modify its state\n  in predefined ways. Each directive type is implemented as a separate struct with its own\n  validation rules, helping ensure type safety and consistent state transitions.","ref":"Jido.Agent.Directive.html#module-overview"},{"type":"module","title":"Available Directives - Jido.Agent.Directive","doc":"* `Enqueue` - Adds a new instruction to the agent's pending queue\n      - Requires an action atom\n      - Supports optional params and context maps\n      - Supports optional opts keyword list\n      - Example: `%Enqueue{action: :move, params: %{location: :kitchen}}`\n\n  * `RegisterAction` - Registers a new action module with the agent\n      - Requires a valid module atom\n      - Example: `%RegisterAction{action_module: MyApp.Actions.Move}`\n\n  * `DeregisterAction` - Removes an action module from the agent\n      - Requires a valid module atom\n      - Example: `%DeregisterAction{action_module: MyApp.Actions.Move}`\n\n  * `Spawn` - Spawns a child process under the agent's supervisor\n      - Requires a module atom and arguments\n      - Example: `%Spawn{module: MyWorker, args: [id: 1]}`\n\n  * `Kill` - Terminates a child process\n      - Requires a valid PID\n      - Example: `%Kill{pid: #PID<0.123.0>}`","ref":"Jido.Agent.Directive.html#module-available-directives"},{"type":"module","title":"Usage - Jido.Agent.Directive","doc":"Directives can be applied to either an Agent or ServerState struct:\n\n      # Apply to Agent - returns updated agent\n      {:ok, updated_agent} = Directive.apply_directives(agent, directives)\n\n      # Apply to ServerState - returns updated state\n      {:ok, updated_state} = Directive.apply_directives(server_state, directives)\n\n  Each function validates directives before applying them and returns either:\n  * `{:ok, updated_state}` - Directives were successfully applied\n  * `{:error, reason}` - Failed to apply directives","ref":"Jido.Agent.Directive.html#module-usage"},{"type":"module","title":"Validation - Jido.Agent.Directive","doc":"Each directive type has its own validation rules:\n\n  * `Enqueue` requires a non-nil atom for the action\n  * `RegisterAction` requires a valid module atom\n  * `DeregisterAction` requires a valid module atom\n  * `Spawn` requires a valid module atom and arguments\n  * `Kill` requires a valid PID\n\n  Failed validation results in an error tuple being returned and processing being halted.","ref":"Jido.Agent.Directive.html#module-validation"},{"type":"module","title":"Error Handling - Jido.Agent.Directive","doc":"The module uses tagged tuples for error handling:\n\n  * `{:ok, updated_state}` - Successful application of directives\n  * `{:error, reason}` - Failed validation or application\n\n  Common error reasons include:\n\n  * `:invalid_action` - The action specified in an `Enqueue` is invalid\n  * `:invalid_action_module` - The module specified in a `Register/DeregisterAction` is invalid\n  * `:invalid_module` - The module specified in a `Spawn` is invalid\n  * `:invalid_pid` - The PID specified in a `Kill` is invalid\n  * `:invalid_topic` - The topic specified in a broadcast/subscribe/unsubscribe directive is invalid","ref":"Jido.Agent.Directive.html#module-error-handling"},{"type":"module","title":"Ideas - Jido.Agent.Directive","doc":"Change Mode\n  Change Verbosity\n  Manage Router (add/remove/etc)\n  Manage Skills (add/remove/etc)\n  Manage Dispatchers (add/remove/etc)","ref":"Jido.Agent.Directive.html#module-ideas"},{"type":"function","title":"Jido.Agent.Directive.agent_directive?/1","doc":"","ref":"Jido.Agent.Directive.html#agent_directive?/1"},{"type":"function","title":"Jido.Agent.Directive.apply_agent_directive/3","doc":"Applies agent directives to an Agent struct.","ref":"Jido.Agent.Directive.html#apply_agent_directive/3"},{"type":"function","title":"Parameters - Jido.Agent.Directive.apply_agent_directive/3","doc":"- agent: The Agent struct to modify\n  - directives: A list of directives to apply\n  - opts: Optional keyword list of options (default: [])","ref":"Jido.Agent.Directive.html#apply_agent_directive/3-parameters"},{"type":"function","title":"Returns - Jido.Agent.Directive.apply_agent_directive/3","doc":"- `{:ok, updated_agent, unapplied_directives}` - Successfully applied agent directives\n  - `{:error, reason}` - Failed to apply directives","ref":"Jido.Agent.Directive.html#apply_agent_directive/3-returns"},{"type":"function","title":"Jido.Agent.Directive.apply_server_directive/3","doc":"Applies only server directives to a ServerState struct.","ref":"Jido.Agent.Directive.html#apply_server_directive/3"},{"type":"function","title":"Parameters - Jido.Agent.Directive.apply_server_directive/3","doc":"- state: The ServerState struct to modify\n  - directives: A list of directives to apply\n  - opts: Optional keyword list of options (default: [])","ref":"Jido.Agent.Directive.html#apply_server_directive/3-parameters"},{"type":"function","title":"Returns - Jido.Agent.Directive.apply_server_directive/3","doc":"- `{:ok, updated_state, unapplied_directives}` - Successfully applied server directives\n  - `{:error, reason}` - Failed to apply directives","ref":"Jido.Agent.Directive.html#apply_server_directive/3-returns"},{"type":"type","title":"Jido.Agent.Directive.directive_result/0","doc":"","ref":"Jido.Agent.Directive.html#t:directive_result/0"},{"type":"function","title":"Jido.Agent.Directive.split_directives/1","doc":"","ref":"Jido.Agent.Directive.html#split_directives/1"},{"type":"type","title":"Jido.Agent.Directive.t/0","doc":"","ref":"Jido.Agent.Directive.html#t:t/0"},{"type":"module","title":"Jido.Agent.Directive.DeregisterAction","doc":"Directive to deregister an existing action module","ref":"Jido.Agent.Directive.DeregisterAction.html"},{"type":"type","title":"Jido.Agent.Directive.DeregisterAction.t/0","doc":"","ref":"Jido.Agent.Directive.DeregisterAction.html#t:t/0"},{"type":"module","title":"Jido.Agent.Directive.Enqueue","doc":"Directive to enqueue a new instruction","ref":"Jido.Agent.Directive.Enqueue.html"},{"type":"function","title":"Jido.Agent.Directive.Enqueue.new/1","doc":"Creates a new Enqueue directive from an Instruction struct.","ref":"Jido.Agent.Directive.Enqueue.html#new/1"},{"type":"function","title":"Parameters - Jido.Agent.Directive.Enqueue.new/1","doc":"* `instruction` - %Instruction{} struct to convert to directive","ref":"Jido.Agent.Directive.Enqueue.html#new/1-parameters"},{"type":"function","title":"Returns - Jido.Agent.Directive.Enqueue.new/1","doc":"* `%Enqueue{}` - New enqueue directive with instruction fields","ref":"Jido.Agent.Directive.Enqueue.html#new/1-returns"},{"type":"type","title":"Jido.Agent.Directive.Enqueue.t/0","doc":"","ref":"Jido.Agent.Directive.Enqueue.html#t:t/0"},{"type":"module","title":"Jido.Agent.Directive.Kill","doc":"Directive to terminate a child process","ref":"Jido.Agent.Directive.Kill.html"},{"type":"type","title":"Jido.Agent.Directive.Kill.t/0","doc":"","ref":"Jido.Agent.Directive.Kill.html#t:t/0"},{"type":"module","title":"Jido.Agent.Directive.RegisterAction","doc":"Directive to register a new action module","ref":"Jido.Agent.Directive.RegisterAction.html"},{"type":"type","title":"Jido.Agent.Directive.RegisterAction.t/0","doc":"","ref":"Jido.Agent.Directive.RegisterAction.html#t:t/0"},{"type":"module","title":"Jido.Agent.Directive.Spawn","doc":"Directive to spawn a child process","ref":"Jido.Agent.Directive.Spawn.html"},{"type":"type","title":"Jido.Agent.Directive.Spawn.t/0","doc":"","ref":"Jido.Agent.Directive.Spawn.html#t:t/0"},{"type":"module","title":"Jido.Action.Tool","doc":"Provides functionality to convert Jido Workflows into tool representations.\n\nThis module allows Jido Workflows to be easily integrated with AI systems\nlike LangChain or Instructor by converting them into a standardized tool format.","ref":"Jido.Action.Tool.html"},{"type":"function","title":"Jido.Action.Tool.build_parameters_schema/1","doc":"Builds a parameters schema for the tool based on the workflow's schema.","ref":"Jido.Action.Tool.html#build_parameters_schema/1"},{"type":"function","title":"Arguments - Jido.Action.Tool.build_parameters_schema/1","doc":"* `schema` - The NimbleOptions schema from the workflow.","ref":"Jido.Action.Tool.html#build_parameters_schema/1-arguments"},{"type":"function","title":"Returns - Jido.Action.Tool.build_parameters_schema/1","doc":"A map representing the parameters schema in a format compatible with LangChain.","ref":"Jido.Action.Tool.html#build_parameters_schema/1-returns"},{"type":"function","title":"Jido.Action.Tool.execute_workflow/3","doc":"Executes an workflow and formats the result for tool output.\n\nThis function is typically used as the function value in the tool representation.","ref":"Jido.Action.Tool.html#execute_workflow/3"},{"type":"function","title":"Jido.Action.Tool.nimble_type_to_json_schema_type/1","doc":"Converts a NimbleOptions type to a JSON Schema type.","ref":"Jido.Action.Tool.html#nimble_type_to_json_schema_type/1"},{"type":"function","title":"Arguments - Jido.Action.Tool.nimble_type_to_json_schema_type/1","doc":"* `type` - The NimbleOptions type.","ref":"Jido.Action.Tool.html#nimble_type_to_json_schema_type/1-arguments"},{"type":"function","title":"Returns - Jido.Action.Tool.nimble_type_to_json_schema_type/1","doc":"A string representing the equivalent JSON Schema type.","ref":"Jido.Action.Tool.html#nimble_type_to_json_schema_type/1-returns"},{"type":"function","title":"Jido.Action.Tool.parameter_to_json_schema/1","doc":"Converts a NimbleOptions parameter definition to a JSON Schema representation.","ref":"Jido.Action.Tool.html#parameter_to_json_schema/1"},{"type":"function","title":"Arguments - Jido.Action.Tool.parameter_to_json_schema/1","doc":"* `opts` - The options for a single parameter from the NimbleOptions schema.","ref":"Jido.Action.Tool.html#parameter_to_json_schema/1-arguments"},{"type":"function","title":"Returns - Jido.Action.Tool.parameter_to_json_schema/1","doc":"A map representing the parameter in JSON Schema format.","ref":"Jido.Action.Tool.html#parameter_to_json_schema/1-returns"},{"type":"function","title":"Jido.Action.Tool.to_tool/1","doc":"Converts a Jido Workflow into a tool representation.","ref":"Jido.Action.Tool.html#to_tool/1"},{"type":"function","title":"Arguments - Jido.Action.Tool.to_tool/1","doc":"* `workflow` - The module implementing the Jido.Action behavior.","ref":"Jido.Action.Tool.html#to_tool/1-arguments"},{"type":"function","title":"Returns - Jido.Action.Tool.to_tool/1","doc":"A map representing the workflow as a tool, compatible with systems like LangChain.","ref":"Jido.Action.Tool.html#to_tool/1-returns"},{"type":"function","title":"Examples - Jido.Action.Tool.to_tool/1","doc":"iex> tool = Jido.Action.Tool.to_tool(MyWorkflow)\n    %{\n      name: \"my_workflow\",\n      description: \"Performs a specific task\",\n      function: #Function<...>,\n      parameters_schema: %{...}\n    }","ref":"Jido.Action.Tool.html#to_tool/1-examples"},{"type":"type","title":"Jido.Action.Tool.tool/0","doc":"","ref":"Jido.Action.Tool.html#t:tool/0"},{"type":"module","title":"Jido.Signal","doc":"Defines the core Signal structure in Jido, implementing the CloudEvents specification (v1.0.2)\nwith Jido-specific extensions for agent-based systems.\n\nhttps://cloudevents.io/","ref":"Jido.Signal.html"},{"type":"module","title":"Overview - Jido.Signal","doc":"Signals are the universal message format in Jido, serving as the nervous system of your\nagent-based application. Every event, command, and state change flows through the system\nas a Signal, providing:\n\n- Standardized event structure (CloudEvents v1.0.2 compatible)\n- Rich metadata and context tracking\n- Built-in instruction handling\n- Flexible dispatch configuration\n- Automatic serialization","ref":"Jido.Signal.html#module-overview"},{"type":"module","title":"CloudEvents Compliance - Jido.Signal","doc":"Each Signal implements the CloudEvents v1.0.2 specification with these required fields:\n\n- `specversion`: Always \"1.0.2\"\n- `id`: Unique identifier (UUID v4)\n- `source`: Origin of the event (\"/service/component\")\n- `type`: Classification of the event (\"domain.entity.action\")\n\nAnd optional fields:\n\n- `subject`: Specific subject of the event\n- `time`: Timestamp in ISO 8601 format\n- `datacontenttype`: Media type of the data (defaults to \"application/json\")\n- `dataschema`: Schema defining the data structure\n- `data`: The actual event payload","ref":"Jido.Signal.html#module-cloudevents-compliance"},{"type":"module","title":"Jido Extensions - Jido.Signal","doc":"Beyond the CloudEvents spec, Signals include Jido-specific fields:\n\n- `jido_instructions`: List of instructions to execute (optional)\n- `jido_opts`: Processing options and flags (optional)\n- `jido_dispatch`: Routing and delivery configuration (optional)\n- `jido_metadata`: Additional context and tracking data (optional)","ref":"Jido.Signal.html#module-jido-extensions"},{"type":"module","title":"Creating Signals - Jido.Signal","doc":"Signals can be created in several ways:\n\n```elixir\n# Basic event\n{:ok, signal} = Signal.new(%{\n  type: \"user.created\",\n  source: \"/auth/registration\",\n  data: %{user_id: \"123\", email: \"user@example.com\"}\n})\n\n# With instructions\n{:ok, signal} = Signal.new(%{\n  type: \"order.process\",\n  source: \"/orders\",\n  data: %{order_id: \"456\"},\n  jido_instructions: [\n    ProcessOrder,\n    {NotifyUser, %{template: \"order_confirmation\"}}\n  ]\n})\n\n# With dispatch config\n{:ok, signal} = Signal.new(%{\n  type: \"metrics.collected\",\n  source: \"/monitoring\",\n  data: %{cpu: 80, memory: 70},\n  jido_dispatch: {:pubsub, topic: \"metrics\"}\n})\n```","ref":"Jido.Signal.html#module-creating-signals"},{"type":"module","title":"Signal Types - Jido.Signal","doc":"Signal types are strings, but typically use a hierarchical dot notation:\n\n```\n . . [. ]\n```\n\nExamples:\n- `user.profile.updated`\n- `order.payment.processed.success`\n- `system.metrics.collected`\n\nGuidelines for type naming:\n- Use lowercase with dots\n- Keep segments meaningful\n- Order from general to specific\n- Include qualifiers when needed","ref":"Jido.Signal.html#module-signal-types"},{"type":"module","title":"Data Content Types - Jido.Signal","doc":"The `datacontenttype` field indicates the format of the `data` field:\n\n- `application/json` (default) - JSON-structured data\n- `text/plain` - Unstructured text\n- `application/octet-stream` - Binary data\n- Custom MIME types for specific formats","ref":"Jido.Signal.html#module-data-content-types"},{"type":"module","title":"Instruction Handling - Jido.Signal","doc":"Signals can carry instructions for agents to execute:\n\n```elixir\nSignal.new(%{\n  type: \"task.assigned\",\n  source: \"/workflow\",\n  jido_instructions: [\n    ValidateTask,\n    {AssignTask, %{worker: \"agent_1\"}},\n    NotifyAssignment\n  ]\n})\n```\n\nInstructions are normalized and validated during Signal creation.","ref":"Jido.Signal.html#module-instruction-handling"},{"type":"module","title":"Dispatch Configuration - Jido.Signal","doc":"The `jido_dispatch` field controls how the Signal is delivered:\n\n```elixir\n# Single dispatch config\njido_dispatch: {:pubsub, topic: \"events\"}\n\n# Multiple dispatch targets\njido_dispatch: [\n  {:pubsub, topic: \"events\"},\n  {:logger, level: :info},\n  {:webhook, url: \"https://api.example.com/webhook\"}\n]\n```","ref":"Jido.Signal.html#module-dispatch-configuration"},{"type":"module","title":"See Also - Jido.Signal","doc":"- `Jido.Instruction` - Instruction specification\n- `Jido.Signal.Router` - Signal routing\n- `Jido.Signal.Dispatch` - Dispatch handling\n- CloudEvents spec: https://cloudevents.io/","ref":"Jido.Signal.html#module-see-also"},{"type":"function","title":"Jido.Signal.from_map/1","doc":"Creates a new Signal struct from a map.","ref":"Jido.Signal.html#from_map/1"},{"type":"function","title":"Parameters - Jido.Signal.from_map/1","doc":"- `map`: A map containing the Signal attributes.","ref":"Jido.Signal.html#from_map/1-parameters"},{"type":"function","title":"Returns - Jido.Signal.from_map/1","doc":"`{:ok, Signal.t()}` if the map is valid, `{:error, String.t()}` otherwise.","ref":"Jido.Signal.html#from_map/1-returns"},{"type":"function","title":"Examples - Jido.Signal.from_map/1","doc":"iex> Jido.Signal.from_map(%{\"type\" => \"example.event\", \"source\" => \"/example\", \"id\" => \"123\"})\n    {:ok, %Jido.Signal{type: \"example.event\", source: \"/example\", id: \"123\", ...}}","ref":"Jido.Signal.html#from_map/1-examples"},{"type":"function","title":"Jido.Signal.map_to_signal_data/2","doc":"","ref":"Jido.Signal.html#map_to_signal_data/2"},{"type":"function","title":"Jido.Signal.new/1","doc":"Creates a new Signal struct.","ref":"Jido.Signal.html#new/1"},{"type":"function","title":"Parameters - Jido.Signal.new/1","doc":"- `attrs`: A map or keyword list containing the Signal attributes.","ref":"Jido.Signal.html#new/1-parameters"},{"type":"function","title":"Returns - Jido.Signal.new/1","doc":"`{:ok, Signal.t()}` if the attributes are valid, `{:error, String.t()}` otherwise.","ref":"Jido.Signal.html#new/1-returns"},{"type":"function","title":"Examples - Jido.Signal.new/1","doc":"iex> Jido.Signal.new(%{type: \"example.event\", source: \"/example\", id: \"123\"})\n    {:ok, %Jido.Signal{type: \"example.event\", source: \"/example\", id: \"123\", ...}}\n\n    iex> Jido.Signal.new(type: \"example.event\", source: \"/example\")\n    {:ok, %Jido.Signal{type: \"example.event\", source: \"/example\", ...}}","ref":"Jido.Signal.html#new/1-examples"},{"type":"function","title":"Jido.Signal.new!/1","doc":"Creates a new Signal struct, raising an error if invalid.","ref":"Jido.Signal.html#new!/1"},{"type":"function","title":"Parameters - Jido.Signal.new!/1","doc":"- `attrs`: A map or keyword list containing the Signal attributes.","ref":"Jido.Signal.html#new!/1-parameters"},{"type":"function","title":"Returns - Jido.Signal.new!/1","doc":"`Signal.t()` if the attributes are valid.","ref":"Jido.Signal.html#new!/1-returns"},{"type":"function","title":"Raises - Jido.Signal.new!/1","doc":"`RuntimeError` if the attributes are invalid.","ref":"Jido.Signal.html#new!/1-raises"},{"type":"function","title":"Examples - Jido.Signal.new!/1","doc":"iex> Jido.Signal.new!(%{type: \"example.event\", source: \"/example\"})\n    %Jido.Signal{type: \"example.event\", source: \"/example\", ...}\n\n    iex> Jido.Signal.new!(type: \"example.event\", source: \"/example\")\n    %Jido.Signal{type: \"example.event\", source: \"/example\", ...}","ref":"Jido.Signal.html#new!/1-examples"},{"type":"type","title":"Jido.Signal.t/0","doc":"","ref":"Jido.Signal.html#t:t/0"},{"type":"module","title":"Jido.Signal.Router","doc":"The Router module implements a high-performance, trie-based signal routing system designed specifically for agent-based architectures. It provides sophisticated message routing capabilities with support for exact matches, wildcards, and pattern matching functions.","ref":"Jido.Signal.Router.html"},{"type":"module","title":"Core Concepts - Jido.Signal.Router","doc":"The Router organizes signal handlers into an efficient trie (prefix tree) structure that enables:\n- Fast path-based routing using dot-notation (e.g., \"user.created.verified\")\n- Priority-based handler execution (-100 to 100)\n- Complexity-based ordering for wildcard resolution\n- Dynamic route management (add/remove at runtime)\n- Pattern matching through custom functions","ref":"Jido.Signal.Router.html#module-core-concepts"},{"type":"module","title":"Path Patterns - Jido.Signal.Router","doc":"Routes use a dot-notation pattern system supporting:\n- Exact matches: `\"user.created\"`\n- Single wildcards: `\"user.*.updated\"` (matches one segment)\n- Multi-level wildcards: `\"audit.**\"` (matches zero or more segments)\n\nPattern rules:\n- Paths must match: `^[a-zA-Z0-9.*_-]+(.[a-zA-Z0-9.*_-]+)*$`\n- Cannot contain consecutive dots (..)\n- Cannot contain consecutive multi-wildcards (`**...**`)","ref":"Jido.Signal.Router.html#module-path-patterns"},{"type":"module","title":"Handler Priority - Jido.Signal.Router","doc":"Handlers execute in order based on:\n1. Path complexity (more specific paths execute first)\n2. Priority (-100 to 100, higher executes first)\n3. Registration order (for equal priority/complexity)","ref":"Jido.Signal.Router.html#module-handler-priority"},{"type":"module","title":"Usage Examples - Jido.Signal.Router","doc":"Basic route creation:\n```elixir\n{:ok, router} = Router.new([\n  # Simple route with default priority\n  {\"user.created\", %Instruction{action: HandleUserCreated}},\n\n  # High-priority audit logging\n  {\"audit.**\", %Instruction{action: AuditLogger}, 100},\n\n  # Pattern matching for large payments\n  {\"payment.processed\",\n    fn signal -> signal.data.amount > 1000 end,\n    %Instruction{action: HandleLargePayment}}\n])\n```\n\nDynamic route management:\n```elixir\n# Add routes\n{:ok, router} = Router.add(router, [\n  {\"metrics.**\", %Instruction{action: CollectMetrics}}\n])\n\n# Remove routes\n{:ok, router} = Router.remove(router, \"metrics.**\")\n```\n\nSignal routing:\n```elixir\n{:ok, instructions} = Router.route(router, %Signal{\n  type: \"payment.processed\",\n  data: %{amount: 2000}\n})\n```","ref":"Jido.Signal.Router.html#module-usage-examples"},{"type":"module","title":"Path Complexity Scoring - Jido.Signal.Router","doc":"The router uses a sophisticated scoring system to determine handler execution order:\n\n1. Base score from segment count (length * 2000)\n2. Exact match bonuses (3000 per segment, weighted by position)\n3. Wildcard penalties:\n   - Single wildcard (*): 1000 - position_index * 100\n   - Multi-wildcard (**): 2000 - position_index * 200\n\nThis ensures more specific routes take precedence over wildcards, while maintaining predictable execution order.","ref":"Jido.Signal.Router.html#module-path-complexity-scoring"},{"type":"module","title":"Best Practices - Jido.Signal.Router","doc":"1. Route Design\n   - Use consistent, hierarchical path patterns\n   - Prefer exact matches over wildcards when possible\n   - Keep path segments meaningful and well-structured\n   - Document your path hierarchy\n\n2. Priority Management\n   - Reserve high priorities (75-100) for critical handlers\n   - Use default priority (0) for standard business logic\n   - Reserve low priorities (-100 to -75) for metrics/logging\n   - Document priority ranges for your application\n\n3. Pattern Matching\n   - Keep match functions simple and fast\n   - Handle nil/missing data gracefully\n   - Avoid side effects in match functions\n   - Test edge cases thoroughly\n\n4. Performance Considerations\n   - Monitor route count in production\n   - Use pattern matching sparingly\n   - Consider complexity scores when designing paths\n   - Profile routing performance under load","ref":"Jido.Signal.Router.html#module-best-practices"},{"type":"module","title":"Error Handling - Jido.Signal.Router","doc":"The router provides detailed error feedback for:\n- Invalid path patterns\n- Priority out of bounds\n- Invalid match functions\n- Missing handlers\n- Malformed signals","ref":"Jido.Signal.Router.html#module-error-handling"},{"type":"module","title":"Implementation Details - Jido.Signal.Router","doc":"The router uses several specialized structs:\n- `Route` - Defines a single routing rule\n- `TrieNode` - Internal trie structure node\n- `HandlerInfo` - Stores handler metadata\n- `PatternMatch` - Encapsulates pattern matching rules\n\nSee the corresponding typespecs for detailed field information.","ref":"Jido.Signal.Router.html#module-implementation-details"},{"type":"module","title":"See Also - Jido.Signal.Router","doc":"- `Jido.Signal` - Signal structure and validation\n- `Jido.Instruction` - Handler instruction format\n- `Jido.Error` - Error types and handling","ref":"Jido.Signal.Router.html#module-see-also"},{"type":"function","title":"Jido.Signal.Router.add/2","doc":"Adds one or more routes to the router.","ref":"Jido.Signal.Router.html#add/2"},{"type":"function","title":"Parameters - Jido.Signal.Router.add/2","doc":"- router: The existing router struct\n- routes: A route specification or list of route specifications in one of these formats:\n  - %Route{}\n  - {path, instruction}\n  - {path, instruction, priority}\n  - {path, [match: match_fn], instruction}\n  - {path, [match: match_fn], instruction, priority}","ref":"Jido.Signal.Router.html#add/2-parameters"},{"type":"function","title":"Returns - Jido.Signal.Router.add/2","doc":"`{:ok, updated_router}` or `{:error, reason}`","ref":"Jido.Signal.Router.html#add/2-returns"},{"type":"function","title":"Jido.Signal.Router.list/1","doc":"Lists all routes currently registered in the router.\n\nReturns a list of Route structs containing the path, instruction, priority and match function\nfor each registered route.","ref":"Jido.Signal.Router.html#list/1"},{"type":"function","title":"Returns - Jido.Signal.Router.list/1","doc":"`{:ok, [%Route{}]}` - List of Route structs","ref":"Jido.Signal.Router.html#list/1-returns"},{"type":"function","title":"Examples - Jido.Signal.Router.list/1","doc":"{:ok, routes} = Router.list(router)\n\n    # Returns:\n    [\n      %Route{\n        path: \"user.created\",\n        instruction: %Instruction{action: MyApp.Actions.HandleUserCreated},\n        priority: 0,\n        match: nil\n      },\n      %Route{\n        path: \"payment.processed\",\n        instruction: %Instruction{action: MyApp.Actions.HandleLargePayment},\n        priority: 90,\n        match: #Function<1.123456789/1>\n      }\n    ]","ref":"Jido.Signal.Router.html#list/1-examples"},{"type":"type","title":"Jido.Signal.Router.match/0","doc":"","ref":"Jido.Signal.Router.html#t:match/0"},{"type":"function","title":"Jido.Signal.Router.merge/2","doc":"Merges two routers by combining their routes.\n\nTakes a target router and a list of routes from another router (obtained via `list/1`) and\nmerges them together, preserving priorities and match functions.","ref":"Jido.Signal.Router.html#merge/2"},{"type":"function","title":"Parameters - Jido.Signal.Router.merge/2","doc":"- router: The target Router struct to merge into\n- routes: List of Route structs to merge in (from Router.list/1)","ref":"Jido.Signal.Router.html#merge/2-parameters"},{"type":"function","title":"Returns - Jido.Signal.Router.merge/2","doc":"`{:ok, merged_router}` or `{:error, reason}`","ref":"Jido.Signal.Router.html#merge/2-returns"},{"type":"function","title":"Examples - Jido.Signal.Router.merge/2","doc":"{:ok, router1} = Router.new([{\"user.created\", instruction1}])\n    {:ok, router2} = Router.new([{\"payment.processed\", instruction2}])\n    {:ok, routes2} = Router.list(router2)\n\n    # Merge router2's routes into router1\n    {:ok, merged} = Router.merge(router1, routes2)","ref":"Jido.Signal.Router.html#merge/2-examples"},{"type":"function","title":"Jido.Signal.Router.new/1","doc":"Creates a new router with the given routes.","ref":"Jido.Signal.Router.html#new/1"},{"type":"function","title":"Jido.Signal.Router.new!/1","doc":"Creates a new router with the given routes, raising on error.","ref":"Jido.Signal.Router.html#new!/1"},{"type":"function","title":"Jido.Signal.Router.normalize/1","doc":"Normalizes route specifications into Route structs.","ref":"Jido.Signal.Router.html#normalize/1"},{"type":"function","title":"Parameters - Jido.Signal.Router.normalize/1","doc":"* `input` - One of:\n    * Single Route struct\n    * List of Route structs\n    * List of route_spec tuples\n    * {path, instruction} tuple\n    * {path, instruction, priority} tuple\n    * {path, match_fn, instruction} tuple\n    * {path, match_fn, instruction, priority} tuple","ref":"Jido.Signal.Router.html#normalize/1-parameters"},{"type":"function","title":"Returns - Jido.Signal.Router.normalize/1","doc":"* `{:ok, [%Route{}]}` - List of normalized Route structs\n  * `{:error, term()}` - If normalization fails","ref":"Jido.Signal.Router.html#normalize/1-returns"},{"type":"type","title":"Jido.Signal.Router.priority/0","doc":"","ref":"Jido.Signal.Router.html#t:priority/0"},{"type":"function","title":"Jido.Signal.Router.remove/2","doc":"Removes one or more routes from the router.","ref":"Jido.Signal.Router.html#remove/2"},{"type":"function","title":"Parameters - Jido.Signal.Router.remove/2","doc":"- router: The existing router struct\n- paths: A path string or list of path strings to remove","ref":"Jido.Signal.Router.html#remove/2-parameters"},{"type":"function","title":"Returns - Jido.Signal.Router.remove/2","doc":"`{:ok, updated_router}` or `{:error, reason}`","ref":"Jido.Signal.Router.html#remove/2-returns"},{"type":"function","title":"Examples - Jido.Signal.Router.remove/2","doc":"# Remove a single route\n    {:ok, router} = Router.remove(router, \"metrics.**\")\n\n    # Remove multiple routes\n    {:ok, router} = Router.remove(router, [\"audit.*\", \"user.created\"])","ref":"Jido.Signal.Router.html#remove/2-examples"},{"type":"function","title":"Jido.Signal.Router.route/2","doc":"Routes a signal through the router to find and execute matching handlers.","ref":"Jido.Signal.Router.html#route/2"},{"type":"function","title":"Parameters - Jido.Signal.Router.route/2","doc":"- router: The router struct to use for routing\n- signal: The signal to route","ref":"Jido.Signal.Router.html#route/2-parameters"},{"type":"function","title":"Returns - Jido.Signal.Router.route/2","doc":"- {:ok, [instruction]} - List of matching instructions, may be empty if no matches\n- {:error, term()} - Other errors that occurred during routing","ref":"Jido.Signal.Router.html#route/2-returns"},{"type":"function","title":"Examples - Jido.Signal.Router.route/2","doc":"{:ok, results} = Router.route(router, %Signal{\n      type: \"payment.processed\",\n      data: %{amount: 100}\n    })","ref":"Jido.Signal.Router.html#route/2-examples"},{"type":"type","title":"Jido.Signal.Router.route_spec/0","doc":"","ref":"Jido.Signal.Router.html#t:route_spec/0"},{"type":"function","title":"Jido.Signal.Router.validate/1","doc":"Validates one or more Route structs.","ref":"Jido.Signal.Router.html#validate/1"},{"type":"function","title":"Parameters - Jido.Signal.Router.validate/1","doc":"- routes: A %Route{} struct or list of %Route{} structs to validate","ref":"Jido.Signal.Router.html#validate/1-parameters"},{"type":"function","title":"Returns - Jido.Signal.Router.validate/1","doc":"- {:ok, %Route{}} - Single validated Route struct\n- {:ok, [%Route{}]} - List of validated Route structs\n- {:error, term()} - If validation fails","ref":"Jido.Signal.Router.html#validate/1-returns"},{"type":"type","title":"Jido.Signal.Router.wildcard_type/0","doc":"","ref":"Jido.Signal.Router.html#t:wildcard_type/0"},{"type":"module","title":"Jido.Bus","doc":"Use the signal store configured for a Commanded application.","ref":"Jido.Bus.html"},{"type":"module","title":"Telemetry Signals - Jido.Bus","doc":"Adds telemetry signals for the following functions. Signals are emitted in the form\n\n`[:jido, :bus, signal]` with their spannable postfixes (`start`, `stop`, `exception`)\n\n  * ack/3\n  * adapter/2\n  * publish/4\n  * delete_snapshot/2\n  * unsubscribe/3\n  * read_snapshot/2\n  * record_snapshot/2\n  * replay/2\n  * replay/3\n  * replay/4\n  * subscribe/2\n  * subscribe_persistent/5\n  * subscribe_persistent/6\n  * unsubscribe/2","ref":"Jido.Bus.html#module-telemetry-signals"},{"type":"function","title":"Jido.Bus.ack/3","doc":"Acknowledge receipt and successful processing of a signal.","ref":"Jido.Bus.html#ack/3"},{"type":"function","title":"Jido.Bus.child_spec/1","doc":"Start a bus as part of a supervision tree.","ref":"Jido.Bus.html#child_spec/1"},{"type":"function","title":"Options - Jido.Bus.child_spec/1","doc":"* `:name` - Required. The name to register the bus process under\n  * `:adapter` - The adapter type. Either `:pubsub`, `:in_memory`, or a custom module\n  * Other options are passed to the chosen adapter","ref":"Jido.Bus.html#child_spec/1-options"},{"type":"function","title":"Examples - Jido.Bus.child_spec/1","doc":"children = [\n      {Jido.Bus, name: :jido_agent_123, adapter: :pubsub, pubsub_name: MyApp.PubSub}\n    ]","ref":"Jido.Bus.html#child_spec/1-examples"},{"type":"function","title":"Jido.Bus.delete_snapshot/2","doc":"Delete a previously recorded snapshot for a given source","ref":"Jido.Bus.html#delete_snapshot/2"},{"type":"function","title":"Jido.Bus.publish/5","doc":"Append one or more signals to a stream atomically.","ref":"Jido.Bus.html#publish/5"},{"type":"function","title":"Jido.Bus.read_snapshot/2","doc":"Read a snapshot, if available, for a given source.","ref":"Jido.Bus.html#read_snapshot/2"},{"type":"function","title":"Jido.Bus.record_snapshot/2","doc":"Record a snapshot of the data and metadata for a given source","ref":"Jido.Bus.html#record_snapshot/2"},{"type":"function","title":"Jido.Bus.replay/4","doc":"Streams signals from the given stream, in the order in which they were originally written.","ref":"Jido.Bus.html#replay/4"},{"type":"function","title":"Jido.Bus.start_link/1","doc":"Start a bus process.\n\nSee `child_spec/1` for options.","ref":"Jido.Bus.html#start_link/1"},{"type":"type","title":"Jido.Bus.start_option/0","doc":"","ref":"Jido.Bus.html#t:start_option/0"},{"type":"function","title":"Jido.Bus.subscribe/2","doc":"Create a transient subscription to a single signal stream.","ref":"Jido.Bus.html#subscribe/2"},{"type":"function","title":"Jido.Bus.subscribe_persistent/6","doc":"Create a persistent subscription to an signal stream.","ref":"Jido.Bus.html#subscribe_persistent/6"},{"type":"type","title":"Jido.Bus.t/0","doc":"","ref":"Jido.Bus.html#t:t/0"},{"type":"function","title":"Jido.Bus.unsubscribe/2","doc":"Unsubscribe an existing subscriber from signal notifications.","ref":"Jido.Bus.html#unsubscribe/2"},{"type":"function","title":"Jido.Bus.unsubscribe/3","doc":"Delete an existing subscription.","ref":"Jido.Bus.html#unsubscribe/3"},{"type":"function","title":"Jido.Bus.via_tuple/2","doc":"Returns a via tuple for addressing the bus process.","ref":"Jido.Bus.html#via_tuple/2"},{"type":"function","title":"Options - Jido.Bus.via_tuple/2","doc":"* `:registry` - The registry to use (defaults to Jido.BusRegistry)","ref":"Jido.Bus.html#via_tuple/2-options"},{"type":"function","title":"Examples - Jido.Bus.via_tuple/2","doc":"iex> Jido.Bus.via_tuple(:my_bus)\n    {:via, Registry, {Jido.BusRegistry, :my_bus}}\n\n    iex> Jido.Bus.via_tuple(:my_bus, registry: MyApp.Registry)\n    {:via, Registry, {MyApp.Registry, :my_bus}}","ref":"Jido.Bus.html#via_tuple/2-examples"},{"type":"function","title":"Jido.Bus.whereis/2","doc":"Gets the PID of a running bus by name.\n\nReturns `{:ok, pid}` if found, `{:error, :not_found}` otherwise.","ref":"Jido.Bus.html#whereis/2"},{"type":"behaviour","title":"Jido.Bus.Adapter","doc":"Defines the behaviour to be implemented by an signal store adapter to be used by Commanded.","ref":"Jido.Bus.Adapter.html"},{"type":"callback","title":"Jido.Bus.Adapter.ack/3","doc":"Acknowledge receipt and successful processing of the given signal received from\na subscription to an signal stream.","ref":"Jido.Bus.Adapter.html#c:ack/3"},{"type":"type","title":"Jido.Bus.Adapter.application/0","doc":"","ref":"Jido.Bus.Adapter.html#t:application/0"},{"type":"type","title":"Jido.Bus.Adapter.bus/0","doc":"","ref":"Jido.Bus.Adapter.html#t:bus/0"},{"type":"callback","title":"Jido.Bus.Adapter.child_spec/2","doc":"Return a child spec defining all processes required by the signal store.","ref":"Jido.Bus.Adapter.html#c:child_spec/2"},{"type":"type","title":"Jido.Bus.Adapter.config/0","doc":"","ref":"Jido.Bus.Adapter.html#t:config/0"},{"type":"callback","title":"Jido.Bus.Adapter.delete_snapshot/2","doc":"Delete a previously recorded snapshot for a given source","ref":"Jido.Bus.Adapter.html#c:delete_snapshot/2"},{"type":"type","title":"Jido.Bus.Adapter.error/0","doc":"","ref":"Jido.Bus.Adapter.html#t:error/0"},{"type":"type","title":"Jido.Bus.Adapter.expected_version/0","doc":"","ref":"Jido.Bus.Adapter.html#t:expected_version/0"},{"type":"callback","title":"Jido.Bus.Adapter.publish/5","doc":"Append one or more signals to a stream atomically.","ref":"Jido.Bus.Adapter.html#c:publish/5"},{"type":"callback","title":"Jido.Bus.Adapter.read_snapshot/2","doc":"Read a snapshot, if available, for a given source.","ref":"Jido.Bus.Adapter.html#c:read_snapshot/2"},{"type":"callback","title":"Jido.Bus.Adapter.record_snapshot/2","doc":"Record a snapshot of the data and metadata for a given source","ref":"Jido.Bus.Adapter.html#c:record_snapshot/2"},{"type":"callback","title":"Jido.Bus.Adapter.replay/4","doc":"Streams signals from the given stream, in the order in which they were\noriginally written.","ref":"Jido.Bus.Adapter.html#c:replay/4"},{"type":"type","title":"Jido.Bus.Adapter.source_id/0","doc":"","ref":"Jido.Bus.Adapter.html#t:source_id/0"},{"type":"type","title":"Jido.Bus.Adapter.start_from/0","doc":"","ref":"Jido.Bus.Adapter.html#t:start_from/0"},{"type":"type","title":"Jido.Bus.Adapter.stream_id/0","doc":"","ref":"Jido.Bus.Adapter.html#t:stream_id/0"},{"type":"callback","title":"Jido.Bus.Adapter.subscribe/2","doc":"Create a transient subscription to a single signal stream.\n\nThe signal store will publish any signals appended to the given stream to the\n`subscriber` process as an `{:signals, signals}` message.\n\nThe subscriber does not need to acknowledge receipt of the signals.","ref":"Jido.Bus.Adapter.html#c:subscribe/2"},{"type":"callback","title":"Jido.Bus.Adapter.subscribe_persistent/6","doc":"Create a persistent subscription to an signal stream.","ref":"Jido.Bus.Adapter.html#c:subscribe_persistent/6"},{"type":"type","title":"Jido.Bus.Adapter.subscriber/0","doc":"","ref":"Jido.Bus.Adapter.html#t:subscriber/0"},{"type":"type","title":"Jido.Bus.Adapter.subscription/0","doc":"","ref":"Jido.Bus.Adapter.html#t:subscription/0"},{"type":"type","title":"Jido.Bus.Adapter.subscription_name/0","doc":"","ref":"Jido.Bus.Adapter.html#t:subscription_name/0"},{"type":"callback","title":"Jido.Bus.Adapter.unsubscribe/2","doc":"Unsubscribe an existing subscriber from signal notifications.\n\nThis should not delete the subscription.","ref":"Jido.Bus.Adapter.html#c:unsubscribe/2"},{"type":"callback","title":"Jido.Bus.Adapter.unsubscribe/3","doc":"Delete an existing subscription.","ref":"Jido.Bus.Adapter.html#c:unsubscribe/3"},{"type":"module","title":"Jido.Bus.Adapters.InMemory","doc":"An in-memory signal store adapter implemented as a `GenServer` process which\nstores signals in memory only.\n\nThis is only designed for testing purposes.","ref":"Jido.Bus.Adapters.InMemory.html"},{"type":"function","title":"Jido.Bus.Adapters.InMemory.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Jido.Bus.Adapters.InMemory.html#child_spec/1"},{"type":"function","title":"Jido.Bus.Adapters.InMemory.reset!/2","doc":"","ref":"Jido.Bus.Adapters.InMemory.html#reset!/2"},{"type":"function","title":"Jido.Bus.Adapters.InMemory.start_link/1","doc":"","ref":"Jido.Bus.Adapters.InMemory.html#start_link/1"},{"type":"module","title":"Jido.Bus.Adapters.PubSub","doc":"A Jido Bus adapter implementation using Phoenix.PubSub for simple pub/sub messaging.\n\nThis adapter provides basic publish/subscribe functionality without persistence or replay capabilities.\nIt is suitable for scenarios where you need lightweight message distribution without message history.","ref":"Jido.Bus.Adapters.PubSub.html"},{"type":"module","title":"Features - Jido.Bus.Adapters.PubSub","doc":"- Simple publish/subscribe messaging using Phoenix.PubSub\n- Support for stream-based subscriptions\n- Auto-cleanup of subscriptions when subscribers terminate\n- No message persistence or replay capabilities","ref":"Jido.Bus.Adapters.PubSub.html#module-features"},{"type":"module","title":"Configuration - Jido.Bus.Adapters.PubSub","doc":"The adapter requires a Phoenix.PubSub server to be started. This is handled automatically\nwhen starting the adapter through `child_spec/2`.","ref":"Jido.Bus.Adapters.PubSub.html#module-configuration"},{"type":"module","title":"Usage - Jido.Bus.Adapters.PubSub","doc":"```elixir\n# Start the bus with the PubSub adapter\n{:ok, children, bus} = Jido.Bus.Adapters.PubSub.child_spec(MyApp, [])\nSupervisor.start_link(children, strategy: :one_for_one)\n\n# Subscribe to a stream\n:ok = Jido.Bus.subscribe(bus, \"my-stream\")\n\n# Publish signals\n:ok = Jido.Bus.publish(bus, \"my-stream\", :any_version, [signal], [])\n\n# Receive signals\nreceive do\n  {:signal, signal} -> handle_signal(signal)\nend\n```","ref":"Jido.Bus.Adapters.PubSub.html#module-usage"},{"type":"module","title":"Limitations - Jido.Bus.Adapters.PubSub","doc":"This adapter does not support:\n- Message persistence\n- Stream replay\n- Persistent subscriptions\n- Snapshots\n- Message acknowledgments","ref":"Jido.Bus.Adapters.PubSub.html#module-limitations"},{"type":"function","title":"Jido.Bus.Adapters.PubSub.ack/3","doc":"Not implemented - this adapter does not support message acknowledgments.","ref":"Jido.Bus.Adapters.PubSub.html#ack/3"},{"type":"function","title":"Jido.Bus.Adapters.PubSub.child_spec/2","doc":"Returns the child specification for starting the PubSub adapter.","ref":"Jido.Bus.Adapters.PubSub.html#child_spec/2"},{"type":"function","title":"Options - Jido.Bus.Adapters.PubSub.child_spec/2","doc":"* `:name` - The name to use for the PubSub server. Defaults to the provided context.","ref":"Jido.Bus.Adapters.PubSub.html#child_spec/2-options"},{"type":"function","title":"Returns - Jido.Bus.Adapters.PubSub.child_spec/2","doc":"* `{:ok, children, bus}` - A tuple containing the child specs to start and the configured bus.","ref":"Jido.Bus.Adapters.PubSub.html#child_spec/2-returns"},{"type":"function","title":"Jido.Bus.Adapters.PubSub.delete_snapshot/2","doc":"Not implemented - this adapter does not support snapshots.","ref":"Jido.Bus.Adapters.PubSub.html#delete_snapshot/2"},{"type":"function","title":"Jido.Bus.Adapters.PubSub.publish/5","doc":"Publishes signals to a stream.\n\nThe expected_version parameter is ignored as this adapter does not maintain stream versions.","ref":"Jido.Bus.Adapters.PubSub.html#publish/5"},{"type":"function","title":"Jido.Bus.Adapters.PubSub.read_snapshot/2","doc":"Not implemented - this adapter does not support snapshots.","ref":"Jido.Bus.Adapters.PubSub.html#read_snapshot/2"},{"type":"function","title":"Jido.Bus.Adapters.PubSub.record_snapshot/2","doc":"Not implemented - this adapter does not support snapshots.","ref":"Jido.Bus.Adapters.PubSub.html#record_snapshot/2"},{"type":"function","title":"Jido.Bus.Adapters.PubSub.replay/4","doc":"Not implemented - this adapter does not support stream replay.","ref":"Jido.Bus.Adapters.PubSub.html#replay/4"},{"type":"function","title":"Jido.Bus.Adapters.PubSub.subscribe/2","doc":"Subscribes the current process to a stream.","ref":"Jido.Bus.Adapters.PubSub.html#subscribe/2"},{"type":"function","title":"Jido.Bus.Adapters.PubSub.subscribe_persistent/6","doc":"Not implemented - this adapter does not support persistent subscriptions.","ref":"Jido.Bus.Adapters.PubSub.html#subscribe_persistent/6"},{"type":"function","title":"Jido.Bus.Adapters.PubSub.unsubscribe/2","doc":"Unsubscribes from a stream or subscription.\n\nWhen given a PID, returns :ok since PubSub handles cleanup automatically.\nWhen given a stream_id, unsubscribes the current process from that stream.","ref":"Jido.Bus.Adapters.PubSub.html#unsubscribe/2"},{"type":"function","title":"Jido.Bus.Adapters.PubSub.unsubscribe/3","doc":"Not implemented - this adapter does not support snapshots.","ref":"Jido.Bus.Adapters.PubSub.html#unsubscribe/3"},{"type":"module","title":"Jido.Signal.Dispatch","doc":"A flexible signal dispatching system that routes signals to various destinations using configurable adapters.\n\nThe Dispatch module serves as the central hub for signal delivery in the Jido system. It provides a unified\ninterface for sending signals to different destinations through various adapters. Each adapter implements\nspecific delivery mechanisms suited for different use cases.","ref":"Jido.Signal.Dispatch.html"},{"type":"module","title":"Built-in Adapters - Jido.Signal.Dispatch","doc":"The following adapters are provided out of the box:\n\n* `:pid` - Direct delivery to a specific process (see `Jido.Signal.Dispatch.PidAdapter`)\n* `:bus` - Delivery to an event bus (see `Jido.Signal.Dispatch.Bus`)\n* `:named` - Delivery to a named process (see `Jido.Signal.Dispatch.Named`)\n* `:pubsub` - Delivery via PubSub mechanism (see `Jido.Signal.Dispatch.PubSub`)\n* `:logger` - Log signals using Logger (see `Jido.Signal.Dispatch.LoggerAdapter`)\n* `:console` - Print signals to console (see `Jido.Signal.Dispatch.ConsoleAdapter`)\n* `:noop` - No-op adapter for testing/development (see `Jido.Signal.Dispatch.NoopAdapter`)","ref":"Jido.Signal.Dispatch.html#module-built-in-adapters"},{"type":"module","title":"Configuration - Jido.Signal.Dispatch","doc":"Each adapter requires specific configuration options. A dispatch configuration is a tuple of\n`{adapter_type, options}` where:\n\n* `adapter_type` - One of the built-in adapter types above or a custom module implementing the `Jido.Signal.Dispatch.Adapter` behaviour\n* `options` - Keyword list of options specific to the chosen adapter\n\nMultiple dispatch configurations can be provided as a list to send signals to multiple destinations.","ref":"Jido.Signal.Dispatch.html#module-configuration"},{"type":"module","title":"Examples - Jido.Signal.Dispatch","doc":"# Send to a specific PID\n    config = {:pid, [target: {:pid, destination_pid}, delivery_mode: :async]}\n    Jido.Signal.Dispatch.dispatch(signal, config)\n\n    # Send to multiple destinations\n    config = [\n      {:bus, [target: {:bus, :default}, stream: \"events\"]},\n      {:logger, [level: :info]},\n      {:pubsub, [target: :audit, topic: \"audit\"]}\n    ]\n    Jido.Signal.Dispatch.dispatch(signal, config)\n\n    # Using a custom adapter\n    config = {MyCustomAdapter, [custom_option: \"value\"]}\n    Jido.Signal.Dispatch.dispatch(signal, config)","ref":"Jido.Signal.Dispatch.html#module-examples"},{"type":"module","title":"Custom Adapters - Jido.Signal.Dispatch","doc":"To implement a custom adapter, create a module that implements the `Jido.Signal.Dispatch.Adapter`\nbehaviour. The module must implement:\n\n* `validate_opts/1` - Validates the adapter-specific options\n* `deliver/2` - Handles the actual signal delivery\n\nSee `Jido.Signal.Dispatch.Adapter` for more details.","ref":"Jido.Signal.Dispatch.html#module-custom-adapters"},{"type":"type","title":"Jido.Signal.Dispatch.adapter/0","doc":"","ref":"Jido.Signal.Dispatch.html#t:adapter/0"},{"type":"function","title":"Jido.Signal.Dispatch.dispatch/2","doc":"Dispatches a signal using the provided configuration.","ref":"Jido.Signal.Dispatch.html#dispatch/2"},{"type":"function","title":"Parameters - Jido.Signal.Dispatch.dispatch/2","doc":"- `signal` - The signal to dispatch\n- `config` - Either a single dispatch configuration tuple or a list of configurations","ref":"Jido.Signal.Dispatch.html#dispatch/2-parameters"},{"type":"function","title":"Examples - Jido.Signal.Dispatch.dispatch/2","doc":"# Single destination\n    iex> config = {:pid, [target: {:pid, pid}, delivery_mode: :async]}\n    iex> Jido.Signal.Dispatch.dispatch(signal, config)\n    :ok\n\n    # Multiple destinations\n    iex> config = [\n    ...>   {:bus, [target: {:bus, :default}, stream: \"events\"]},\n    ...>   {:pubsub, [target: :audit, topic: \"audit\"]}\n    ...> ]\n    iex> Jido.Signal.Dispatch.dispatch(signal, config)\n    :ok","ref":"Jido.Signal.Dispatch.html#dispatch/2-examples"},{"type":"type","title":"Jido.Signal.Dispatch.dispatch_config/0","doc":"","ref":"Jido.Signal.Dispatch.html#t:dispatch_config/0"},{"type":"type","title":"Jido.Signal.Dispatch.dispatch_configs/0","doc":"","ref":"Jido.Signal.Dispatch.html#t:dispatch_configs/0"},{"type":"function","title":"Jido.Signal.Dispatch.validate_opts/1","doc":"Validates a dispatch configuration.","ref":"Jido.Signal.Dispatch.html#validate_opts/1"},{"type":"function","title":"Parameters - Jido.Signal.Dispatch.validate_opts/1","doc":"- `config` - Either a single dispatch configuration tuple or a list of dispatch configurations","ref":"Jido.Signal.Dispatch.html#validate_opts/1-parameters"},{"type":"function","title":"Returns - Jido.Signal.Dispatch.validate_opts/1","doc":"- `{:ok, config}` if the configuration is valid\n- `{:error, reason}` if the configuration is invalid","ref":"Jido.Signal.Dispatch.html#validate_opts/1-returns"},{"type":"function","title":"Examples - Jido.Signal.Dispatch.validate_opts/1","doc":"# Single config\n    iex> config = {:pid, [target: {:pid, self()}, delivery_mode: :async]}\n    iex> Jido.Signal.Dispatch.validate_opts(config)\n    {:ok, ^config}\n\n    # Multiple configs\n    iex> config = [\n    ...>   {:bus, [target: {:bus, :default}, stream: \"events\"]},\n    ...>   {:pubsub, [target: {:pubsub, :audit}, topic: \"audit\"]}\n    ...> ]\n    iex> Jido.Signal.Dispatch.validate_opts(config)\n    {:ok, ^config}","ref":"Jido.Signal.Dispatch.html#validate_opts/1-examples"},{"type":"behaviour","title":"Jido.Signal.Dispatch.Adapter","doc":"Defines the behaviour for signal dispatch adapters in the Jido system.\n\nThis behaviour specifies the contract that all signal dispatch adapters must implement.\nAdapters are responsible for validating their configuration options and delivering\nsignals to their respective destinations.","ref":"Jido.Signal.Dispatch.Adapter.html"},{"type":"behaviour","title":"Implementing an Adapter - Jido.Signal.Dispatch.Adapter","doc":"To create a custom adapter, implement this behaviour in your module:\n\n    defmodule MyApp.CustomAdapter do\n      @behaviour Jido.Signal.Dispatch.Adapter\n\n      @impl true\n      def validate_opts(opts) do\n        # Validate the options and return {:ok, validated_opts} or {:error, reason}\n      end\n\n      @impl true\n      def deliver(signal, opts) do\n        # Deliver the signal using the validated options\n        # Return :ok on success or {:error, reason} on failure\n      end\n    end","ref":"Jido.Signal.Dispatch.Adapter.html#module-implementing-an-adapter"},{"type":"behaviour","title":"Required Callbacks - Jido.Signal.Dispatch.Adapter","doc":"* `validate_opts/1` - Validates adapter-specific options before use\n* `deliver/2` - Handles the actual delivery of signals to their destination\n\nSee the callback documentation for detailed specifications.","ref":"Jido.Signal.Dispatch.Adapter.html#module-required-callbacks"},{"type":"callback","title":"Jido.Signal.Dispatch.Adapter.deliver/2","doc":"Delivers a signal using the validated options.\n\nThis callback handles the actual delivery of the signal to its destination. The options\npassed to this function will have already been validated by `validate_opts/1`.","ref":"Jido.Signal.Dispatch.Adapter.html#c:deliver/2"},{"type":"callback","title":"Parameters - Jido.Signal.Dispatch.Adapter.deliver/2","doc":"* `signal` - The signal to deliver (type: `Jido.Signal.t()`)\n* `opts` - Keyword list of validated adapter-specific options","ref":"Jido.Signal.Dispatch.Adapter.html#c:deliver/2-parameters"},{"type":"callback","title":"Returns - Jido.Signal.Dispatch.Adapter.deliver/2","doc":"* `:ok` - Signal was successfully delivered\n* `{:error, reason}` - Delivery failed with reason for failure","ref":"Jido.Signal.Dispatch.Adapter.html#c:deliver/2-returns"},{"type":"callback","title":"Jido.Signal.Dispatch.Adapter.validate_opts/1","doc":"Validates the adapter-specific options.\n\nThis callback is called before any signal delivery to ensure the adapter is properly configured.\nIt should validate all required options are present and have valid values.","ref":"Jido.Signal.Dispatch.Adapter.html#c:validate_opts/1"},{"type":"callback","title":"Parameters - Jido.Signal.Dispatch.Adapter.validate_opts/1","doc":"* `opts` - Keyword list of adapter-specific options","ref":"Jido.Signal.Dispatch.Adapter.html#c:validate_opts/1-parameters"},{"type":"callback","title":"Returns - Jido.Signal.Dispatch.Adapter.validate_opts/1","doc":"* `{:ok, validated_opts}` - Options are valid and possibly normalized\n* `{:error, reason}` - Options are invalid with reason for failure","ref":"Jido.Signal.Dispatch.Adapter.html#c:validate_opts/1-returns"},{"type":"module","title":"Jido.Signal.Dispatch.Bus","doc":"An adapter for dispatching signals through the Jido event bus system.\n\nThis adapter implements the `Jido.Signal.Dispatch.Adapter` behaviour and provides\nfunctionality to publish signals to named event buses. It integrates with the\n`Jido.Bus` system for event distribution.","ref":"Jido.Signal.Dispatch.Bus.html"},{"type":"module","title":"Configuration Options - Jido.Signal.Dispatch.Bus","doc":"* `:target` - (required) The atom name of the target bus\n* `:stream` - (optional) The stream name to publish to, defaults to \"default\"","ref":"Jido.Signal.Dispatch.Bus.html#module-configuration-options"},{"type":"module","title":"Event Bus Integration - Jido.Signal.Dispatch.Bus","doc":"The adapter uses `Jido.Bus` to:\n* Locate the target bus process using `Jido.Bus.whereis/1`\n* Publish signals to the specified stream using `Jido.Bus.publish/4`","ref":"Jido.Signal.Dispatch.Bus.html#module-event-bus-integration"},{"type":"module","title":"Examples - Jido.Signal.Dispatch.Bus","doc":"# Basic usage with default stream\n    config = {:bus, [\n      target: :my_bus\n    ]}\n\n    # Specify custom stream\n    config = {:bus, [\n      target: :my_bus,\n      stream: \"custom_events\"\n    ]}","ref":"Jido.Signal.Dispatch.Bus.html#module-examples"},{"type":"module","title":"Error Handling - Jido.Signal.Dispatch.Bus","doc":"The adapter handles these error conditions:\n\n* `:bus_not_found` - The target bus is not registered\n* Other errors from the bus system","ref":"Jido.Signal.Dispatch.Bus.html#module-error-handling"},{"type":"function","title":"Jido.Signal.Dispatch.Bus.deliver/2","doc":"Delivers a signal to the specified event bus.","ref":"Jido.Signal.Dispatch.Bus.html#deliver/2"},{"type":"function","title":"Parameters - Jido.Signal.Dispatch.Bus.deliver/2","doc":"* `signal` - The signal to deliver\n* `opts` - Validated options from `validate_opts/1`","ref":"Jido.Signal.Dispatch.Bus.html#deliver/2-parameters"},{"type":"function","title":"Options - Jido.Signal.Dispatch.Bus.deliver/2","doc":"* `:target` - (required) The atom name of the target bus\n* `:stream` - (required) The stream name to publish to","ref":"Jido.Signal.Dispatch.Bus.html#deliver/2-options"},{"type":"function","title":"Returns - Jido.Signal.Dispatch.Bus.deliver/2","doc":"* `:ok` - Signal published successfully\n* `{:error, :bus_not_found}` - Target bus not found\n* `{:error, reason}` - Other delivery failure","ref":"Jido.Signal.Dispatch.Bus.html#deliver/2-returns"},{"type":"type","title":"Jido.Signal.Dispatch.Bus.delivery_error/0","doc":"","ref":"Jido.Signal.Dispatch.Bus.html#t:delivery_error/0"},{"type":"type","title":"Jido.Signal.Dispatch.Bus.delivery_opts/0","doc":"","ref":"Jido.Signal.Dispatch.Bus.html#t:delivery_opts/0"},{"type":"type","title":"Jido.Signal.Dispatch.Bus.delivery_target/0","doc":"","ref":"Jido.Signal.Dispatch.Bus.html#t:delivery_target/0"},{"type":"function","title":"Jido.Signal.Dispatch.Bus.validate_opts/1","doc":"Validates the bus adapter configuration options.","ref":"Jido.Signal.Dispatch.Bus.html#validate_opts/1"},{"type":"function","title":"Parameters - Jido.Signal.Dispatch.Bus.validate_opts/1","doc":"* `opts` - Keyword list of options to validate","ref":"Jido.Signal.Dispatch.Bus.html#validate_opts/1-parameters"},{"type":"function","title":"Options - Jido.Signal.Dispatch.Bus.validate_opts/1","doc":"* `:target` - Must be an atom representing the bus name\n* `:stream` - Must be a string, defaults to \"default\"","ref":"Jido.Signal.Dispatch.Bus.html#validate_opts/1-options"},{"type":"function","title":"Returns - Jido.Signal.Dispatch.Bus.validate_opts/1","doc":"* `{:ok, validated_opts}` - Options are valid\n* `{:error, reason}` - Options are invalid with string reason","ref":"Jido.Signal.Dispatch.Bus.html#validate_opts/1-returns"},{"type":"module","title":"Jido.Signal.Dispatch.ConsoleAdapter","doc":"An adapter for dispatching signals directly to the console output (stdout).\n\nThis adapter implements the `Jido.Signal.Dispatch.Adapter` behaviour and provides\nfunctionality to print signals in a human-readable format to the console. It's\nparticularly useful for:\n\n* Interactive development and debugging in IEx sessions\n* Local development and testing\n* Command-line tools and scripts\n* Direct observation of signal flow","ref":"Jido.Signal.Dispatch.ConsoleAdapter.html"},{"type":"module","title":"Output Format - Jido.Signal.Dispatch.ConsoleAdapter","doc":"The adapter prints signals in a structured, easy-to-read format:\n\n```\n[2024-03-21T10:15:30Z] SIGNAL DISPATCHED\nid=signal_id_here\ntype=signal_type_here\nsource=signal_source_here\nmetadata={...}\ndata={...}\n```\n\nFeatures of the output:\n* ISO 8601 timestamp in UTC\n* Clear section headers\n* Pretty-printed metadata and data\n* Consistent formatting for easy parsing","ref":"Jido.Signal.Dispatch.ConsoleAdapter.html#module-output-format"},{"type":"module","title":"Configuration - Jido.Signal.Dispatch.ConsoleAdapter","doc":"This adapter requires no configuration options. Any provided options will be ignored.","ref":"Jido.Signal.Dispatch.ConsoleAdapter.html#module-configuration"},{"type":"module","title":"Examples - Jido.Signal.Dispatch.ConsoleAdapter","doc":"# Basic usage\n    config = {:console, []}\n\n    # Options are ignored but allowed\n    config = {:console, [\n      any_option: :is_ignored\n    ]}","ref":"Jido.Signal.Dispatch.ConsoleAdapter.html#module-examples"},{"type":"module","title":"Use Cases - Jido.Signal.Dispatch.ConsoleAdapter","doc":"* Development and debugging\n* Local testing and verification\n* Signal flow monitoring\n* Educational purposes and demonstrations\n* Command-line tools","ref":"Jido.Signal.Dispatch.ConsoleAdapter.html#module-use-cases"},{"type":"module","title":"Notes - Jido.Signal.Dispatch.ConsoleAdapter","doc":"* Output is always sent to stdout\n* Data structures are pretty-printed for readability\n* Timestamps are always in UTC\n* No configuration options are required or used","ref":"Jido.Signal.Dispatch.ConsoleAdapter.html#module-notes"},{"type":"function","title":"Jido.Signal.Dispatch.ConsoleAdapter.deliver/2","doc":"Prints a signal to the console in a human-readable format.","ref":"Jido.Signal.Dispatch.ConsoleAdapter.html#deliver/2"},{"type":"function","title":"Parameters - Jido.Signal.Dispatch.ConsoleAdapter.deliver/2","doc":"* `signal` - The signal to print\n* `_opts` - Options (ignored)","ref":"Jido.Signal.Dispatch.ConsoleAdapter.html#deliver/2-parameters"},{"type":"function","title":"Returns - Jido.Signal.Dispatch.ConsoleAdapter.deliver/2","doc":"* `:ok` - Signal was printed successfully","ref":"Jido.Signal.Dispatch.ConsoleAdapter.html#deliver/2-returns"},{"type":"function","title":"Examples - Jido.Signal.Dispatch.ConsoleAdapter.deliver/2","doc":"iex> signal = %Jido.Signal{\n    ...>   id: \"123\",\n    ...>   type: \"user:created\",\n    ...>   source: \"user_service\",\n    ...>   data: %{id: 456, name: \"John\"}\n    ...> }\n    iex> ConsoleAdapter.deliver(signal, [])\n    [2024-03-21T10:15:30Z] SIGNAL DISPATCHED\n    id=123\n    type=user:created\n    source=user_service\n    metadata={}\n    data=%{id: 456, name: \"John\"}\n    :ok","ref":"Jido.Signal.Dispatch.ConsoleAdapter.html#deliver/2-examples"},{"type":"function","title":"Jido.Signal.Dispatch.ConsoleAdapter.validate_opts/1","doc":"Validates the console adapter configuration options.\n\nThis adapter accepts any options but doesn't use them. All options are\nconsidered valid.","ref":"Jido.Signal.Dispatch.ConsoleAdapter.html#validate_opts/1"},{"type":"function","title":"Parameters - Jido.Signal.Dispatch.ConsoleAdapter.validate_opts/1","doc":"* `opts` - Keyword list of options (ignored)","ref":"Jido.Signal.Dispatch.ConsoleAdapter.html#validate_opts/1-parameters"},{"type":"function","title":"Returns - Jido.Signal.Dispatch.ConsoleAdapter.validate_opts/1","doc":"* `{:ok, opts}` - Always returns ok with the unchanged options","ref":"Jido.Signal.Dispatch.ConsoleAdapter.html#validate_opts/1-returns"},{"type":"module","title":"Jido.Signal.Dispatch.LoggerAdapter","doc":"An adapter for dispatching signals through Elixir's Logger system.\n\nThis adapter implements the `Jido.Signal.Dispatch.Adapter` behaviour and provides\nfunctionality to log signals using Elixir's built-in Logger. It supports both\nstructured and unstructured logging formats and respects configured log levels.","ref":"Jido.Signal.Dispatch.LoggerAdapter.html"},{"type":"module","title":"Configuration Options - Jido.Signal.Dispatch.LoggerAdapter","doc":"* `:level` - (optional) The log level to use, one of [:debug, :info, :warn, :error], defaults to `:info`\n* `:structured` - (optional) Whether to use structured logging format, defaults to `false`","ref":"Jido.Signal.Dispatch.LoggerAdapter.html#module-configuration-options"},{"type":"module","title":"Logging Formats - Jido.Signal.Dispatch.LoggerAdapter","doc":"","ref":"Jido.Signal.Dispatch.LoggerAdapter.html#module-logging-formats"},{"type":"module","title":"Unstructured (default) - Jido.Signal.Dispatch.LoggerAdapter","doc":"```\nSignal dispatched: signal_type from source with data={...}\n```","ref":"Jido.Signal.Dispatch.LoggerAdapter.html#module-unstructured-default"},{"type":"module","title":"Structured - Jido.Signal.Dispatch.LoggerAdapter","doc":"```elixir\n%{\n  event: \"signal_dispatched\",\n  id: \"signal_id\",\n  type: \"signal_type\",\n  data: {...},\n  source: \"source\"\n}\n```","ref":"Jido.Signal.Dispatch.LoggerAdapter.html#module-structured"},{"type":"module","title":"Examples - Jido.Signal.Dispatch.LoggerAdapter","doc":"# Basic usage with default level\n    config = {:logger, []}\n\n    # Custom log level\n    config = {:logger, [\n      level: :debug\n    ]}\n\n    # Structured logging\n    config = {:logger, [\n      level: :info,\n      structured: true\n    ]}","ref":"Jido.Signal.Dispatch.LoggerAdapter.html#module-examples"},{"type":"module","title":"Integration with Logger - Jido.Signal.Dispatch.LoggerAdapter","doc":"The adapter integrates with Elixir's Logger system, which means:\n* Log messages respect the global Logger configuration\n* Metadata and formatting can be customized through Logger backends\n* Log levels can be filtered at runtime\n* Structured logging can be processed by log aggregators","ref":"Jido.Signal.Dispatch.LoggerAdapter.html#module-integration-with-logger"},{"type":"module","title":"Notes - Jido.Signal.Dispatch.LoggerAdapter","doc":"* Consider using structured logging when integrating with log aggregation systems\n* Log levels should be chosen based on the signal's importance\n* High-volume signals should use `:debug` level to avoid log spam","ref":"Jido.Signal.Dispatch.LoggerAdapter.html#module-notes"},{"type":"function","title":"Jido.Signal.Dispatch.LoggerAdapter.deliver/2","doc":"Logs a signal using the configured format and level.","ref":"Jido.Signal.Dispatch.LoggerAdapter.html#deliver/2"},{"type":"function","title":"Parameters - Jido.Signal.Dispatch.LoggerAdapter.deliver/2","doc":"* `signal` - The signal to log\n* `opts` - Validated options from `validate_opts/1`","ref":"Jido.Signal.Dispatch.LoggerAdapter.html#deliver/2-parameters"},{"type":"function","title":"Options - Jido.Signal.Dispatch.LoggerAdapter.deliver/2","doc":"* `:level` - (optional) The log level to use, defaults to `:info`\n* `:structured` - (optional) Whether to use structured format, defaults to `false`","ref":"Jido.Signal.Dispatch.LoggerAdapter.html#deliver/2-options"},{"type":"function","title":"Returns - Jido.Signal.Dispatch.LoggerAdapter.deliver/2","doc":"* `:ok` - Signal was logged successfully","ref":"Jido.Signal.Dispatch.LoggerAdapter.html#deliver/2-returns"},{"type":"function","title":"Examples - Jido.Signal.Dispatch.LoggerAdapter.deliver/2","doc":"iex> signal = %Jido.Signal{type: \"user:created\", data: %{id: 123}}\n    iex> LoggerAdapter.deliver(signal, [level: :info])\n    :ok\n    # Logs: \"Signal dispatched: user:created from source with data=%{id: 123}\"\n\n    iex> LoggerAdapter.deliver(signal, [level: :info, structured: true])\n    :ok\n    # Logs structured map with event details","ref":"Jido.Signal.Dispatch.LoggerAdapter.html#deliver/2-examples"},{"type":"function","title":"Jido.Signal.Dispatch.LoggerAdapter.validate_opts/1","doc":"Validates the logger adapter configuration options.","ref":"Jido.Signal.Dispatch.LoggerAdapter.html#validate_opts/1"},{"type":"function","title":"Parameters - Jido.Signal.Dispatch.LoggerAdapter.validate_opts/1","doc":"* `opts` - Keyword list of options to validate","ref":"Jido.Signal.Dispatch.LoggerAdapter.html#validate_opts/1-parameters"},{"type":"function","title":"Options - Jido.Signal.Dispatch.LoggerAdapter.validate_opts/1","doc":"* `:level` - (optional) One of [:debug, :info, :warn, :error], defaults to `:info`\n* `:structured` - (optional) Boolean, defaults to `false`","ref":"Jido.Signal.Dispatch.LoggerAdapter.html#validate_opts/1-options"},{"type":"function","title":"Returns - Jido.Signal.Dispatch.LoggerAdapter.validate_opts/1","doc":"* `{:ok, validated_opts}` - Options are valid\n* `{:error, reason}` - Options are invalid with string reason","ref":"Jido.Signal.Dispatch.LoggerAdapter.html#validate_opts/1-returns"},{"type":"module","title":"Jido.Signal.Dispatch.Named","doc":"An adapter for dispatching signals to named processes in the Erlang registry.\n\nThis adapter implements the `Jido.Signal.Dispatch.Adapter` behaviour and provides\nfunctionality to send signals to processes registered under a name in the Erlang\nprocess registry. It supports both synchronous and asynchronous delivery modes.","ref":"Jido.Signal.Dispatch.Named.html"},{"type":"module","title":"Configuration Options - Jido.Signal.Dispatch.Named","doc":"* `:target` - (required) A tuple of `{:name, atom()}` specifying the registered process name\n* `:delivery_mode` - (optional) Either `:sync` or `:async`, defaults to `:async`\n* `:timeout` - (optional) Timeout for synchronous delivery in milliseconds, defaults to 5000\n* `:message_format` - (optional) Function to format the signal before sending, defaults to wrapping in `{:signal, signal}`","ref":"Jido.Signal.Dispatch.Named.html#module-configuration-options"},{"type":"module","title":"Delivery Modes - Jido.Signal.Dispatch.Named","doc":"* `:async` - Uses `send/2` to deliver the signal without waiting for a response\n* `:sync` - Uses `GenServer.call/3` to deliver the signal and wait for a response","ref":"Jido.Signal.Dispatch.Named.html#module-delivery-modes"},{"type":"module","title":"Examples - Jido.Signal.Dispatch.Named","doc":"# Asynchronous delivery to named process\n    config = {:named, [\n      target: {:name, :my_process},\n      delivery_mode: :async\n    ]}\n\n    # Synchronous delivery with custom timeout\n    config = {:named, [\n      target: {:name, :my_process},\n      delivery_mode: :sync,\n      timeout: 10_000\n    ]}\n\n    # Custom message format\n    config = {:named, [\n      target: {:name, :my_process},\n      message_format: fn signal -> {:custom_signal, signal} end\n    ]}","ref":"Jido.Signal.Dispatch.Named.html#module-examples"},{"type":"module","title":"Error Handling - Jido.Signal.Dispatch.Named","doc":"The adapter handles these error conditions:\n\n* `:process_not_found` - The named process is not registered\n* `:process_not_alive` - The process exists but is not alive\n* `:timeout` - Synchronous delivery timed out\n* Other errors from the target process","ref":"Jido.Signal.Dispatch.Named.html#module-error-handling"},{"type":"function","title":"Jido.Signal.Dispatch.Named.deliver/2","doc":"Delivers a signal to the named process.","ref":"Jido.Signal.Dispatch.Named.html#deliver/2"},{"type":"function","title":"Parameters - Jido.Signal.Dispatch.Named.deliver/2","doc":"* `signal` - The signal to deliver\n* `opts` - Validated options from `validate_opts/1`","ref":"Jido.Signal.Dispatch.Named.html#deliver/2-parameters"},{"type":"function","title":"Options - Jido.Signal.Dispatch.Named.deliver/2","doc":"* `:target` - (required) Tuple of `{:name, atom()}` identifying the process\n* `:delivery_mode` - (required) Either `:sync` or `:async`\n* `:timeout` - (optional) Timeout for sync delivery, defaults to 5000ms\n* `:message_format` - (optional) Function to format the signal","ref":"Jido.Signal.Dispatch.Named.html#deliver/2-options"},{"type":"function","title":"Returns - Jido.Signal.Dispatch.Named.deliver/2","doc":"* `:ok` - Signal delivered successfully (async mode)\n* `{:ok, term()}` - Signal delivered and response received (sync mode)\n* `{:error, reason}` - Delivery failed","ref":"Jido.Signal.Dispatch.Named.html#deliver/2-returns"},{"type":"type","title":"Jido.Signal.Dispatch.Named.delivery_error/0","doc":"","ref":"Jido.Signal.Dispatch.Named.html#t:delivery_error/0"},{"type":"type","title":"Jido.Signal.Dispatch.Named.delivery_mode/0","doc":"","ref":"Jido.Signal.Dispatch.Named.html#t:delivery_mode/0"},{"type":"type","title":"Jido.Signal.Dispatch.Named.delivery_opts/0","doc":"","ref":"Jido.Signal.Dispatch.Named.html#t:delivery_opts/0"},{"type":"type","title":"Jido.Signal.Dispatch.Named.delivery_target/0","doc":"","ref":"Jido.Signal.Dispatch.Named.html#t:delivery_target/0"},{"type":"type","title":"Jido.Signal.Dispatch.Named.message_format/0","doc":"","ref":"Jido.Signal.Dispatch.Named.html#t:message_format/0"},{"type":"function","title":"Jido.Signal.Dispatch.Named.validate_opts/1","doc":"Validates the named process adapter configuration options.","ref":"Jido.Signal.Dispatch.Named.html#validate_opts/1"},{"type":"function","title":"Parameters - Jido.Signal.Dispatch.Named.validate_opts/1","doc":"* `opts` - Keyword list of options to validate","ref":"Jido.Signal.Dispatch.Named.html#validate_opts/1-parameters"},{"type":"function","title":"Options - Jido.Signal.Dispatch.Named.validate_opts/1","doc":"* `:target` - Must be a tuple of `{:name, atom()}`\n* `:delivery_mode` - Must be either `:sync` or `:async`","ref":"Jido.Signal.Dispatch.Named.html#validate_opts/1-options"},{"type":"function","title":"Returns - Jido.Signal.Dispatch.Named.validate_opts/1","doc":"* `{:ok, validated_opts}` - Options are valid\n* `{:error, :invalid_target}` - Target is not a valid name tuple\n* `{:error, :invalid_delivery_mode}` - Delivery mode is invalid","ref":"Jido.Signal.Dispatch.Named.html#validate_opts/1-returns"},{"type":"module","title":"Jido.Signal.Dispatch.NoopAdapter","doc":"A no-operation adapter that silently discards all signals.\n\nThis adapter implements the `Jido.Signal.Dispatch.Adapter` behaviour but performs\nno actual signal delivery. It's useful for:\n\n* Testing and development environments where signals should be ignored\n* Disabling signal delivery without changing configuration structure\n* Performance testing without signal delivery overhead\n* Mocking signal dispatch in test environments","ref":"Jido.Signal.Dispatch.NoopAdapter.html"},{"type":"module","title":"Configuration - Jido.Signal.Dispatch.NoopAdapter","doc":"This adapter accepts any options but ignores them all. No configuration is needed\nor used.","ref":"Jido.Signal.Dispatch.NoopAdapter.html#module-configuration"},{"type":"module","title":"Examples - Jido.Signal.Dispatch.NoopAdapter","doc":"# Basic usage\n    config = {:noop, []}\n\n    # Options are allowed but ignored\n    config = {:noop, [\n      any_option: :is_ignored\n    ]}","ref":"Jido.Signal.Dispatch.NoopAdapter.html#module-examples"},{"type":"module","title":"Use Cases - Jido.Signal.Dispatch.NoopAdapter","doc":"* Test environments\n* Development configurations\n* Performance benchmarking\n* Signal flow testing\n* Temporary signal suppression","ref":"Jido.Signal.Dispatch.NoopAdapter.html#module-use-cases"},{"type":"module","title":"Notes - Jido.Signal.Dispatch.NoopAdapter","doc":"* All signals are silently discarded\n* No errors are ever returned\n* No side effects are produced\n* Completely thread-safe and concurrent","ref":"Jido.Signal.Dispatch.NoopAdapter.html#module-notes"},{"type":"function","title":"Jido.Signal.Dispatch.NoopAdapter.deliver/2","doc":"Silently discards the signal without performing any operation.","ref":"Jido.Signal.Dispatch.NoopAdapter.html#deliver/2"},{"type":"function","title":"Parameters - Jido.Signal.Dispatch.NoopAdapter.deliver/2","doc":"* `_signal` - The signal to discard (ignored)\n* `_opts` - Options (ignored)","ref":"Jido.Signal.Dispatch.NoopAdapter.html#deliver/2-parameters"},{"type":"function","title":"Returns - Jido.Signal.Dispatch.NoopAdapter.deliver/2","doc":"* `:ok` - Always returns `:ok`","ref":"Jido.Signal.Dispatch.NoopAdapter.html#deliver/2-returns"},{"type":"function","title":"Examples - Jido.Signal.Dispatch.NoopAdapter.deliver/2","doc":"iex> signal = %Jido.Signal{type: \"test:event\"}\n    iex> NoopAdapter.deliver(signal, [])\n    :ok","ref":"Jido.Signal.Dispatch.NoopAdapter.html#deliver/2-examples"},{"type":"function","title":"Jido.Signal.Dispatch.NoopAdapter.validate_opts/1","doc":"Validates the noop adapter configuration options.\n\nThis adapter accepts any options but doesn't use them. All options are\nconsidered valid.","ref":"Jido.Signal.Dispatch.NoopAdapter.html#validate_opts/1"},{"type":"function","title":"Parameters - Jido.Signal.Dispatch.NoopAdapter.validate_opts/1","doc":"* `opts` - Keyword list of options (ignored)","ref":"Jido.Signal.Dispatch.NoopAdapter.html#validate_opts/1-parameters"},{"type":"function","title":"Returns - Jido.Signal.Dispatch.NoopAdapter.validate_opts/1","doc":"* `{:ok, opts}` - Always returns ok with the unchanged options","ref":"Jido.Signal.Dispatch.NoopAdapter.html#validate_opts/1-returns"},{"type":"module","title":"Jido.Signal.Dispatch.PidAdapter","doc":"An adapter for dispatching signals directly to Erlang processes using PIDs.\n\nThis adapter implements the `Jido.Signal.Dispatch.Adapter` behaviour and provides\nfunctionality to send signals to specific processes either synchronously or\nasynchronously.","ref":"Jido.Signal.Dispatch.PidAdapter.html"},{"type":"module","title":"Configuration Options - Jido.Signal.Dispatch.PidAdapter","doc":"* `:target` - (required) The PID of the destination process\n* `:delivery_mode` - (optional) Either `:sync` or `:async`, defaults to `:async`\n* `:timeout` - (optional) Timeout for synchronous delivery in milliseconds, defaults to 5000\n* `:message_format` - (optional) Function to format the signal before sending, defaults to wrapping in `{:signal, signal}`","ref":"Jido.Signal.Dispatch.PidAdapter.html#module-configuration-options"},{"type":"module","title":"Delivery Modes - Jido.Signal.Dispatch.PidAdapter","doc":"* `:async` - Uses `send/2` to deliver the signal without waiting for a response\n* `:sync` - Uses `GenServer.call/3` to deliver the signal and wait for a response","ref":"Jido.Signal.Dispatch.PidAdapter.html#module-delivery-modes"},{"type":"module","title":"Examples - Jido.Signal.Dispatch.PidAdapter","doc":"# Asynchronous delivery\n    config = {:pid, [\n      target: destination_pid,\n      delivery_mode: :async\n    ]}\n\n    # Synchronous delivery with custom timeout\n    config = {:pid, [\n      target: destination_pid,\n      delivery_mode: :sync,\n      timeout: 10_000\n    ]}\n\n    # Custom message format\n    config = {:pid, [\n      target: destination_pid,\n      message_format: fn signal -> {:custom_signal, signal} end\n    ]}","ref":"Jido.Signal.Dispatch.PidAdapter.html#module-examples"},{"type":"module","title":"Error Handling - Jido.Signal.Dispatch.PidAdapter","doc":"The adapter handles various error conditions:\n\n* `:process_not_alive` - The target process is not alive\n* `:timeout` - Synchronous delivery timed out\n* Other errors from the target process","ref":"Jido.Signal.Dispatch.PidAdapter.html#module-error-handling"},{"type":"function","title":"Jido.Signal.Dispatch.PidAdapter.deliver/2","doc":"Delivers a signal to the target process.","ref":"Jido.Signal.Dispatch.PidAdapter.html#deliver/2"},{"type":"function","title":"Parameters - Jido.Signal.Dispatch.PidAdapter.deliver/2","doc":"* `signal` - The signal to deliver\n* `opts` - Validated options from `validate_opts/1`","ref":"Jido.Signal.Dispatch.PidAdapter.html#deliver/2-parameters"},{"type":"function","title":"Options - Jido.Signal.Dispatch.PidAdapter.deliver/2","doc":"* `:target` - (required) Target PID to deliver to\n* `:delivery_mode` - (required) Either `:sync` or `:async`\n* `:timeout` - (optional) Timeout for sync delivery, defaults to 5000ms\n* `:message_format` - (optional) Function to format the signal","ref":"Jido.Signal.Dispatch.PidAdapter.html#deliver/2-options"},{"type":"function","title":"Returns - Jido.Signal.Dispatch.PidAdapter.deliver/2","doc":"* `:ok` - Signal delivered successfully (async mode)\n* `{:ok, term()}` - Signal delivered and response received (sync mode)\n* `{:error, reason}` - Delivery failed","ref":"Jido.Signal.Dispatch.PidAdapter.html#deliver/2-returns"},{"type":"type","title":"Jido.Signal.Dispatch.PidAdapter.delivery_error/0","doc":"","ref":"Jido.Signal.Dispatch.PidAdapter.html#t:delivery_error/0"},{"type":"type","title":"Jido.Signal.Dispatch.PidAdapter.delivery_mode/0","doc":"","ref":"Jido.Signal.Dispatch.PidAdapter.html#t:delivery_mode/0"},{"type":"type","title":"Jido.Signal.Dispatch.PidAdapter.delivery_opts/0","doc":"","ref":"Jido.Signal.Dispatch.PidAdapter.html#t:delivery_opts/0"},{"type":"type","title":"Jido.Signal.Dispatch.PidAdapter.delivery_target/0","doc":"","ref":"Jido.Signal.Dispatch.PidAdapter.html#t:delivery_target/0"},{"type":"type","title":"Jido.Signal.Dispatch.PidAdapter.message_format/0","doc":"","ref":"Jido.Signal.Dispatch.PidAdapter.html#t:message_format/0"},{"type":"function","title":"Jido.Signal.Dispatch.PidAdapter.validate_opts/1","doc":"Validates the PID adapter configuration options.","ref":"Jido.Signal.Dispatch.PidAdapter.html#validate_opts/1"},{"type":"function","title":"Parameters - Jido.Signal.Dispatch.PidAdapter.validate_opts/1","doc":"* `opts` - Keyword list of options to validate","ref":"Jido.Signal.Dispatch.PidAdapter.html#validate_opts/1-parameters"},{"type":"function","title":"Options - Jido.Signal.Dispatch.PidAdapter.validate_opts/1","doc":"* `:target` - Must be a valid PID\n* `:delivery_mode` - Must be either `:sync` or `:async`","ref":"Jido.Signal.Dispatch.PidAdapter.html#validate_opts/1-options"},{"type":"function","title":"Returns - Jido.Signal.Dispatch.PidAdapter.validate_opts/1","doc":"* `{:ok, validated_opts}` - Options are valid\n* `{:error, :invalid_target}` - Target is not a valid PID\n* `{:error, :invalid_delivery_mode}` - Delivery mode is invalid","ref":"Jido.Signal.Dispatch.PidAdapter.html#validate_opts/1-returns"},{"type":"module","title":"Jido.Signal.Dispatch.PubSub","doc":"An adapter for dispatching signals through Phoenix.PubSub.\n\nThis adapter implements the `Jido.Signal.Dispatch.Adapter` behaviour and provides\nfunctionality to broadcast signals through Phoenix.PubSub to all subscribers of a\nspecific topic. It integrates with Phoenix's pub/sub system for distributed\nmessage broadcasting.","ref":"Jido.Signal.Dispatch.PubSub.html"},{"type":"module","title":"Configuration Options - Jido.Signal.Dispatch.PubSub","doc":"* `:target` - (required) An atom specifying the PubSub server name\n* `:topic` - (required) A string specifying the topic to broadcast on","ref":"Jido.Signal.Dispatch.PubSub.html#module-configuration-options"},{"type":"module","title":"Phoenix.PubSub Integration - Jido.Signal.Dispatch.PubSub","doc":"The adapter uses `Phoenix.PubSub.broadcast/3` to:\n* Broadcast signals to all subscribers of a topic\n* Handle distributed message delivery across nodes\n* Manage subscription-based message routing","ref":"Jido.Signal.Dispatch.PubSub.html#module-phoenix-pubsub-integration"},{"type":"module","title":"Examples - Jido.Signal.Dispatch.PubSub","doc":"# Basic usage\n    config = {:pubsub, [\n      target: :my_app_pubsub,\n      topic: \"events\"\n    ]}\n\n    # Using with specific event topics\n    config = {:pubsub, [\n      target: :my_app_pubsub,\n      topic: \"user:123:events\"\n    ]}","ref":"Jido.Signal.Dispatch.PubSub.html#module-examples"},{"type":"module","title":"Error Handling - Jido.Signal.Dispatch.PubSub","doc":"The adapter handles these error conditions:\n\n* `:pubsub_not_found` - The target PubSub server is not running\n* Other errors from the Phoenix.PubSub system","ref":"Jido.Signal.Dispatch.PubSub.html#module-error-handling"},{"type":"module","title":"Notes - Jido.Signal.Dispatch.PubSub","doc":"* Ensure the PubSub server is started in your application supervision tree\n* Topics can be any string, but consider using consistent naming patterns\n* Messages are broadcast to all subscribers, so consider message volume","ref":"Jido.Signal.Dispatch.PubSub.html#module-notes"},{"type":"function","title":"Jido.Signal.Dispatch.PubSub.deliver/2","doc":"Broadcasts a signal through Phoenix.PubSub.","ref":"Jido.Signal.Dispatch.PubSub.html#deliver/2"},{"type":"function","title":"Parameters - Jido.Signal.Dispatch.PubSub.deliver/2","doc":"* `signal` - The signal to broadcast\n* `opts` - Validated options from `validate_opts/1`","ref":"Jido.Signal.Dispatch.PubSub.html#deliver/2-parameters"},{"type":"function","title":"Options - Jido.Signal.Dispatch.PubSub.deliver/2","doc":"* `:target` - (required) Atom identifying the PubSub server\n* `:topic` - (required) String topic to broadcast on","ref":"Jido.Signal.Dispatch.PubSub.html#deliver/2-options"},{"type":"function","title":"Returns - Jido.Signal.Dispatch.PubSub.deliver/2","doc":"* `:ok` - Signal broadcast successfully\n* `{:error, :pubsub_not_found}` - PubSub server not found\n* `{:error, reason}` - Other broadcast failure","ref":"Jido.Signal.Dispatch.PubSub.html#deliver/2-returns"},{"type":"type","title":"Jido.Signal.Dispatch.PubSub.delivery_error/0","doc":"","ref":"Jido.Signal.Dispatch.PubSub.html#t:delivery_error/0"},{"type":"type","title":"Jido.Signal.Dispatch.PubSub.delivery_opts/0","doc":"","ref":"Jido.Signal.Dispatch.PubSub.html#t:delivery_opts/0"},{"type":"type","title":"Jido.Signal.Dispatch.PubSub.delivery_target/0","doc":"","ref":"Jido.Signal.Dispatch.PubSub.html#t:delivery_target/0"},{"type":"function","title":"Jido.Signal.Dispatch.PubSub.validate_opts/1","doc":"Validates the PubSub adapter configuration options.","ref":"Jido.Signal.Dispatch.PubSub.html#validate_opts/1"},{"type":"function","title":"Parameters - Jido.Signal.Dispatch.PubSub.validate_opts/1","doc":"* `opts` - Keyword list of options to validate","ref":"Jido.Signal.Dispatch.PubSub.html#validate_opts/1-parameters"},{"type":"function","title":"Options - Jido.Signal.Dispatch.PubSub.validate_opts/1","doc":"* `:target` - Must be an atom representing the PubSub server name\n* `:topic` - Must be a string","ref":"Jido.Signal.Dispatch.PubSub.html#validate_opts/1-options"},{"type":"function","title":"Returns - Jido.Signal.Dispatch.PubSub.validate_opts/1","doc":"* `{:ok, validated_opts}` - Options are valid\n* `{:error, reason}` - Options are invalid with string reason","ref":"Jido.Signal.Dispatch.PubSub.html#validate_opts/1-returns"},{"type":"behaviour","title":"Jido.Skill","doc":"Defines the core behavior and structure for Jido Skills, the fundamental building blocks\nof agent capabilities in the Jido framework.","ref":"Jido.Skill.html"},{"type":"behaviour","title":"Overview - Jido.Skill","doc":"Skills encapsulate discrete sets of functionality that agents can use to accomplish tasks.\nThink of them as feature packs that give agents new abilities - similar to how a person might\nlearn skills like \"cooking\" or \"programming\". Each Skill provides:\n\n- Signal routing and handling patterns\n- Isolated state management\n- Process supervision\n- Configuration validation\n- Runtime adaptation","ref":"Jido.Skill.html#module-overview"},{"type":"behaviour","title":"Core Concepts - Jido.Skill","doc":"","ref":"Jido.Skill.html#module-core-concepts"},{"type":"behaviour","title":"State Isolation - Jido.Skill","doc":"Skills use schema-based state isolation to prevent different capabilities from interfering\nwith each other. Each skill defines:\n\n- A unique `schema_key` for namespace isolation\n- An `initial_state/0` callback for state setup\n- Validation rules for configuration","ref":"Jido.Skill.html#module-state-isolation"},{"type":"behaviour","title":"Signal Patterns - Jido.Skill","doc":"Skills define what signals they can handle through pattern matching:\n\n```elixir\nuse Jido.Skill,\n  name: \"weather_monitor\",\n  signals: [\n    input: [\"weather.data.*\", \"weather.alert.**\"],\n    output: [\"weather.alert.generated\"]\n  ]\n```\n\nPattern rules:\n- Exact matches: \"user.created\"\n- Single wildcards: \"user.*.updated\"\n- Multi-wildcards: \"audit.**\"","ref":"Jido.Skill.html#module-signal-patterns"},{"type":"behaviour","title":"Configuration Management - Jido.Skill","doc":"Skills provide schema-based config validation:\n\n```elixir\nconfig: [\n  api_key: [\n    type: :string,\n    required: true,\n    doc: \"API key for weather service\"\n  ],\n  update_interval: [\n    type: :pos_integer,\n    default: 60_000,\n    doc: \"Update interval in milliseconds\"\n  ]\n]\n```","ref":"Jido.Skill.html#module-configuration-management"},{"type":"behaviour","title":"Process Supervision - Jido.Skill","doc":"Skills can define child processes through the `child_spec/1` callback:\n\n```elixir\ndef child_spec(config) do\n  [\n    {WeatherAPI.Client, config.api_key},\n    {MetricsCollector, name: config.metrics_name}\n  ]\nend\n```","ref":"Jido.Skill.html#module-process-supervision"},{"type":"behaviour","title":"Usage Example - Jido.Skill","doc":"Here's a complete skill example:\n\n```elixir\ndefmodule MyApp.WeatherSkill do\n  use Jido.Skill,\n    name: \"weather_monitor\",\n    description: \"Monitors weather conditions and generates alerts\",\n    category: \"monitoring\",\n    tags: [\"weather\", \"alerts\"],\n    vsn: \"1.0.0\",\n    schema_key: :weather,\n    signals: [\n      input: [\"weather.data.*\", \"weather.alert.**\"],\n      output: [\"weather.alert.generated\"]\n    ],\n    config: [\n      api_key: [type: :string, required: true]\n    ]\n\n  def initial_state do\n    %{\n      current_conditions: nil,\n      alert_history: [],\n      last_update: nil\n    }\n  end\n\n  def child_spec(config) do\n    [\n      {WeatherAPI.Client, config.api_key}\n    ]\n  end\n\n  def handle_signal(%Signal{type: \"weather.data.updated\"} = signal) do\n    # Handle weather updates\n    {:ok, signal}\n  end\nend\n```","ref":"Jido.Skill.html#module-usage-example"},{"type":"behaviour","title":"Callbacks - Jido.Skill","doc":"Skills implement these callbacks:\n\n- `initial_state/0` - Returns the skill's initial state map\n- `child_spec/1` - Returns child process specifications\n- `routes/0` - Returns signal routing rules\n- `handle_signal/1` - Processes incoming signals\n- `process_result/2` - Post-processes signal handling results","ref":"Jido.Skill.html#module-callbacks"},{"type":"behaviour","title":"Behavior - Jido.Skill","doc":"The Skill behavior enforces a consistent interface:\n\n```elixir\n@callback initial_state() :: map()\n@callback child_spec(config :: map()) :: Supervisor.child_spec() | [Supervisor.child_spec()]\n@callback routes() :: [map()]\n@callback handle_signal(signal :: Signal.t()) :: {:ok, Signal.t()} | {:error, term()}\n@callback process_result(signal :: Signal.t(), result :: term()) ::\n            {:ok, term()} | {:error, term()}\n```","ref":"Jido.Skill.html#module-behavior"},{"type":"behaviour","title":"Configuration - Jido.Skill","doc":"Skills validate their configuration at compile time using these fields:\n\n- `name` - Unique identifier (required)\n- `description` - Human-readable explanation\n- `category` - Broad classification\n- `tags` - List of searchable tags\n- `vsn` - Version string\n- `schema_key` - State namespace key\n- `signals` - Input/output patterns\n- `config` - Configuration schema","ref":"Jido.Skill.html#module-configuration"},{"type":"behaviour","title":"Best Practices - Jido.Skill","doc":"1. **State Isolation**\n   - Use meaningful schema_key names\n   - Keep state focused and minimal\n   - Document state structure\n\n2. **Signal Design**\n   - Use consistent naming patterns\n   - Document signal formats\n   - Consider routing efficiency\n\n3. **Configuration**\n   - Validate thoroughly\n   - Provide good defaults\n   - Document all options\n\n4. **Process Management**\n   - Supervise child processes\n   - Handle crashes gracefully\n   - Monitor resource usage","ref":"Jido.Skill.html#module-best-practices"},{"type":"behaviour","title":"See Also - Jido.Skill","doc":"- `Jido.Signal` - Signal structure and validation\n- `Jido.Error` - Error handling\n- `Jido.Agent` - Agent integration","ref":"Jido.Skill.html#module-see-also"},{"type":"macro","title":"Jido.Skill.__using__/1","doc":"Implements the skill behavior and configuration validation.\n\nThis macro:\n1. Validates configuration at compile time\n2. Defines metadata accessors\n3. Provides JSON serialization\n4. Sets up default implementations","ref":"Jido.Skill.html#__using__/1"},{"type":"macro","title":"Example - Jido.Skill.__using__/1","doc":"defmodule MySkill do\n      use Jido.Skill,\n        name: \"my_skill\",\n        schema_key: :my_skill,\n        signals: [\n          input: [\"my.event.*\"],\n          output: [\"my.result.*\"]\n        ]\n    end","ref":"Jido.Skill.html#__using__/1-example"},{"type":"callback","title":"Jido.Skill.child_spec/1","doc":"","ref":"Jido.Skill.html#c:child_spec/1"},{"type":"function","title":"Jido.Skill.get_config_schema/1","doc":"Gets a skill's configuration schema.","ref":"Jido.Skill.html#get_config_schema/1"},{"type":"function","title":"Parameters - Jido.Skill.get_config_schema/1","doc":"- `skill_module`: The skill module to inspect","ref":"Jido.Skill.html#get_config_schema/1-parameters"},{"type":"function","title":"Returns - Jido.Skill.get_config_schema/1","doc":"- `{:ok, schema}`: The skill's config schema\n- `{:error, reason}`: Schema not found","ref":"Jido.Skill.html#get_config_schema/1-returns"},{"type":"function","title":"Example - Jido.Skill.get_config_schema/1","doc":"Skill.get_config_schema(WeatherSkill)","ref":"Jido.Skill.html#get_config_schema/1-example"},{"type":"callback","title":"Jido.Skill.handle_signal/1","doc":"","ref":"Jido.Skill.html#c:handle_signal/1"},{"type":"callback","title":"Jido.Skill.initial_state/0","doc":"","ref":"Jido.Skill.html#c:initial_state/0"},{"type":"function","title":"Jido.Skill.new/0","doc":"Skills must be defined at compile time, not runtime.\n\nThis function always returns an error to enforce compile-time definition.","ref":"Jido.Skill.html#new/0"},{"type":"callback","title":"Jido.Skill.process_result/2","doc":"","ref":"Jido.Skill.html#c:process_result/2"},{"type":"callback","title":"Jido.Skill.routes/0","doc":"","ref":"Jido.Skill.html#c:routes/0"},{"type":"type","title":"Jido.Skill.t/0","doc":"Represents a skill's core structure and metadata.\n\nFields:\n- `name`: Unique identifier for the skill\n- `description`: Human-readable explanation of purpose\n- `category`: Broad classification for organization\n- `tags`: List of searchable tags\n- `vsn`: Version string for compatibility\n- `schema_key`: Atom key for state namespace\n- `signals`: Input/output signal patterns\n- `config`: Configuration schema","ref":"Jido.Skill.html#t:t/0"},{"type":"function","title":"Jido.Skill.validate_config/2","doc":"Validates a skill's configuration against its schema.","ref":"Jido.Skill.html#validate_config/2"},{"type":"function","title":"Parameters - Jido.Skill.validate_config/2","doc":"- `skill_module`: The skill module to validate\n- `config`: Configuration map to validate","ref":"Jido.Skill.html#validate_config/2-parameters"},{"type":"function","title":"Returns - Jido.Skill.validate_config/2","doc":"- `{:ok, validated_config}`: Successfully validated config\n- `{:error, reason}`: Validation failed","ref":"Jido.Skill.html#validate_config/2-returns"},{"type":"function","title":"Example - Jido.Skill.validate_config/2","doc":"Skill.validate_config(WeatherSkill, %{\n      api_key: \"abc123\",\n      interval: 1000\n    })","ref":"Jido.Skill.html#validate_config/2-example"},{"type":"function","title":"Jido.Skill.validate_signal/2","doc":"Validates a signal against a skill's defined patterns.","ref":"Jido.Skill.html#validate_signal/2"},{"type":"function","title":"Parameters - Jido.Skill.validate_signal/2","doc":"- `signal`: The signal to validate\n- `patterns`: Map of input/output patterns","ref":"Jido.Skill.html#validate_signal/2-parameters"},{"type":"function","title":"Returns - Jido.Skill.validate_signal/2","doc":"- `:ok`: Signal matches patterns\n- `{:error, reason}`: No matching patterns","ref":"Jido.Skill.html#validate_signal/2-returns"},{"type":"function","title":"Example - Jido.Skill.validate_signal/2","doc":"Skill.validate_signal(\n      %Signal{type: \"weather.data.updated\"},\n      %{input: [\"weather.data.*\"], output: []}\n    )","ref":"Jido.Skill.html#validate_signal/2-example"},{"type":"module","title":"Jido.Skills.Arithmetic","doc":"Provides basic arithmetic operations as a Skill.\n\nThis skill exposes arithmetic operations like addition, subtraction, multiplication,\ndivision and squaring through a signal-based interface.","ref":"Jido.Skills.Arithmetic.html"},{"type":"function","title":"Jido.Skills.Arithmetic.handle_signal/1","doc":"Handle an arithmetic signal.","ref":"Jido.Skills.Arithmetic.html#handle_signal/1"},{"type":"function","title":"Jido.Skills.Arithmetic.initial_state/0","doc":"Get the initial state for the arithmetic skill.","ref":"Jido.Skills.Arithmetic.html#initial_state/0"},{"type":"function","title":"Jido.Skills.Arithmetic.process_result/2","doc":"Process the result of an arithmetic operation.","ref":"Jido.Skills.Arithmetic.html#process_result/2"},{"type":"function","title":"Jido.Skills.Arithmetic.routes/0","doc":"Skill: Arithmetic\nSignal Contracts:\n- Incoming:\n  * arithmetic.add: Add two numbers\n  * arithmetic.subtract: Subtract two numbers\n  * arithmetic.multiply: Multiply two numbers\n  * arithmetic.divide: Divide two numbers\n  * arithmetic.square: Square a number\n  * arithmetic.eval: Evaluate a mathematical expression\n- Outgoing:\n  * arithmetic.result: Result of arithmetic operation\n  * arithmetic.error: Error from arithmetic operation","ref":"Jido.Skills.Arithmetic.html#routes/0"},{"type":"module","title":"Jido.Actions.Arithmetic","doc":"Provides basic arithmetic operations as actions. The operations defined here are obviously trivial.\nThis module is intended to be used as a reference for how to write your own actions.\n\nThis module defines a set of arithmetic actions that can be used in workflows:\n- Add: Adds two numbers together\n- Subtract: Subtracts one number from another\n- Multiply: Multiplies two numbers\n- Divide: Divides one number by another, handling division by zero\n- Square: Squares a number\n\nEach action is implemented as a separate submodule and follows the Jido.Action behavior.","ref":"Jido.Actions.Arithmetic.html"},{"type":"module","title":"Jido.Actions.Basic","doc":"A collection of basic actions for common workflows.\n\nThis module provides a set of simple, reusable actions:\n- Sleep: Pauses execution for a specified duration\n- Log: Logs a message with a specified level\n- Todo: Logs a todo item as a placeholder or reminder\n- RandomSleep: Introduces a random delay within a specified range\n- Increment: Increments a value by 1\n- Decrement: Decrements a value by 1\n- Noop: No operation, returns input unchanged\n\nEach action is implemented as a separate submodule and follows the Jido.Action behavior.","ref":"Jido.Actions.Basic.html"},{"type":"module","title":"Jido.Actions.Files","doc":"A collection of file system actions for common file operations.\n\nThis module provides a set of actions for working with files and directories:\n- WriteFile: Writes content to a file with optional directory creation\n- ReadFile: Reads content from a file\n- CopyFile: Copies a file from source to destination\n- MoveFile: Moves/renames a file from source to destination\n- DeleteFile: Deletes a file or directory with optional recursive deletion\n- MakeDirectory: Creates a new directory with optional recursive creation\n- ListDirectory: Lists directory contents with optional pattern matching and recursion\n\nEach action is implemented as a separate submodule and follows the Jido.Action behavior.\nThe actions provide comprehensive file system operations with error handling and options\nlike recursive deletion, force flags, and parent directory creation.","ref":"Jido.Actions.Files.html"},{"type":"module","title":"Jido.Actions.Simplebot","doc":"A collection of actions for a simple robot simulation. These are intended to be used as an example and reference for how to write your own bot logic.\n\nThis module provides actions for:\n- Move: Simulates moving the robot to a specified location\n- Idle: Simulates the robot idling\n- DoWork: Simulates the robot performing work tasks\n- Report: Simulates the robot reporting its status\n- Recharge: Simulates recharging the robot's battery","ref":"Jido.Actions.Simplebot.html"},{"type":"module","title":"Jido.Sensors.Cron","doc":"A sensor that emits signals based on cron schedules via Quantum.\n\nBy default, the sensor uses the Quantum scheduler referenced by `:jido_quantum`,\nbut you can override the scheduler name by specifying `quantum_ref` in the sensor's\nstart options.\n\nYou can also specify a list of cron jobs to be automatically added at startup via\nthe `jobs` parameter. Each entry can be:\n- `{ ,  }` -> auto-generated job name\n- `{ ,  ,  }` -> manual job name\n\nExample Usage:\n\n    {:ok, sensor} =\n      Jido.Sensors.Cron.start_link(\n        id: \"my_cron_sensor\",\n        target: {:bus, [target: :my_bus, stream: \"some_stream\"]},\n        scheduler: Jido.Scheduler,\n        jobs: [\n          # Auto-named job\n          {~e\"* * * * * *\"e, :task1},\n          # Named job\n          {:my_job, ~e\"0 */2 * * * *\"e, :task2}\n        ]\n      )\n\n    # Add a new job later\n    :ok = Jido.Sensors.Cron.add_job(sensor, :manual_job, ~e\"* * * * *\"e, :another_task)\n\n    # Remove the job\n    :ok = Jido.Sensors.Cron.remove_job(sensor, :manual_job)\n\n    # Run a job immediately\n    :ok = Jido.Sensors.Cron.run_job(sensor, :my_job)","ref":"Jido.Sensors.Cron.html"},{"type":"function","title":"Jido.Sensors.Cron.activate_job/2","doc":"Activates a cron job by name.","ref":"Jido.Sensors.Cron.html#activate_job/2"},{"type":"function","title":"Jido.Sensors.Cron.add_job/4","doc":"Adds a new cron job with the given name, schedule, and task.\n\n`name` must be an atom.\n`schedule` is a Crontab.CronExpression (supports `~e\"\"` sigil or built manually).\n`task` can be any term you want included in the dispatched signal's data.","ref":"Jido.Sensors.Cron.html#add_job/4"},{"type":"function","title":"Jido.Sensors.Cron.category/0","doc":"Returns the configured category of the sensor.","ref":"Jido.Sensors.Cron.html#category/0"},{"type":"function","title":"Jido.Sensors.Cron.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Jido.Sensors.Cron.html#child_spec/1"},{"type":"function","title":"Jido.Sensors.Cron.deactivate_job/2","doc":"Deactivates a cron job by name.","ref":"Jido.Sensors.Cron.html#deactivate_job/2"},{"type":"function","title":"Jido.Sensors.Cron.description/0","doc":"Returns the configured description of the sensor.","ref":"Jido.Sensors.Cron.html#description/0"},{"type":"function","title":"Jido.Sensors.Cron.get_config/1","doc":"Retrieves the complete configuration of a sensor.","ref":"Jido.Sensors.Cron.html#get_config/1"},{"type":"function","title":"Parameters - Jido.Sensors.Cron.get_config/1","doc":"* `sensor` - The sensor to get configuration from. Can be a PID, atom name, or string name.","ref":"Jido.Sensors.Cron.html#get_config/1-parameters"},{"type":"function","title":"Return Values - Jido.Sensors.Cron.get_config/1","doc":"* `{:ok, config}` - The complete configuration map\n  * `{:error, reason}` - If the sensor cannot be found or accessed","ref":"Jido.Sensors.Cron.html#get_config/1-return-values"},{"type":"function","title":"Examples - Jido.Sensors.Cron.get_config/1","doc":"iex> {:ok, config} = MySensor.get_config(sensor_pid)\n    {:ok, %{id: \"sensor1\", target: {:bus, :my_bus}}}\n\n    iex> MySensor.get_config(:nonexistent_sensor)\n    {:error, :invalid_sensor}","ref":"Jido.Sensors.Cron.html#get_config/1-examples"},{"type":"function","title":"Jido.Sensors.Cron.get_config/2","doc":"Retrieves a specific configuration value from a sensor.","ref":"Jido.Sensors.Cron.html#get_config/2"},{"type":"function","title":"Parameters - Jido.Sensors.Cron.get_config/2","doc":"* `sensor` - The sensor to get configuration from. Can be a PID, atom name, or string name.\n  * `key` - The configuration key to retrieve","ref":"Jido.Sensors.Cron.html#get_config/2-parameters"},{"type":"function","title":"Return Values - Jido.Sensors.Cron.get_config/2","doc":"* `{:ok, value}` - The configuration value for the key\n  * `{:error, :not_found}` - If the key does not exist\n  * `{:error, reason}` - If the sensor cannot be found or accessed","ref":"Jido.Sensors.Cron.html#get_config/2-return-values"},{"type":"function","title":"Examples - Jido.Sensors.Cron.get_config/2","doc":"iex> {:ok, value} = MySensor.get_config(sensor_pid, :target)\n    {:ok, {:bus, :my_bus}}\n\n    iex> MySensor.get_config(sensor_pid, :nonexistent_key)\n    {:error, :not_found}","ref":"Jido.Sensors.Cron.html#get_config/2-examples"},{"type":"function","title":"Jido.Sensors.Cron.name/0","doc":"Returns the configured name of the sensor.","ref":"Jido.Sensors.Cron.html#name/0"},{"type":"function","title":"Jido.Sensors.Cron.remove_job/2","doc":"Removes a cron job by name.","ref":"Jido.Sensors.Cron.html#remove_job/2"},{"type":"function","title":"Jido.Sensors.Cron.run_job/2","doc":"Runs a job immediately (outside its normal schedule).","ref":"Jido.Sensors.Cron.html#run_job/2"},{"type":"function","title":"Jido.Sensors.Cron.schema/0","doc":"Returns the configured schema for the sensor.","ref":"Jido.Sensors.Cron.html#schema/0"},{"type":"function","title":"Jido.Sensors.Cron.set_config/2","doc":"Updates multiple configuration values for a sensor.","ref":"Jido.Sensors.Cron.html#set_config/2"},{"type":"function","title":"Parameters - Jido.Sensors.Cron.set_config/2","doc":"* `sensor` - The sensor to update. Can be a PID, atom name, or string name.\n  * `config` - A map containing the configuration key-value pairs to update","ref":"Jido.Sensors.Cron.html#set_config/2-parameters"},{"type":"function","title":"Return Values - Jido.Sensors.Cron.set_config/2","doc":"* `:ok` - The configuration was updated successfully\n  * `{:error, reason}` - If the sensor cannot be found or accessed","ref":"Jido.Sensors.Cron.html#set_config/2-return-values"},{"type":"function","title":"Examples - Jido.Sensors.Cron.set_config/2","doc":"iex> MySensor.set_config(sensor_pid, %{key1: \"value1\", key2: \"value2\"})\n    :ok\n\n    iex> MySensor.set_config(:nonexistent_sensor, %{key: \"value\"})\n    {:error, :invalid_sensor}","ref":"Jido.Sensors.Cron.html#set_config/2-examples"},{"type":"function","title":"Jido.Sensors.Cron.set_config/3","doc":"Updates a single configuration value for a sensor.","ref":"Jido.Sensors.Cron.html#set_config/3"},{"type":"function","title":"Parameters - Jido.Sensors.Cron.set_config/3","doc":"* `sensor` - The sensor to update. Can be a PID, atom name, or string name.\n  * `key` - The configuration key to update\n  * `value` - The new value to set","ref":"Jido.Sensors.Cron.html#set_config/3-parameters"},{"type":"function","title":"Return Values - Jido.Sensors.Cron.set_config/3","doc":"* `:ok` - The configuration was updated successfully\n  * `{:error, reason}` - If the sensor cannot be found or accessed","ref":"Jido.Sensors.Cron.html#set_config/3-return-values"},{"type":"function","title":"Examples - Jido.Sensors.Cron.set_config/3","doc":"iex> MySensor.set_config(sensor_pid, :some_key, \"new_value\")\n    :ok\n\n    iex> MySensor.set_config(:nonexistent_sensor, :key, \"value\")\n    {:error, :invalid_sensor}","ref":"Jido.Sensors.Cron.html#set_config/3-examples"},{"type":"function","title":"Jido.Sensors.Cron.start_link/1","doc":"Starts a new Sensor process.","ref":"Jido.Sensors.Cron.html#start_link/1"},{"type":"function","title":"Options - Jido.Sensors.Cron.start_link/1","doc":"* `:id` (`t:String.t/0`) - Unique identifier for the sensor instance\n\n* `:target` - Required. Target for signal delivery. Can be a single dispatch config tuple or a keyword list of named configurations.\n\n* `:retain_last` (`t:pos_integer/0`) - Number of last values to retain The default value is `10`.\n\n* `:scheduler` (`t:atom/0`) - Reference to the Quantum scheduler module to use The default value is `Jido.Scheduler`.\n\n* `:jobs` (list of `t:term/0`) - A list of cron jobs to be added at sensor startup. Each entry can be:\n    - {cron_expr, task}\n    - {name, cron_expr, task} The default value is `[]`.","ref":"Jido.Sensors.Cron.html#start_link/1-options"},{"type":"function","title":"Return Values - Jido.Sensors.Cron.start_link/1","doc":"* `{:ok, pid}` - The sensor was started successfully\n  * `{:error, reason}` - The sensor failed to start","ref":"Jido.Sensors.Cron.html#start_link/1-return-values"},{"type":"function","title":"Examples - Jido.Sensors.Cron.start_link/1","doc":"iex> MySensor.start_link(id: \"sensor1\", target: {:bus, :my_bus})\n    {:ok, #PID<0.123.0>}\n\n    iex> MySensor.start_link(id: \"sensor1\", target: {:invalid, :target})\n    {:error, \"invalid target specification\"}","ref":"Jido.Sensors.Cron.html#start_link/1-examples"},{"type":"function","title":"Jido.Sensors.Cron.tags/0","doc":"Returns the configured tags for the sensor.","ref":"Jido.Sensors.Cron.html#tags/0"},{"type":"function","title":"Jido.Sensors.Cron.to_json/0","doc":"Converts the sensor metadata to a JSON-compatible map.\n\nReturns a map containing the sensor's name, description, category, tags,\nversion and schema configuration.","ref":"Jido.Sensors.Cron.html#to_json/0"},{"type":"function","title":"Example - Jido.Sensors.Cron.to_json/0","doc":"iex> MySensor.to_json()\n    %{\n      name: \"my_sensor\",\n      description: \"A test sensor\",\n      category: :test,\n      tags: [:test, :example],\n      vsn: \"1.0.0\",\n      schema: []\n    }","ref":"Jido.Sensors.Cron.html#to_json/0-example"},{"type":"function","title":"Jido.Sensors.Cron.vsn/0","doc":"Returns the configured version of the sensor.","ref":"Jido.Sensors.Cron.html#vsn/0"},{"type":"module","title":"Jido.Sensors.Heartbeat","doc":"A sensor that emits heartbeat signals at configurable intervals.","ref":"Jido.Sensors.Heartbeat.html"},{"type":"function","title":"Jido.Sensors.Heartbeat.category/0","doc":"Returns the configured category of the sensor.","ref":"Jido.Sensors.Heartbeat.html#category/0"},{"type":"function","title":"Jido.Sensors.Heartbeat.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Jido.Sensors.Heartbeat.html#child_spec/1"},{"type":"function","title":"Jido.Sensors.Heartbeat.description/0","doc":"Returns the configured description of the sensor.","ref":"Jido.Sensors.Heartbeat.html#description/0"},{"type":"function","title":"Jido.Sensors.Heartbeat.get_config/1","doc":"Retrieves the complete configuration of a sensor.","ref":"Jido.Sensors.Heartbeat.html#get_config/1"},{"type":"function","title":"Parameters - Jido.Sensors.Heartbeat.get_config/1","doc":"* `sensor` - The sensor to get configuration from. Can be a PID, atom name, or string name.","ref":"Jido.Sensors.Heartbeat.html#get_config/1-parameters"},{"type":"function","title":"Return Values - Jido.Sensors.Heartbeat.get_config/1","doc":"* `{:ok, config}` - The complete configuration map\n  * `{:error, reason}` - If the sensor cannot be found or accessed","ref":"Jido.Sensors.Heartbeat.html#get_config/1-return-values"},{"type":"function","title":"Examples - Jido.Sensors.Heartbeat.get_config/1","doc":"iex> {:ok, config} = MySensor.get_config(sensor_pid)\n    {:ok, %{id: \"sensor1\", target: {:bus, :my_bus}}}\n\n    iex> MySensor.get_config(:nonexistent_sensor)\n    {:error, :invalid_sensor}","ref":"Jido.Sensors.Heartbeat.html#get_config/1-examples"},{"type":"function","title":"Jido.Sensors.Heartbeat.get_config/2","doc":"Retrieves a specific configuration value from a sensor.","ref":"Jido.Sensors.Heartbeat.html#get_config/2"},{"type":"function","title":"Parameters - Jido.Sensors.Heartbeat.get_config/2","doc":"* `sensor` - The sensor to get configuration from. Can be a PID, atom name, or string name.\n  * `key` - The configuration key to retrieve","ref":"Jido.Sensors.Heartbeat.html#get_config/2-parameters"},{"type":"function","title":"Return Values - Jido.Sensors.Heartbeat.get_config/2","doc":"* `{:ok, value}` - The configuration value for the key\n  * `{:error, :not_found}` - If the key does not exist\n  * `{:error, reason}` - If the sensor cannot be found or accessed","ref":"Jido.Sensors.Heartbeat.html#get_config/2-return-values"},{"type":"function","title":"Examples - Jido.Sensors.Heartbeat.get_config/2","doc":"iex> {:ok, value} = MySensor.get_config(sensor_pid, :target)\n    {:ok, {:bus, :my_bus}}\n\n    iex> MySensor.get_config(sensor_pid, :nonexistent_key)\n    {:error, :not_found}","ref":"Jido.Sensors.Heartbeat.html#get_config/2-examples"},{"type":"function","title":"Jido.Sensors.Heartbeat.name/0","doc":"Returns the configured name of the sensor.","ref":"Jido.Sensors.Heartbeat.html#name/0"},{"type":"function","title":"Jido.Sensors.Heartbeat.schema/0","doc":"Returns the configured schema for the sensor.","ref":"Jido.Sensors.Heartbeat.html#schema/0"},{"type":"function","title":"Jido.Sensors.Heartbeat.set_config/2","doc":"Updates multiple configuration values for a sensor.","ref":"Jido.Sensors.Heartbeat.html#set_config/2"},{"type":"function","title":"Parameters - Jido.Sensors.Heartbeat.set_config/2","doc":"* `sensor` - The sensor to update. Can be a PID, atom name, or string name.\n  * `config` - A map containing the configuration key-value pairs to update","ref":"Jido.Sensors.Heartbeat.html#set_config/2-parameters"},{"type":"function","title":"Return Values - Jido.Sensors.Heartbeat.set_config/2","doc":"* `:ok` - The configuration was updated successfully\n  * `{:error, reason}` - If the sensor cannot be found or accessed","ref":"Jido.Sensors.Heartbeat.html#set_config/2-return-values"},{"type":"function","title":"Examples - Jido.Sensors.Heartbeat.set_config/2","doc":"iex> MySensor.set_config(sensor_pid, %{key1: \"value1\", key2: \"value2\"})\n    :ok\n\n    iex> MySensor.set_config(:nonexistent_sensor, %{key: \"value\"})\n    {:error, :invalid_sensor}","ref":"Jido.Sensors.Heartbeat.html#set_config/2-examples"},{"type":"function","title":"Jido.Sensors.Heartbeat.set_config/3","doc":"Updates a single configuration value for a sensor.","ref":"Jido.Sensors.Heartbeat.html#set_config/3"},{"type":"function","title":"Parameters - Jido.Sensors.Heartbeat.set_config/3","doc":"* `sensor` - The sensor to update. Can be a PID, atom name, or string name.\n  * `key` - The configuration key to update\n  * `value` - The new value to set","ref":"Jido.Sensors.Heartbeat.html#set_config/3-parameters"},{"type":"function","title":"Return Values - Jido.Sensors.Heartbeat.set_config/3","doc":"* `:ok` - The configuration was updated successfully\n  * `{:error, reason}` - If the sensor cannot be found or accessed","ref":"Jido.Sensors.Heartbeat.html#set_config/3-return-values"},{"type":"function","title":"Examples - Jido.Sensors.Heartbeat.set_config/3","doc":"iex> MySensor.set_config(sensor_pid, :some_key, \"new_value\")\n    :ok\n\n    iex> MySensor.set_config(:nonexistent_sensor, :key, \"value\")\n    {:error, :invalid_sensor}","ref":"Jido.Sensors.Heartbeat.html#set_config/3-examples"},{"type":"function","title":"Jido.Sensors.Heartbeat.start_link/1","doc":"Starts a new Sensor process.","ref":"Jido.Sensors.Heartbeat.html#start_link/1"},{"type":"function","title":"Options - Jido.Sensors.Heartbeat.start_link/1","doc":"* `:id` (`t:String.t/0`) - Unique identifier for the sensor instance\n\n* `:target` - Required. Target for signal delivery. Can be a single dispatch config tuple or a keyword list of named configurations.\n\n* `:retain_last` (`t:pos_integer/0`) - Number of last values to retain The default value is `10`.\n\n* `:interval` (`t:pos_integer/0`) - Interval between heartbeats in milliseconds The default value is `5000`.\n\n* `:message` (`t:String.t/0`) - Message to include in heartbeat signal The default value is `\"heartbeat\"`.","ref":"Jido.Sensors.Heartbeat.html#start_link/1-options"},{"type":"function","title":"Return Values - Jido.Sensors.Heartbeat.start_link/1","doc":"* `{:ok, pid}` - The sensor was started successfully\n  * `{:error, reason}` - The sensor failed to start","ref":"Jido.Sensors.Heartbeat.html#start_link/1-return-values"},{"type":"function","title":"Examples - Jido.Sensors.Heartbeat.start_link/1","doc":"iex> MySensor.start_link(id: \"sensor1\", target: {:bus, :my_bus})\n    {:ok, #PID<0.123.0>}\n\n    iex> MySensor.start_link(id: \"sensor1\", target: {:invalid, :target})\n    {:error, \"invalid target specification\"}","ref":"Jido.Sensors.Heartbeat.html#start_link/1-examples"},{"type":"function","title":"Jido.Sensors.Heartbeat.tags/0","doc":"Returns the configured tags for the sensor.","ref":"Jido.Sensors.Heartbeat.html#tags/0"},{"type":"function","title":"Jido.Sensors.Heartbeat.to_json/0","doc":"Converts the sensor metadata to a JSON-compatible map.\n\nReturns a map containing the sensor's name, description, category, tags,\nversion and schema configuration.","ref":"Jido.Sensors.Heartbeat.html#to_json/0"},{"type":"function","title":"Example - Jido.Sensors.Heartbeat.to_json/0","doc":"iex> MySensor.to_json()\n    %{\n      name: \"my_sensor\",\n      description: \"A test sensor\",\n      category: :test,\n      tags: [:test, :example],\n      vsn: \"1.0.0\",\n      schema: []\n    }","ref":"Jido.Sensors.Heartbeat.html#to_json/0-example"},{"type":"function","title":"Jido.Sensors.Heartbeat.vsn/0","doc":"Returns the configured version of the sensor.","ref":"Jido.Sensors.Heartbeat.html#vsn/0"},{"type":"module","title":"Jido.Discovery","doc":"The Discovery module is Jido's component registry system, providing efficient caching and lookup\nof system components like Actions, Sensors, Agents, and Skills. Think of it as a \"service registry\"\nthat helps different parts of your system find and interact with each other.","ref":"Jido.Discovery.html"},{"type":"module","title":"Core Concepts - Jido.Discovery","doc":"","ref":"Jido.Discovery.html#module-core-concepts"},{"type":"module","title":"Component Discovery - Jido.Discovery","doc":"Discovery works by scanning all loaded applications for components that implement Jido's\nmetadata protocols. It automatically finds and indexes:\n\n- **Actions** - Discrete units of work\n- **Sensors** - Event monitoring components\n- **Agents** - Autonomous workers\n- **Skills** - Reusable capability packs\n- **Demos** - Example implementations - used in the Jido Workbench (https://github.com/agentjido/jido_workbench)\n\nThe module uses Erlang's `:persistent_term` for optimal lookup performance","ref":"Jido.Discovery.html#module-component-discovery"},{"type":"module","title":"Component Metadata - Jido.Discovery","doc":"Each discovered component includes the following metadata:\n\n```elixir\n%{\n  module: MyApp.CoolAction,        # The actual module\n  name: \"cool_action\",            # Human-readable name\n  description: \"Does cool stuff\", # What it does\n  slug: \"abc123de\",              # Unique identifier\n  category: :utility,            # Broad classification\n  tags: [:cool, :stuff]          # Searchable tags\n}\n```","ref":"Jido.Discovery.html#module-component-metadata"},{"type":"module","title":"Usage Examples - Jido.Discovery","doc":"","ref":"Jido.Discovery.html#module-usage-examples"},{"type":"module","title":"Basic Component Lookup - Jido.Discovery","doc":"Find components by their unique slugs:\n\n```elixir\n# Find a specific action\ncase Discovery.get_action_by_slug(\"abc123de\") do\n  %{module: module} ->\n    # Use the action module\n    {:ok, result} = module.run()\n  nil ->\n    # Handle missing action\nend\n\n# Find a sensor\nsensor = Discovery.get_sensor_by_slug(\"def456gh\")\n```","ref":"Jido.Discovery.html#module-basic-component-lookup"},{"type":"module","title":"Filtered Component Lists - Jido.Discovery","doc":"Get filtered lists of components:\n\n```elixir\n# List all monitoring sensors\nsensors = Discovery.list_sensors(\n  category: :monitoring,\n  tag: :metrics\n)\n\n# Get the first 10 utility actions\nactions = Discovery.list_actions(\n  category: :utility,\n  limit: 10\n)\n\n# Search agents by name\nagents = Discovery.list_agents(\n  name: \"processor\",\n  offset: 5,\n  limit: 5\n)\n```","ref":"Jido.Discovery.html#module-filtered-component-lists"},{"type":"module","title":"Cache Management - Jido.Discovery","doc":"Control the discovery cache lifecycle:\n\n```elixir\n# Initialize cache (usually done at startup)\n:ok = Discovery.init()\n\n# Force cache refresh if needed\n:ok = Discovery.refresh()\n\n# Check last update time\n{:ok, last_updated} = Discovery.last_updated()\n```","ref":"Jido.Discovery.html#module-cache-management"},{"type":"module","title":"Component Registration - Jido.Discovery","doc":"Components are automatically discovered when they:\n1. Are loaded in any application in the system\n2. Implement the appropriate metadata callback\n3. Include required metadata fields\n\nExample component:\n```elixir\ndefmodule MyApp.CoolAction do\n  use Jido.Action,\n    name: \"cool_action\",\n    description: \"Does cool stuff\",\n    category: :utility,\n    tags: [:cool, :stuff]\n\n  # Metadata is automatically generated from use params\n  # def __action_metadata__ do\n  #   %{\n  #     name: \"cool_action\",\n  #     description: \"Does cool stuff\",\n  #     category: :utility,\n  #     tags: [:cool, :stuff]\n  #   }\n  # end\nend\n```\n```","ref":"Jido.Discovery.html#module-component-registration"},{"type":"module","title":"Cache Structure - Jido.Discovery","doc":"The discovery cache maintains separate collections for each component type:\n\n```elixir\n%{\n  version: \"1.0\",              # Cache format version\n  last_updated: ~U[...],       # Last refresh timestamp\n  actions: [...],              # List of actions\n  sensors: [...],              # List of sensors\n  agents: [...],               # List of agents\n  skills: [...],              # List of skills\n  demos: [...]                # List of demos\n}\n```","ref":"Jido.Discovery.html#module-cache-structure"},{"type":"module","title":"Filtering Options - Jido.Discovery","doc":"All list functions support these filters:\n\n- `:limit` - Maximum results to return\n- `:offset` - Results to skip (pagination)\n- `:name` - Filter by name (partial match)\n- `:description` - Filter by description (partial match)\n- `:category` - Filter by category (exact match)\n- `:tag` - Filter by tag (must have exact tag)","ref":"Jido.Discovery.html#module-filtering-options"},{"type":"module","title":"Common Patterns - Jido.Discovery","doc":"","ref":"Jido.Discovery.html#module-common-patterns"},{"type":"module","title":"1. Component Lookup with Fallback - Jido.Discovery","doc":"```elixir\ndef find_action(slug, fallback \\ nil) do\n  case Discovery.get_action_by_slug(slug) do\n    %{module: module} -> {:ok, module}\n    nil -> {:ok, fallback}\n  end\nend\n```","ref":"Jido.Discovery.html#module-1-component-lookup-with-fallback"},{"type":"module","title":"2. Category-Based Component Loading - Jido.Discovery","doc":"```elixir\ndef load_monitoring_sensors do\n  Discovery.list_sensors(\n    category: :monitoring,\n    tag: :active\n  )\nend\n```","ref":"Jido.Discovery.html#module-2-category-based-component-loading"},{"type":"module","title":"3. Paginated Search - Jido.Discovery","doc":"```elixir\ndef search_agents(term, page, per_page) do\n  Discovery.list_agents(\n    name: term,\n    offset: (page - 1) * per_page,\n    limit: per_page\n  )\nend\n```","ref":"Jido.Discovery.html#module-3-paginated-search"},{"type":"module","title":"Important Notes - Jido.Discovery","doc":"- Cache is shared across all processes\n- Components must be loaded before discovery\n- Metadata changes require cache refresh\n- Slug generation is deterministic\n- Filter options are additive (AND logic)","ref":"Jido.Discovery.html#module-important-notes"},{"type":"module","title":"See Also - Jido.Discovery","doc":"- `Jido.Action` - Action component behavior\n- `Jido.Sensor` - Sensor component behavior\n- `Jido.Agent` - Agent component behavior\n- `Jido.Skill` - Skill component behavior","ref":"Jido.Discovery.html#module-see-also"},{"type":"type","title":"Jido.Discovery.cache_entry/0","doc":"","ref":"Jido.Discovery.html#t:cache_entry/0"},{"type":"type","title":"Jido.Discovery.component_metadata/0","doc":"","ref":"Jido.Discovery.html#t:component_metadata/0"},{"type":"type","title":"Jido.Discovery.component_type/0","doc":"","ref":"Jido.Discovery.html#t:component_type/0"},{"type":"function","title":"Jido.Discovery.get_action_by_slug/1","doc":"Retrieves an Action by its slug.","ref":"Jido.Discovery.html#get_action_by_slug/1"},{"type":"function","title":"Parameters - Jido.Discovery.get_action_by_slug/1","doc":"- `slug`: A string representing the unique identifier of the Action.","ref":"Jido.Discovery.html#get_action_by_slug/1-parameters"},{"type":"function","title":"Returns - Jido.Discovery.get_action_by_slug/1","doc":"The Action metadata if found, otherwise `nil`.","ref":"Jido.Discovery.html#get_action_by_slug/1-returns"},{"type":"function","title":"Examples - Jido.Discovery.get_action_by_slug/1","doc":"iex> Jido.get_action_by_slug(\"abc123de\")\n    %{module: MyApp.SomeAction, name: \"some_action\", description: \"Does something\", slug: \"abc123de\"}\n\n    iex> Jido.get_action_by_slug(\"nonexistent\")\n    nil","ref":"Jido.Discovery.html#get_action_by_slug/1-examples"},{"type":"function","title":"Jido.Discovery.get_agent_by_slug/1","doc":"Retrieves an Agent by its slug.","ref":"Jido.Discovery.html#get_agent_by_slug/1"},{"type":"function","title":"Parameters - Jido.Discovery.get_agent_by_slug/1","doc":"- `slug`: A string representing the unique identifier of the Agent.","ref":"Jido.Discovery.html#get_agent_by_slug/1-parameters"},{"type":"function","title":"Returns - Jido.Discovery.get_agent_by_slug/1","doc":"The Agent metadata if found, otherwise `nil`.","ref":"Jido.Discovery.html#get_agent_by_slug/1-returns"},{"type":"function","title":"Examples - Jido.Discovery.get_agent_by_slug/1","doc":"iex> Jido.get_agent_by_slug(\"ghi789jk\")\n    %{module: MyApp.SomeAgent, name: \"some_agent\", description: \"Represents an agent\", slug: \"ghi789jk\"}\n\n    iex> Jido.get_agent_by_slug(\"nonexistent\")\n    nil","ref":"Jido.Discovery.html#get_agent_by_slug/1-examples"},{"type":"function","title":"Jido.Discovery.get_demo_by_slug/1","doc":"Retrieves a Demo by its slug.","ref":"Jido.Discovery.html#get_demo_by_slug/1"},{"type":"function","title":"Parameters - Jido.Discovery.get_demo_by_slug/1","doc":"- `slug`: A string representing the unique identifier of the Demo.","ref":"Jido.Discovery.html#get_demo_by_slug/1-parameters"},{"type":"function","title":"Returns - Jido.Discovery.get_demo_by_slug/1","doc":"The Demo metadata if found, otherwise `nil`.","ref":"Jido.Discovery.html#get_demo_by_slug/1-returns"},{"type":"function","title":"Examples - Jido.Discovery.get_demo_by_slug/1","doc":"iex> Jido.get_demo_by_slug(\"mno345pq\")\n    %{module: MyApp.SomeDemo, name: \"some_demo\", description: \"Demonstrates something\", slug: \"mno345pq\"}\n\n    iex> Jido.get_demo_by_slug(\"nonexistent\")\n    nil","ref":"Jido.Discovery.html#get_demo_by_slug/1-examples"},{"type":"function","title":"Jido.Discovery.get_sensor_by_slug/1","doc":"Retrieves a Sensor by its slug.","ref":"Jido.Discovery.html#get_sensor_by_slug/1"},{"type":"function","title":"Parameters - Jido.Discovery.get_sensor_by_slug/1","doc":"- `slug`: A string representing the unique identifier of the Sensor.","ref":"Jido.Discovery.html#get_sensor_by_slug/1-parameters"},{"type":"function","title":"Returns - Jido.Discovery.get_sensor_by_slug/1","doc":"The Sensor metadata if found, otherwise `nil`.","ref":"Jido.Discovery.html#get_sensor_by_slug/1-returns"},{"type":"function","title":"Examples - Jido.Discovery.get_sensor_by_slug/1","doc":"iex> Jido.get_sensor_by_slug(\"def456gh\")\n    %{module: MyApp.SomeSensor, name: \"some_sensor\", description: \"Monitors something\", slug: \"def456gh\"}\n\n    iex> Jido.get_sensor_by_slug(\"nonexistent\")\n    nil","ref":"Jido.Discovery.html#get_sensor_by_slug/1-examples"},{"type":"function","title":"Jido.Discovery.get_skill_by_slug/1","doc":"Retrieves a Skill by its slug.","ref":"Jido.Discovery.html#get_skill_by_slug/1"},{"type":"function","title":"Parameters - Jido.Discovery.get_skill_by_slug/1","doc":"- `slug`: A string representing the unique identifier of the Skill.","ref":"Jido.Discovery.html#get_skill_by_slug/1-parameters"},{"type":"function","title":"Returns - Jido.Discovery.get_skill_by_slug/1","doc":"The Skill metadata if found, otherwise `nil`.","ref":"Jido.Discovery.html#get_skill_by_slug/1-returns"},{"type":"function","title":"Examples - Jido.Discovery.get_skill_by_slug/1","doc":"iex> Jido.get_skill_by_slug(\"jkl012mn\")\n    %{module: MyApp.SomeSkill, name: \"some_skill\", description: \"Provides some capability\", slug: \"jkl012mn\"}\n\n    iex> Jido.get_skill_by_slug(\"nonexistent\")\n    nil","ref":"Jido.Discovery.html#get_skill_by_slug/1-examples"},{"type":"function","title":"Jido.Discovery.init/0","doc":"Initializes the discovery cache. Should be called during application startup.","ref":"Jido.Discovery.html#init/0"},{"type":"function","title":"Returns - Jido.Discovery.init/0","doc":"- `:ok` if cache was initialized successfully\n- `{:error, reason}` if initialization failed","ref":"Jido.Discovery.html#init/0-returns"},{"type":"function","title":"Jido.Discovery.last_updated/0","doc":"Gets the last time the cache was updated.","ref":"Jido.Discovery.html#last_updated/0"},{"type":"function","title":"Returns - Jido.Discovery.last_updated/0","doc":"- `{:ok, datetime}` with the last update time\n- `{:error, :not_initialized}` if cache hasn't been initialized","ref":"Jido.Discovery.html#last_updated/0-returns"},{"type":"function","title":"Jido.Discovery.list_actions/1","doc":"Lists all Actions with optional filtering and pagination.","ref":"Jido.Discovery.html#list_actions/1"},{"type":"function","title":"Parameters - Jido.Discovery.list_actions/1","doc":"- `opts`: A keyword list of options for filtering and pagination. Available options:\n  - `:limit`: Maximum number of results to return.\n  - `:offset`: Number of results to skip before starting to return.\n  - `:name`: Filter Actions by name (partial match).\n  - `:description`: Filter Actions by description (partial match).\n  - `:category`: Filter Actions by category (exact match).\n  - `:tag`: Filter Actions by tag (must have the exact tag).","ref":"Jido.Discovery.html#list_actions/1-parameters"},{"type":"function","title":"Returns - Jido.Discovery.list_actions/1","doc":"A list of Action metadata.","ref":"Jido.Discovery.html#list_actions/1-returns"},{"type":"function","title":"Examples - Jido.Discovery.list_actions/1","doc":"iex> Jido.list_actions(limit: 10, offset: 5, category: :utility)\n    [%{module: MyApp.SomeAction, name: \"some_action\", description: \"Does something\", slug: \"abc123de\", category: :utility}]","ref":"Jido.Discovery.html#list_actions/1-examples"},{"type":"function","title":"Jido.Discovery.list_agents/1","doc":"Lists all Agents with optional filtering and pagination.","ref":"Jido.Discovery.html#list_agents/1"},{"type":"function","title":"Parameters - Jido.Discovery.list_agents/1","doc":"- `opts`: A keyword list of options for filtering and pagination. Available options:\n  - `:limit`: Maximum number of results to return.\n  - `:offset`: Number of results to skip before starting to return.\n  - `:name`: Filter Agents by name (partial match).\n  - `:description`: Filter Agents by description (partial match).\n  - `:category`: Filter Agents by category (exact match).\n  - `:tag`: Filter Agents by tag (must have the exact tag).","ref":"Jido.Discovery.html#list_agents/1-parameters"},{"type":"function","title":"Returns - Jido.Discovery.list_agents/1","doc":"A list of Agent metadata.","ref":"Jido.Discovery.html#list_agents/1-returns"},{"type":"function","title":"Examples - Jido.Discovery.list_agents/1","doc":"iex> Jido.list_agents(limit: 10, offset: 5, category: :business)\n    [%{module: MyApp.SomeAgent, name: \"some_agent\", description: \"Represents an agent\", slug: \"ghi789jk\", category: :business}]","ref":"Jido.Discovery.html#list_agents/1-examples"},{"type":"function","title":"Jido.Discovery.list_demos/1","doc":"Lists all Demos with optional filtering and pagination.","ref":"Jido.Discovery.html#list_demos/1"},{"type":"function","title":"Parameters - Jido.Discovery.list_demos/1","doc":"- `opts`: A keyword list of options for filtering and pagination. Available options:\n  - `:limit`: Maximum number of results to return.\n  - `:offset`: Number of results to skip before starting to return.\n  - `:name`: Filter Demos by name (partial match).\n  - `:description`: Filter Demos by description (partial match).\n  - `:category`: Filter Demos by category (exact match).\n  - `:tag`: Filter Demos by tag (must have the exact tag).","ref":"Jido.Discovery.html#list_demos/1-parameters"},{"type":"function","title":"Returns - Jido.Discovery.list_demos/1","doc":"A list of Demo metadata.","ref":"Jido.Discovery.html#list_demos/1-returns"},{"type":"function","title":"Examples - Jido.Discovery.list_demos/1","doc":"iex> Jido.list_demos(limit: 10, offset: 5, category: :example)\n    [%{module: MyApp.SomeDemo, name: \"some_demo\", description: \"Demonstrates something\", slug: \"mno345pq\", category: :example}]","ref":"Jido.Discovery.html#list_demos/1-examples"},{"type":"function","title":"Jido.Discovery.list_sensors/1","doc":"Lists all Sensors with optional filtering and pagination.","ref":"Jido.Discovery.html#list_sensors/1"},{"type":"function","title":"Parameters - Jido.Discovery.list_sensors/1","doc":"- `opts`: A keyword list of options for filtering and pagination. Available options:\n  - `:limit`: Maximum number of results to return.\n  - `:offset`: Number of results to skip before starting to return.\n  - `:name`: Filter Sensors by name (partial match).\n  - `:description`: Filter Sensors by description (partial match).\n  - `:category`: Filter Sensors by category (exact match).\n  - `:tag`: Filter Sensors by tag (must have the exact tag).","ref":"Jido.Discovery.html#list_sensors/1-parameters"},{"type":"function","title":"Returns - Jido.Discovery.list_sensors/1","doc":"A list of Sensor metadata.","ref":"Jido.Discovery.html#list_sensors/1-returns"},{"type":"function","title":"Examples - Jido.Discovery.list_sensors/1","doc":"iex> Jido.list_sensors(limit: 10, offset: 5, category: :monitoring)\n    [%{module: MyApp.SomeSensor, name: \"some_sensor\", description: \"Monitors something\", slug: \"def456gh\", category: :monitoring}]","ref":"Jido.Discovery.html#list_sensors/1-examples"},{"type":"function","title":"Jido.Discovery.list_skills/1","doc":"Lists all Skills with optional filtering and pagination.","ref":"Jido.Discovery.html#list_skills/1"},{"type":"function","title":"Parameters - Jido.Discovery.list_skills/1","doc":"- `opts`: A keyword list of options for filtering and pagination. Available options:\n  - `:limit`: Maximum number of results to return.\n  - `:offset`: Number of results to skip before starting to return.\n  - `:name`: Filter Skills by name (partial match).\n  - `:description`: Filter Skills by description (partial match).\n  - `:category`: Filter Skills by category (exact match).\n  - `:tag`: Filter Skills by tag (must have the exact tag).","ref":"Jido.Discovery.html#list_skills/1-parameters"},{"type":"function","title":"Returns - Jido.Discovery.list_skills/1","doc":"A list of Skill metadata.","ref":"Jido.Discovery.html#list_skills/1-returns"},{"type":"function","title":"Examples - Jido.Discovery.list_skills/1","doc":"iex> Jido.list_skills(limit: 10, offset: 5, category: :capability)\n    [%{module: MyApp.SomeSkill, name: \"some_skill\", description: \"Provides some capability\", slug: \"jkl012mn\", category: :capability}]","ref":"Jido.Discovery.html#list_skills/1-examples"},{"type":"function","title":"Jido.Discovery.refresh/0","doc":"Forces a refresh of the discovery cache.","ref":"Jido.Discovery.html#refresh/0"},{"type":"function","title":"Returns - Jido.Discovery.refresh/0","doc":"- `:ok` if cache was refreshed successfully\n- `{:error, reason}` if refresh failed","ref":"Jido.Discovery.html#refresh/0-returns"},{"type":"module","title":"Jido.Error","doc":"Defines error structures and helper functions for Jido\n\nThis module provides a standardized way to create and handle errors within the Jido system.\nIt offers a set of predefined error types and functions to create, manipulate, and convert\nerror structures consistently across the application.\n\n> Why not use Exceptions?\n>\n> Jido is designed to be a functional system, strictly adhering to the use of result tuples.\n> This approach provides several benefits:\n>\n> 1. Consistent error handling: By using `{:ok, result}` or `{:error, reason}` tuples,\n>    we ensure a uniform way of handling success and failure cases throughout the system.\n>\n> 2. Composability: Monadic workflows can be easily chained together, allowing for\n>    cleaner and more maintainable code.\n>\n> 3. Explicit error paths: The use of result tuples makes error cases explicit,\n>    reducing the likelihood of unhandled errors.\n>\n> 4. No silent failures: Unlike exceptions, which can be silently caught and ignored,\n>    result tuples require explicit handling of both success and error cases.\n>\n> 5. Better testability: Monadic workflows are easier to test, as both success and\n>    error paths can be explicitly verified.\n>\n> By using this approach instead of exceptions, we gain more control over the flow of our\n> workflows and ensure that errors are handled consistently across the entire system.","ref":"Jido.Error.html"},{"type":"module","title":"Usage - Jido.Error","doc":"Use this module to create specific error types when exceptions occur in your Jido workflows.\nThis allows for consistent error handling and reporting throughout the system.\n\nExample:\n\n    defmodule MyWorkflow do\n      alias Jido.Error\n\n      def run(params) do\n        case validate(params) do\n          :ok -> perform_workflow(params)\n          {:error, reason} -> Error.validation_error(\"Invalid parameters\")\n        end\n      end\n    end","ref":"Jido.Error.html#module-usage"},{"type":"function","title":"Jido.Error.bad_request/3","doc":"Creates a new bad request error.\n\nUse this when the client sends an invalid or malformed request.","ref":"Jido.Error.html#bad_request/3"},{"type":"function","title":"Parameters - Jido.Error.bad_request/3","doc":"- `message`: A string describing the error.\n- `details`: (optional) A map containing additional error details.\n- `stacktrace`: (optional) The stacktrace at the point of error.","ref":"Jido.Error.html#bad_request/3-parameters"},{"type":"function","title":"Example - Jido.Error.bad_request/3","doc":"iex> Jido.Error.bad_request(\"Missing required parameter 'user_id'\")\n    %Jido.Error{\n      type: :bad_request,\n      message: \"Missing required parameter 'user_id'\",\n      details: nil,\n      stacktrace: [...]\n    }","ref":"Jido.Error.html#bad_request/3-example"},{"type":"function","title":"Jido.Error.capture_stacktrace/0","doc":"Captures the current stacktrace.\n\nThis function is useful when you want to capture the stacktrace at a specific point\nin your code, rather than at the point where the error is created. It drops the first\ntwo entries from the stacktrace to remove internal function calls related to this module.","ref":"Jido.Error.html#capture_stacktrace/0"},{"type":"function","title":"Returns - Jido.Error.capture_stacktrace/0","doc":"The current stacktrace as a list.","ref":"Jido.Error.html#capture_stacktrace/0-returns"},{"type":"function","title":"Example - Jido.Error.capture_stacktrace/0","doc":"iex> stacktrace = Jido.Error.capture_stacktrace()\n    iex> is_list(stacktrace)\n    true","ref":"Jido.Error.html#capture_stacktrace/0-example"},{"type":"function","title":"Jido.Error.compensation_error/3","doc":"Creates a new compensation error with details about the original error and compensation attempt.","ref":"Jido.Error.html#compensation_error/3"},{"type":"function","title":"Parameters - Jido.Error.compensation_error/3","doc":"- `original_error`: The error that triggered compensation\n- `details`: Optional map containing:\n  - `:compensated` - Boolean indicating if compensation succeeded\n  - `:compensation_result` - Result from successful compensation\n  - `:compensation_error` - Error from failed compensation\n- `stacktrace`: Optional stacktrace for debugging","ref":"Jido.Error.html#compensation_error/3-parameters"},{"type":"function","title":"Examples - Jido.Error.compensation_error/3","doc":"iex> original_error = Jido.Error.execution_error(\"Failed to process payment\")\n    iex> Jido.Error.compensation_error(original_error, %{\n    ...>   compensated: true,\n    ...>   compensation_result: %{refund_id: \"ref_123\"}\n    ...> })\n    %Jido.Error{\n      type: :compensation_error,\n      message: \"Compensation completed for: Failed to process payment\",\n      details: %{\n        compensated: true,\n        compensation_result: %{refund_id: \"ref_123\"},\n        original_error: %Jido.Error{...}\n      }\n    }\n\n    iex> # For failed compensation:\n    iex> Jido.Error.compensation_error(original_error, %{\n    ...>   compensated: false,\n    ...>   compensation_error: \"Refund failed\"\n    ...> })","ref":"Jido.Error.html#compensation_error/3-examples"},{"type":"function","title":"Jido.Error.config_error/3","doc":"Creates a new config error.\n\nUse this when there's an issue with the system or workflow configuration.","ref":"Jido.Error.html#config_error/3"},{"type":"function","title":"Parameters - Jido.Error.config_error/3","doc":"- `message`: A string describing the configuration error.\n- `details`: (optional) A map containing additional error details.\n- `stacktrace`: (optional) The stacktrace at the point of error.","ref":"Jido.Error.html#config_error/3-parameters"},{"type":"function","title":"Example - Jido.Error.config_error/3","doc":"iex> Jido.Error.config_error(\"Invalid database connection string\")\n    %Jido.Error{\n      type: :config_error,\n      message: \"Invalid database connection string\",\n      details: nil,\n      stacktrace: [...]\n    }","ref":"Jido.Error.html#config_error/3-example"},{"type":"type","title":"Jido.Error.error_type/0","doc":"Defines the possible error types in the Jido system.\n\n- `:invalid_action`: Used when a action is improperly defined or used.\n- `:invalid_sensor`: Used when a sensor is improperly defined or used.\n- `:bad_request`: Indicates an invalid request from the client.\n- `:validation_error`: Used when input validation fails.\n- `:config_error`: Indicates a configuration issue.\n- `:execution_error`: Used when an error occurs during workflow execution.\n- `:workflow_error`: General workflow-related errors.\n- `:internal_server_error`: Indicates an unexpected internal error.\n- `:timeout`: Used when an workflow exceeds its time limit.\n- `:invalid_async_ref`: Indicates an invalid asynchronous workflow reference.\n- `:compensation_error`: Indicates an error occurred during compensation.\n- `:planning_error`: Used when an error occurs during workflow planning.\n- `:routing_error`: Used when an error occurs during workflow routing.","ref":"Jido.Error.html#t:error_type/0"},{"type":"function","title":"Jido.Error.execution_error/3","doc":"Creates a new execution error.\n\nUse this when an error occurs during the execution of an workflow.","ref":"Jido.Error.html#execution_error/3"},{"type":"function","title":"Parameters - Jido.Error.execution_error/3","doc":"- `message`: A string describing the execution error.\n- `details`: (optional) A map containing additional error details.\n- `stacktrace`: (optional) The stacktrace at the point of error.","ref":"Jido.Error.html#execution_error/3-parameters"},{"type":"function","title":"Example - Jido.Error.execution_error/3","doc":"iex> Jido.Error.execution_error(\"Failed to process data\", %{step: \"data_transformation\"})\n    %Jido.Error{\n      type: :execution_error,\n      message: \"Failed to process data\",\n      details: %{step: \"data_transformation\"},\n      stacktrace: [...]\n    }","ref":"Jido.Error.html#execution_error/3-example"},{"type":"function","title":"Jido.Error.format_nimble_config_error/3","doc":"Formats a NimbleOptions validation error for configuration validation.\nUsed when validating configuration options at compile or runtime.","ref":"Jido.Error.html#format_nimble_config_error/3"},{"type":"function","title":"Parameters - Jido.Error.format_nimble_config_error/3","doc":"- `error`: The NimbleOptions.ValidationError to format\n- `module_type`: String indicating the module type (e.g. \"Action\", \"Agent\", \"Sensor\")","ref":"Jido.Error.html#format_nimble_config_error/3-parameters"},{"type":"function","title":"Examples - Jido.Error.format_nimble_config_error/3","doc":"iex> error = %NimbleOptions.ValidationError{keys_path: [:name], message: \"is required\"}\n    iex> Jido.Error.format_nimble_config_error(error, \"Action\")\n    \"Invalid configuration given to use Jido.Action for key [:name]: is required\"","ref":"Jido.Error.html#format_nimble_config_error/3-examples"},{"type":"function","title":"Jido.Error.format_nimble_validation_error/3","doc":"Formats a NimbleOptions validation error for parameter validation.\nUsed when validating runtime parameters.","ref":"Jido.Error.html#format_nimble_validation_error/3"},{"type":"function","title":"Parameters - Jido.Error.format_nimble_validation_error/3","doc":"- `error`: The NimbleOptions.ValidationError to format\n- `module_type`: String indicating the module type (e.g. \"Action\", \"Agent\", \"Sensor\")","ref":"Jido.Error.html#format_nimble_validation_error/3-parameters"},{"type":"function","title":"Examples - Jido.Error.format_nimble_validation_error/3","doc":"iex> error = %NimbleOptions.ValidationError{keys_path: [:input], message: \"is required\"}\n    iex> Jido.Error.format_nimble_validation_error(error, \"Action\")\n    \"Invalid parameters for Action at [:input]: is required\"","ref":"Jido.Error.html#format_nimble_validation_error/3-examples"},{"type":"function","title":"Jido.Error.internal_server_error/3","doc":"Creates a new internal server error.\n\nUse this for unexpected errors that occur within the system.","ref":"Jido.Error.html#internal_server_error/3"},{"type":"function","title":"Parameters - Jido.Error.internal_server_error/3","doc":"- `message`: A string describing the internal server error.\n- `details`: (optional) A map containing additional error details.\n- `stacktrace`: (optional) The stacktrace at the point of error.","ref":"Jido.Error.html#internal_server_error/3-parameters"},{"type":"function","title":"Example - Jido.Error.internal_server_error/3","doc":"iex> Jido.Error.internal_server_error(\"Unexpected error in data processing\")\n    %Jido.Error{\n      type: :internal_server_error,\n      message: \"Unexpected error in data processing\",\n      details: nil,\n      stacktrace: [...]\n    }","ref":"Jido.Error.html#internal_server_error/3-example"},{"type":"function","title":"Jido.Error.invalid_action/3","doc":"Creates a new invalid action error.\n\nUse this when a action is improperly defined or used within the Jido system.","ref":"Jido.Error.html#invalid_action/3"},{"type":"function","title":"Parameters - Jido.Error.invalid_action/3","doc":"- `message`: A string describing the error.\n- `details`: (optional) A map containing additional error details.\n- `stacktrace`: (optional) The stacktrace at the point of error.","ref":"Jido.Error.html#invalid_action/3-parameters"},{"type":"function","title":"Example - Jido.Error.invalid_action/3","doc":"iex> Jido.Error.invalid_action(\"Action 'MyAction' is missing required callback\")\n    %Jido.Error{\n      type: :invalid_action,\n      message: \"Action 'MyAction' is missing required callback\",\n      details: nil,\n      stacktrace: [...]\n    }","ref":"Jido.Error.html#invalid_action/3-example"},{"type":"function","title":"Jido.Error.invalid_async_ref/3","doc":"Creates a new invalid async ref error.\n\nUse this when an invalid reference to an asynchronous workflow is encountered.","ref":"Jido.Error.html#invalid_async_ref/3"},{"type":"function","title":"Parameters - Jido.Error.invalid_async_ref/3","doc":"- `message`: A string describing the invalid async ref error.\n- `details`: (optional) A map containing additional error details.\n- `stacktrace`: (optional) The stacktrace at the point of error.","ref":"Jido.Error.html#invalid_async_ref/3-parameters"},{"type":"function","title":"Example - Jido.Error.invalid_async_ref/3","doc":"iex> Jido.Error.invalid_async_ref(\"Invalid or expired async workflow reference\")\n    %Jido.Error{\n      type: :invalid_async_ref,\n      message: \"Invalid or expired async workflow reference\",\n      details: nil,\n      stacktrace: [...]\n    }","ref":"Jido.Error.html#invalid_async_ref/3-example"},{"type":"function","title":"Jido.Error.invalid_sensor/3","doc":"Creates a new invalid sensor error.\n\nUse this when a sensor is improperly defined or used within the Jido system.","ref":"Jido.Error.html#invalid_sensor/3"},{"type":"function","title":"Parameters - Jido.Error.invalid_sensor/3","doc":"- `message`: A string describing the error.\n- `details`: (optional) A map containing additional error details.\n- `stacktrace`: (optional) The stacktrace at the point of error.","ref":"Jido.Error.html#invalid_sensor/3-parameters"},{"type":"function","title":"Example - Jido.Error.invalid_sensor/3","doc":"iex> Jido.Error.invalid_sensor(\"Sensor 'MySensor' is missing required callback\")\n    %Jido.Error{\n      type: :invalid_sensor,\n      message: \"Sensor 'MySensor' is missing required callback\",\n      details: nil,\n      stacktrace: [...]\n    }","ref":"Jido.Error.html#invalid_sensor/3-example"},{"type":"function","title":"Jido.Error.new/4","doc":"Creates a new error struct with the given type and message.\n\nThis is a low-level function used by other error creation functions in this module.\nConsider using the specific error creation functions unless you need fine-grained control.","ref":"Jido.Error.html#new/4"},{"type":"function","title":"Parameters - Jido.Error.new/4","doc":"- `type`: The error type (see `t:error_type/0`).\n- `message`: A string describing the error.\n- `details`: (optional) A map containing additional error details.\n- `stacktrace`: (optional) The stacktrace at the point of error.","ref":"Jido.Error.html#new/4-parameters"},{"type":"function","title":"Examples - Jido.Error.new/4","doc":"iex> Jido.Error.new(:config_error, \"Invalid configuration\")\n    %Jido.Error{\n      type: :config_error,\n      message: \"Invalid configuration\",\n      details: nil,\n      stacktrace: [...]\n    }\n\n    iex> Jido.Error.new(:execution_error, \"Workflow failed\", %{step: \"data_processing\"})\n    %Jido.Error{\n      type: :execution_error,\n      message: \"Workflow failed\",\n      details: %{step: \"data_processing\"},\n      stacktrace: [...]\n    }","ref":"Jido.Error.html#new/4-examples"},{"type":"function","title":"Jido.Error.planning_error/3","doc":"Creates a new planning error.\n\nUse this when an error occurs during workflow planning.","ref":"Jido.Error.html#planning_error/3"},{"type":"function","title":"Parameters - Jido.Error.planning_error/3","doc":"- `message`: A string describing the planning error.\n- `details`: (optional) A map containing additional error details.\n- `stacktrace`: (optional) The stacktrace at the point of error.","ref":"Jido.Error.html#planning_error/3-parameters"},{"type":"function","title":"Example - Jido.Error.planning_error/3","doc":"iex> Jido.Error.planning_error(\"Failed to plan workflow\", %{step: \"goal_analysis\"})\n    %Jido.Error{\n      type: :planning_error,\n      message: \"Failed to plan workflow\",\n      details: %{step: \"goal_analysis\"},\n      stacktrace: [...]\n    }","ref":"Jido.Error.html#planning_error/3-example"},{"type":"function","title":"Jido.Error.routing_error/3","doc":"Creates a new routing error.\n\nUse this when an error occurs during workflow routing.","ref":"Jido.Error.html#routing_error/3"},{"type":"function","title":"Parameters - Jido.Error.routing_error/3","doc":"- `message`: A string describing the routing error.\n- `details`: (optional) A map containing additional error details.\n- `stacktrace`: (optional) The stacktrace at the point of error.","ref":"Jido.Error.html#routing_error/3-parameters"},{"type":"function","title":"Example - Jido.Error.routing_error/3","doc":"iex> Jido.Error.routing_error(\"Invalid route configuration\", %{route: \"user_workflow\"})\n    %Jido.Error{\n      type: :routing_error,\n      message: \"Invalid route configuration\",\n      details: %{route: \"user_workflow\"},\n      stacktrace: [...]\n    }","ref":"Jido.Error.html#routing_error/3-example"},{"type":"type","title":"Jido.Error.t/0","doc":"Represents a structured error in the Jido system.\n\nFields:\n- `type`: The category of the error (see `t:error_type/0`).\n- `message`: A human-readable description of the error.\n- `details`: Optional map containing additional error context.\n- `stacktrace`: Optional list representing the error's stacktrace.","ref":"Jido.Error.html#t:t/0"},{"type":"function","title":"Jido.Error.timeout/3","doc":"Creates a new timeout error.\n\nUse this when an workflow exceeds its allocated time limit.","ref":"Jido.Error.html#timeout/3"},{"type":"function","title":"Parameters - Jido.Error.timeout/3","doc":"- `message`: A string describing the timeout error.\n- `details`: (optional) A map containing additional error details.\n- `stacktrace`: (optional) The stacktrace at the point of error.","ref":"Jido.Error.html#timeout/3-parameters"},{"type":"function","title":"Example - Jido.Error.timeout/3","doc":"iex> Jido.Error.timeout(\"Workflow timed out after 30 seconds\", %{workflow: \"FetchUserData\"})\n    %Jido.Error{\n      type: :timeout,\n      message: \"Workflow timed out after 30 seconds\",\n      details: %{workflow: \"FetchUserData\"},\n      stacktrace: [...]\n    }","ref":"Jido.Error.html#timeout/3-example"},{"type":"function","title":"Jido.Error.to_map/1","doc":"Converts the error struct to a plain map.\n\nThis function transforms the error struct into a plain map,\nincluding the error type and stacktrace if available. It's useful\nfor serialization or when working with APIs that expect plain maps.","ref":"Jido.Error.html#to_map/1"},{"type":"function","title":"Parameters - Jido.Error.to_map/1","doc":"- `error`: An error struct of type `t:t/0`.","ref":"Jido.Error.html#to_map/1-parameters"},{"type":"function","title":"Returns - Jido.Error.to_map/1","doc":"A map representation of the error.","ref":"Jido.Error.html#to_map/1-returns"},{"type":"function","title":"Example - Jido.Error.to_map/1","doc":"iex> error = Jido.Error.validation_error(\"Invalid input\")\n    iex> Jido.Error.to_map(error)\n    %{\n      type: :validation_error,\n      message: \"Invalid input\",\n      stacktrace: [...]\n    }","ref":"Jido.Error.html#to_map/1-example"},{"type":"function","title":"Jido.Error.validation_error/3","doc":"Creates a new validation error.\n\nUse this when input validation fails for an workflow.","ref":"Jido.Error.html#validation_error/3"},{"type":"function","title":"Parameters - Jido.Error.validation_error/3","doc":"- `message`: A string describing the validation error.\n- `details`: (optional) A map containing additional error details.\n- `stacktrace`: (optional) The stacktrace at the point of error.","ref":"Jido.Error.html#validation_error/3-parameters"},{"type":"function","title":"Example - Jido.Error.validation_error/3","doc":"iex> Jido.Error.validation_error(\"Invalid email format\", %{field: \"email\", value: \"not-an-email\"})\n    %Jido.Error{\n      type: :validation_error,\n      message: \"Invalid email format\",\n      details: %{field: \"email\", value: \"not-an-email\"},\n      stacktrace: [...]\n    }","ref":"Jido.Error.html#validation_error/3-example"},{"type":"function","title":"Jido.Error.workflow_error/3","doc":"Creates a new workflow error.\n\nUse this for general workflow-related errors that don't fit into other categories.","ref":"Jido.Error.html#workflow_error/3"},{"type":"function","title":"Parameters - Jido.Error.workflow_error/3","doc":"- `message`: A string describing the workflow error.\n- `details`: (optional) A map containing additional error details.\n- `stacktrace`: (optional) The stacktrace at the point of error.","ref":"Jido.Error.html#workflow_error/3-parameters"},{"type":"function","title":"Example - Jido.Error.workflow_error/3","doc":"iex> Jido.Error.workflow_error(\"Workflow 'ProcessOrder' failed\", %{order_id: 12345})\n    %Jido.Error{\n      type: :workflow_error,\n      message: \"Workflow 'ProcessOrder' failed\",\n      details: %{order_id: 12345},\n      stacktrace: [...]\n    }","ref":"Jido.Error.html#workflow_error/3-example"},{"type":"module","title":"Jido.Scheduler","doc":"Defines a quantum Scheduler.\n\nWhen used, the quantum scheduler expects the `:otp_app` as option.\nThe `:otp_app` should point to an OTP application that has\nthe quantum runner configuration. For example, the quantum scheduler:\n\n    defmodule Jido.Scheduler do\n      use Quantum, otp_app: :jido\n    end\n\nCould be configured with:\n\n    config :jido, Jido.Scheduler,\n      jobs: [\n        {\"@daily\", {Backup, :backup, []}},\n      ]","ref":"Jido.Scheduler.html"},{"type":"module","title":"Configuration: - Jido.Scheduler","doc":"* `:clock_broadcaster_name` - GenServer name of clock broadcaster \\\n    *(unstable, may break without major release until declared stable)*\n\n  * `:execution_broadcaster_name` - GenServer name of execution broadcaster \\\n    *(unstable, may break without major release until declared stable)*\n\n  * `:executor_supervisor_name` - GenServer name of execution supervisor \\\n    *(unstable, may break without major release until declared stable)*\n\n  * `:debug_logging` - Turn on debug logging\n\n  * `:jobs` - list of cron jobs to execute\n\n  * `:job_broadcaster_name` - GenServer name of job broadcaster \\\n    *(unstable, may break without major release until declared stable)*\n\n  * `:name` - GenServer name of scheduler \\\n    *(unstable, may break without major release until declared stable)*\n\n  * `:node_selector_broadcaster_name` - GenServer name of node selector broadcaster \\\n    *(unstable, may break without major release until declared stable)*\n\n  * `:overlap` - Default overlap of new Job\n\n  * `:otp_app` - Application where scheduler runs\n\n  * `:run_strategy` - Default Run Strategy of new Job\n\n  * `:schedule` - Default schedule of new Job\n\n  * `:storage` - Storage to use for persistence\n\n  * `:storage_name` - GenServer name of storage \\\n    *(unstable, may break without major release until declared stable)*\n\n  * `:supervisor_module` - Module to supervise scheduler \\\n    Can be overwritten to supervise processes differently (for example for clustering) \\\n    *(unstable, may break without major release until declared stable)*\n\n  * `:task_registry_name` - GenServer name of task registry \\\n    *(unstable, may break without major release until declared stable)*\n\n  * `:task_supervisor_name` - GenServer name of task supervisor \\\n    *(unstable, may break without major release until declared stable)*\n\n  * `:timeout` - Sometimes, you may come across GenServer timeout errors\n    esp. when you have too many jobs or high load. The default `GenServer.call/3`\n    timeout is `5_000`.\n\n  * `:timezone` - Default timezone of new Job","ref":"Jido.Scheduler.html#module-configuration"},{"type":"module","title":"Telemetry - Jido.Scheduler","doc":"* `[:quantum, :job, :add]`\n  * Description: dispatched when a job is added\n  * Measurements: `%{}`\n  * Metadata: `%{job: Quantum.Job.t(), scheduler: atom()}`\n\n* `[:quantum, :job, :update]`\n  * Description: dispatched when a job is updated\n  * Measurements: `%{}`\n  * Metadata: `%{job: Quantum.Job.t(), scheduler: atom()}`\n\n* `[:quantum, :job, :delete]`\n  * Description: dispatched when a job is deleted\n  * Measurements: `%{}`\n  * Metadata: `%{job: Quantum.Job.t(), scheduler: atom()}`\n\n* `[:quantum, :job, :start]`\n  * Description: dispatched on job execution start\n  * Measurements: `%{system_time: integer()}`\n  * Metadata: `%{telemetry_span_context: term(), job: Quantum.Job.t(), node: Node.t(), scheduler: atom()}`\n\n* `[:quantum, :job, :stop]`\n  * Description: dispatched on job execution end\n  * Measurements: `%{duration: integer()}`\n  * Metadata: `%{telemetry_span_context: term(), job: Quantum.Job.t(), node: Node.t(), scheduler: atom(), result: term()}`\n\n* `[:quantum, :job, :exception]`\n  * Description: dispatched on job execution fail\n  * Measurements: `%{duration: integer()}`\n  * Metadata: `%{telemetry_span_context: term(), job: Quantum.Job.t(), node: Node.t(), scheduler: atom(), kind: :throw | :error | :exit, reason: term(), stacktrace: list()}`","ref":"Jido.Scheduler.html#module-telemetry"},{"type":"module","title":"Examples - Jido.Scheduler","doc":"iex(1)> :telemetry_registry.discover_all(:quantum)\n    :ok\n    iex(2)> :telemetry_registry.spannable_events()\n    [{[:quantum, :job], [:start, :stop, :exception]}]\n    iex(3)> :telemetry_registry.list_events\n    [\n      {[:quantum, :job, :add], Quantum,\n       %{\n         description: \"dispatched when a job is added\",\n         measurements: \"%{}\",\n         metadata: \"%{job: Quantum.Job.t(), scheduler: atom()}\"\n       }},\n      {[:quantum, :job, :delete], Quantum,\n       %{\n         description: \"dispatched when a job is deleted\",\n         measurements: \"%{}\",\n         metadata: \"%{job: Quantum.Job.t(), scheduler: atom()}\"\n       }},\n      {[:quantum, :job, :exception], Quantum,\n       %{\n         description: \"dispatched on job execution fail\",\n         measurements: \"%{duration: integer()}\",\n         metadata: \"%{telemetry_span_context: term(), job: Quantum.Job.t(), node: Node.t(), scheduler: atom(), kind: :throw | :error | :exit, reason: term(), stacktrace: list()}\"\n       }},\n      {[:quantum, :job, :start], Quantum,\n       %{\n         description: \"dispatched on job execution start\",\n         measurements: \"%{system_time: integer()}\",\n         metadata: \"%{telemetry_span_context: term(), job: Quantum.Job.t(), node: Node.t(), scheduler: atom()}\"\n       }},\n      {[:quantum, :job, :stop], Quantum,\n       %{\n         description: \"dispatched on job execution end\",\n         measurements: \"%{duration: integer()}\",\n         metadata: \"%{telemetry_span_context: term(), job: Quantum.Job.t(), node: Node.t(), scheduler: atom(), result: term()}\"\n       }},\n      {[:quantum, :job, :update], Quantum,\n       %{\n         description: \"dispatched when a job is updated\",\n         measurements: \"%{}\",\n         metadata: \"%{job: Quantum.Job.t(), scheduler: atom()}\"\n       }}\n    ]","ref":"Jido.Scheduler.html#module-examples"},{"type":"function","title":"Jido.Scheduler.child_spec/1","doc":"","ref":"Jido.Scheduler.html#child_spec/1"},{"type":"protocol","title":"Jido.Serialization.JsonDecoder","doc":"Protocol to allow additional decoding of a value that has been deserialized from JSON.\n\nThis protocol enables custom decoding logic to be applied after JSON deserialization.\nIt is particularly useful when you need to transform or validate data after it has been\ndeserialized from JSON format.","ref":"Jido.Serialization.JsonDecoder.html"},{"type":"protocol","title":"Example - Jido.Serialization.JsonDecoder","doc":"defmodule MyStruct do\n      @derive Jason.Encoder\n      defstruct [:value]\n    end\n\n    # Custom decoder implementation that doubles numeric values\n    defimpl Jido.Serialization.JsonDecoder, for: MyStruct do\n      def decode(%MyStruct{value: value} = data) when is_number(value) do\n        %MyStruct{value: value * 2}\n      end\n\n      def decode(data), do: data\n    end\n\nThe protocol is optional with a fallback implementation for Any that returns the data unchanged.","ref":"Jido.Serialization.JsonDecoder.html#module-example"},{"type":"function","title":"Jido.Serialization.JsonDecoder.decode/1","doc":"Decodes data that has been deserialized using the `Jido.Serialization.JsonSerializer`.\n\nThis function is called after JSON deserialization to perform any additional data transformations\nor validations needed for the specific data type.","ref":"Jido.Serialization.JsonDecoder.html#decode/1"},{"type":"function","title":"Parameters - Jido.Serialization.JsonDecoder.decode/1","doc":"* `data` - The deserialized data to be decoded","ref":"Jido.Serialization.JsonDecoder.html#decode/1-parameters"},{"type":"function","title":"Returns - Jido.Serialization.JsonDecoder.decode/1","doc":"The decoded data, which can be transformed in any way appropriate for the implementing type.\n  The default implementation for Any simply returns the data unchanged.","ref":"Jido.Serialization.JsonDecoder.html#decode/1-returns"},{"type":"type","title":"Jido.Serialization.JsonDecoder.t/0","doc":"All the types that implement this protocol.","ref":"Jido.Serialization.JsonDecoder.html#t:t/0"},{"type":"module","title":"Jido.Serialization.JsonSerializer","doc":"A serializer that uses the JSON format and Jason library.","ref":"Jido.Serialization.JsonSerializer.html"},{"type":"function","title":"Jido.Serialization.JsonSerializer.deserialize/2","doc":"Deserialize given JSON binary data to the expected type.","ref":"Jido.Serialization.JsonSerializer.html#deserialize/2"},{"type":"function","title":"Jido.Serialization.JsonSerializer.serialize/1","doc":"Serialize given term to JSON binary data.","ref":"Jido.Serialization.JsonSerializer.html#serialize/1"},{"type":"module","title":"Jido.Serialization.ModuleNameTypeProvider","doc":"A type provider that uses the Elixir module name\n\nExample:\n\n  - %An.Event{} module mapped to \"Elixir.An.Event\".","ref":"Jido.Serialization.ModuleNameTypeProvider.html"},{"type":"function","title":"Jido.Serialization.ModuleNameTypeProvider.to_string/1","doc":"Convert a struct to a type string.","ref":"Jido.Serialization.ModuleNameTypeProvider.html#to_string/1"},{"type":"function","title":"Jido.Serialization.ModuleNameTypeProvider.to_struct/1","doc":"Convert a type string to a struct.","ref":"Jido.Serialization.ModuleNameTypeProvider.html#to_struct/1"},{"type":"behaviour","title":"Jido.Serialization.TypeProvider","doc":"Specification to convert between an Elixir struct and a corresponding string type.","ref":"Jido.Serialization.TypeProvider.html"},{"type":"type","title":"Jido.Serialization.TypeProvider.t/0","doc":"","ref":"Jido.Serialization.TypeProvider.html#t:t/0"},{"type":"callback","title":"Jido.Serialization.TypeProvider.to_string/1","doc":"Type of the given Elixir struct as a string","ref":"Jido.Serialization.TypeProvider.html#c:to_string/1"},{"type":"callback","title":"Jido.Serialization.TypeProvider.to_struct/1","doc":"Convert the given type string to an Elixir struct","ref":"Jido.Serialization.TypeProvider.html#c:to_struct/1"},{"type":"type","title":"Jido.Serialization.TypeProvider.type/0","doc":"","ref":"Jido.Serialization.TypeProvider.html#t:type/0"},{"type":"function","title":"Jido.Serialization.TypeProvider.type_provider/0","doc":"Get the configured type provider","ref":"Jido.Serialization.TypeProvider.html#type_provider/0"},{"type":"module","title":"Jido.Supervisor","doc":"A helper supervisor that starts and manages the resources for a specific Jido instance.\n\nEach consumer of Jido defines their own module:\n\n    defmodule MyApp.Jido do\n      use Jido, otp_app: :my_app\n    end\n\nThen in your application’s supervision tree:\n\n    children = [\n      MyApp.Jido\n    ]\n\n    Supervisor.start_link(children, strategy: :one_for_one)","ref":"Jido.Supervisor.html"},{"type":"function","title":"Jido.Supervisor.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Jido.Supervisor.html#child_spec/1"},{"type":"function","title":"Jido.Supervisor.start_link/2","doc":"","ref":"Jido.Supervisor.html#start_link/2"},{"type":"module","title":"Jido.Util","doc":"A collection of utility functions for the Jido framework.\n\nThis module provides various helper functions that are used throughout the Jido framework,\nincluding:\n\n- ID generation\n- Name validation\n- Error handling\n- Logging utilities\n\nThese utilities are designed to support common operations and maintain consistency\nacross the Jido ecosystem. They encapsulate frequently used patterns and provide\na centralized location for shared functionality.\n\nMany of the functions in this module are used internally by other Jido modules,\nbut they can also be useful for developers building applications with Jido.","ref":"Jido.Util.html"},{"type":"function","title":"Jido.Util.generate_id/0","doc":"Generates a unique ID.","ref":"Jido.Util.html#generate_id/0"},{"type":"function","title":"Jido.Util.validate_actions/1","doc":"Validates that all modules in a list implement the Jido.Action behavior.\nUsed as a custom validator for NimbleOptions.\n\nThis function ensures that all provided modules are valid Jido.Action implementations\nby checking that they:\n1. Are valid Elixir modules that can be loaded\n2. Export the required __action_metadata__/0 function that indicates Jido.Action behavior","ref":"Jido.Util.html#validate_actions/1"},{"type":"function","title":"Parameters - Jido.Util.validate_actions/1","doc":"- `actions`: A list of module atoms or a single module atom to validate","ref":"Jido.Util.html#validate_actions/1-parameters"},{"type":"function","title":"Returns - Jido.Util.validate_actions/1","doc":"- `{:ok, actions}` if all modules are valid Jido.Action implementations\n- `{:error, reason}` if any module is invalid","ref":"Jido.Util.html#validate_actions/1-returns"},{"type":"function","title":"Examples - Jido.Util.validate_actions/1","doc":"iex> defmodule ValidAction do\n    ...>   use Jido.Action,\n    ...>     name: \"valid_action\"\n    ...> end\n    ...> Jido.Util.validate_actions([ValidAction])\n    {:ok, [ValidAction]}\n\n    iex> Jido.Util.validate_actions([InvalidModule])\n    {:error, \"All actions must implement the Jido.Action behavior\"}\n\n    # Single module validation\n    iex> Jido.Util.validate_actions(ValidAction)\n    {:ok, [ValidAction]}","ref":"Jido.Util.html#validate_actions/1-examples"},{"type":"function","title":"Jido.Util.validate_name/1","doc":"Validates the name of a Action.\n\nThe name must contain only letters, numbers, and underscores.","ref":"Jido.Util.html#validate_name/1"},{"type":"function","title":"Parameters - Jido.Util.validate_name/1","doc":"- `name`: The name to validate.","ref":"Jido.Util.html#validate_name/1-parameters"},{"type":"function","title":"Returns - Jido.Util.validate_name/1","doc":"- `{:ok, name}` if the name is valid.\n- `{:error, reason}` if the name is invalid.","ref":"Jido.Util.html#validate_name/1-returns"},{"type":"function","title":"Examples - Jido.Util.validate_name/1","doc":"iex> Jido.Action.validate_name(\"valid_name_123\")\n    {:ok, \"valid_name_123\"}\n\n    iex> Jido.Action.validate_name(\"invalid-name\")\n    {:error, \"The name must contain only letters, numbers, and underscores.\"}","ref":"Jido.Util.html#validate_name/1-examples"},{"type":"function","title":"Jido.Util.validate_runner/1","doc":"Validates that a module implements the Jido.Runner behavior.\n\nThis function ensures that the provided module is a valid Jido.Runner implementation\nby checking that it:\n1. Is a valid Elixir module that can be loaded\n2. Exports the required run/2 function that indicates Jido.Runner behavior","ref":"Jido.Util.html#validate_runner/1"},{"type":"function","title":"Parameters - Jido.Util.validate_runner/1","doc":"- `module`: The module atom to validate","ref":"Jido.Util.html#validate_runner/1-parameters"},{"type":"function","title":"Returns - Jido.Util.validate_runner/1","doc":"- `{:ok, module}` if the module is a valid Jido.Runner implementation\n- `{:error, Jido.Error.t()}` if the module is invalid","ref":"Jido.Util.html#validate_runner/1-returns"},{"type":"function","title":"Examples - Jido.Util.validate_runner/1","doc":"iex> defmodule ValidRunner do\n    ...>   @behaviour Jido.Runner\n    ...>   def run(agent, opts), do: {:ok, agent}\n    ...> end\n    ...> Jido.Util.validate_runner(ValidRunner)\n    {:ok, ValidRunner}\n\n    iex> Jido.Util.validate_runner(InvalidModule)\n    {:error, %Jido.Error{type: :validation_error, message: \"Module InvalidModule must implement run/2\"}}","ref":"Jido.Util.html#validate_runner/1-examples"},{"type":"extras","title":"Home","doc":"# Jido (自動)\n\nJido is a foundational framework for building autonomous, distributed agent systems in Elixir.\n\nThe name \"Jido\" (自動) comes from the Japanese word meaning \"automatic\" or \"automated\", where 自 (ji) means \"self\" and 動 (dō) means \"movement\".\n\n[![Hex Version](https://img.shields.io/hexpm/v/jido.svg)](https://hex.pm/packages/jido)\n[![Hex Docs](http://img.shields.io/badge/hex.pm-docs-green.svg?style=flat)](https://hexdocs.pm/jido)\n[![Mix Test](https://github.com/agentjido/jido/actions/workflows/elixir-ci.yml/badge.svg)](https://github.com/agentjido/jido/actions/workflows/elixir-ci.yml)\n[![Coverage Status](https://coveralls.io/repos/github/agentjido/jido/badge.svg?branch=main)](https://coveralls.io/github/agentjido/jido?branch=main)\n[![Apache 2 License](https://img.shields.io/hexpm/l/jido)](https://opensource.org/licenses/Apache-2.0)","ref":"readme.html"},{"type":"extras","title":"Quick Start - Home","doc":"```elixir\n# First, define our Calculator agent with supported operations\niex> defmodule CalculatorAgent do\n...>   use Jido.Agent,\n...>     name: \"calculator\",\n...>     actions: [Actions.Add, Actions.Subtract, Actions.Multiply, Actions.Divide]\n...>   # Omitting the router that maps the \"add\" signal to the Add Action\n...> end\n{:module, CalculatorAgent, <<...>>, %{}}\n\n# Start the agent process\niex> {:ok, pid} = CalculatorAgent.start_link()\n{:ok, #PID<0.123.0>}\n\n# Send a synchronous request to the agent\niex> {:ok, result} = CalculatorAgent.call(pid, Signal.new(%{type: \"add\", data: %{a: 1, b: 2}}))\n{:ok, 3}\n\n# Send an asynchronous request to the agent\niex> {:ok, request_id} = CalculatorAgent.cast(pid, Signal.new(%{type: \"multiply\", data: %{a: 2, b: 4}}))\n{:ok, \"req_abc123\"}\n\n# Receive the result of the asynchronous request\niex> flush()\n{:jido_agent, \"req_abc123\", 8}\n:ok\n```\n\nThis example barely scratches the surface of what Jido can do. For more examples, see the [Getting Started Guide](guides/getting-started.md) and [Jido Workbench](https://github.com/agentjido/jido_workbench) to play with our growing catalog of real-life examples.","ref":"readme.html#quick-start"},{"type":"extras","title":"Overview - Home","doc":"Jido provides a robust foundation for building autonomous agents that can plan, execute, and adapt their behavior in distributed Elixir applications. Think of it as a toolkit for creating smart, composable workflows that can evolve and respond to their environment.","ref":"readme.html#overview"},{"type":"extras","title":"Are You Sure You Need an Agent? - Home","doc":"Agents are a hot topic right now, but they aren’t a silver bullet. In particular, Large Language Models (LLMs) are powerful yet slow and costly—if your application doesn’t require dynamic decision-making or complex planning, consider whether you really need an Agent at all.\n\n- **LLMs aren’t required for all tasks** — Avoid building them into your core logic unless necessary\n- **Agents as Dynamic ETL** — Agents dynamically direct data ingestion, transformation, and output based on:\n  - LLMs (e.g., GPT)\n  - Classical planning algorithms (A\\*, Behavior Trees, etc.)\n- **Simplicity often wins** — If you don’t need these dynamic behaviors, you probably don’t need an Agent. This library is likely overkill compared to straightforward code.","ref":"readme.html#are-you-sure-you-need-an-agent"},{"type":"extras","title":"Our Definition of an Agent - Home","doc":"An Agent is a system where LLMs _or_ classical planning algorithms dynamically direct their own processes. Some great definitions from the community:\n\n- “Agents are Dynamic ETL processes directed by LLMs” — [YouTube](https://youtu.be/KY8n96Erp5Q?si=5Itt7QR11jgfWDTY&t=22)\n- “Agents are systems where LLMs dynamically direct their own processes” — [Anthropic Research](https://www.anthropic.com/research/building-effective-agents)\n- “AI Agents are programs where LLM outputs control the workflow” — [Hugging Face Blog](https://huggingface.co/blog/smolagents)\n\nIf your application doesn’t involve dynamic workflows or data pipelines that change based on AI or planning algorithms, you can likely do more with less.\n\n> 💡 **NOTE**: This library intends to support both LLM planning and Classical AI planning (ie. [Behavior Trees](https://github.com/jschomay/elixir-behavior-tree) as a design principle via Actions. See [`jido_ai`](https://github.com/agentjido/jido_ai) for example LLM actions.\n\n_This space is evolving rapidly. Last updated 2025-01-01_","ref":"readme.html#our-definition-of-an-agent"},{"type":"extras","title":"Key Features - Home","doc":"- 🧩 **Composable Actions**: Build complex behaviors from simple, reusable actions\n- 🤖 **Autonomous Agents**: Self-directing entities that plan and execute workflows\n- 📡 **Real-time Sensors**: Event-driven data gathering and monitoring\n- 🔄 **Adaptive Learning**: Agents can modify their capabilities at runtime\n- 📊 **Built-in Telemetry**: Comprehensive observability and debugging\n- ⚡ **Distributed by Design**: Built for multi-node Elixir clusters\n- 🧪 **Testing Tools**: Rich helpers for unit and property-based testing","ref":"readme.html#key-features"},{"type":"extras","title":"Installation - Home","doc":"Add Jido to your dependencies:\n\n```elixir\ndef deps do\n  [\n    {:jido, \"~> 1.0.0\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Core Concepts - Home","doc":"","ref":"readme.html#core-concepts"},{"type":"extras","title":"Actions - Home","doc":"Actions are the fundamental building blocks in Jido. Each Action is a discrete, reusable unit of work with a clear interface:\n\n```elixir\ndefmodule MyApp.Actions.FormatUser do\n  use Jido.Action,\n    name: \"format_user\",\n    description: \"Formats user data by trimming whitespace and normalizing email\",\n    schema: [\n      name: [type: :string, required: true],\n      email: [type: :string, required: true]\n    ]\n\n  def run(params, _context) do\n    {:ok, %{\n      formatted_name: String.trim(params.name),\n      email: String.downcase(params.email)\n    }}\n  end\nend\n```\n\n[Learn more about Actions →](guides/actions.md)","ref":"readme.html#actions"},{"type":"extras","title":"Workflows - Home","doc":"Workflows chain Actions together to accomplish complex tasks. Jido handles data flow and error handling between steps:\n\n```elixir\nalias MyApp.Actions.{FormatUser, EnrichUserData, NotifyUser}\n\n{:ok, result} = Jido.Workflow.Chain.chain(\n  [FormatUser, EnrichUserData, NotifyUser],\n  %{\n    name: \"John Doe \",\n    email: \"JOHN@EXAMPLE.COM\"\n  }\n)\n```\n\n[Learn more about Workflows →](guides/actions.md#combining-actions-into-a-workflow)","ref":"readme.html#workflows"},{"type":"extras","title":"Agents - Home","doc":"Agents are stateful entities that can plan and execute Actions. They maintain their state through a schema and can adapt their behavior:\n\n```elixir\ndefmodule MyApp.CalculatorAgent do\n  use Jido.Agent,\n    name: \"calculator\",\n    description: \"An adaptive calculating agent\",\n    actions: [\n      MyApp.Actions.Add,\n      MyApp.Actions.Multiply,\n      Jido.Actions.Directives.RegisterAction\n    ],\n    schema: [\n      value: [type: :float, default: 0.0],\n      operations: [type: {:list, :atom}, default: []]\n    ]\n\n  def on_after_run(agent, result) do\n    # Track which operations we've used\n    ops = [result.action | agent.state.operations] |> Enum.uniq()\n    {:ok, %{agent | state: %{agent.state | operations: ops}}}\n  end\nend\n```\n\n[Learn more about Agents →](guides/agents.md)","ref":"readme.html#agents"},{"type":"extras","title":"Sensors - Home","doc":"Sensors provide real-time monitoring and data gathering for your agents:\n\n```elixir\ndefmodule MyApp.Sensors.OperationCounter do\n  use Jido.Sensor,\n    name: \"operation_counter\",\n    description: \"Tracks operation usage metrics\",\n    schema: [\n      emit_interval: [type: :pos_integer, default: 1000]\n    ]\n\n  def mount(opts) do\n    {:ok, Map.merge(opts, %{counts: %{}})}\n  end\n\n  def handle_info({:operation, name}, state) do\n    new_counts = Map.update(state.counts, name, 1, & &1 + 1)\n    {:noreply, %{state | counts: new_counts}}\n  end\nend\n```\n\n[Learn more about Sensors →](guides/sensors.md)","ref":"readme.html#sensors"},{"type":"extras","title":"Running in Production - Home","doc":"Start your agents under supervision:\n\n```elixir\n# In your application.ex\nchildren = [\n  {Registry, keys: :unique, name: Jido.AgentRegistry},\n  {Phoenix.PubSub, name: MyApp.PubSub},\n  {Jido.Agent.Supervisor, pubsub: MyApp.PubSub},\n  {Jido.Agent.Server,\n    agent: MyApp.CalculatorAgent.new(),\n    name: \"calculator_1\"\n  }\n]\n\nSupervisor.start_link(children, strategy: :one_for_one)\n```","ref":"readme.html#running-in-production"},{"type":"extras","title":"Example Use Cases - Home","doc":"- **Service Orchestration**: Coordinate complex workflows across multiple services\n- **Data Processing**: Build adaptive ETL pipelines that evolve with your data\n- **Business Automation**: Model complex business processes with autonomous agents\n- **System Monitoring**: Create smart monitoring agents that adapt to system behavior\n- **Transaction Management**: Handle multi-step transactions with built-in compensation\n- **Event Processing**: Process and react to event streams in real-time","ref":"readme.html#example-use-cases"},{"type":"extras","title":"Documentation - Home","doc":"- [📘 Getting Started Guide](guides/getting-started.md)\n- [🧩 Actions & Workflows](guides/actions.md)\n- [🤖 Building Agents](guides/agents.md)\n- [📡 Sensors & Monitoring](guides/sensors.md)\n- [🔄 Agent Directives](guides/directives.md)","ref":"readme.html#documentation"},{"type":"extras","title":"Contributing - Home","doc":"We welcome contributions! Here's how to get started:\n\n1. Fork the repository\n2. Run tests: `mix test`\n3. Run quality checks: `mix quality`\n4. Submit a PR\n\nPlease include tests for any new features or bug fixes.\n\nSee our [Contributing Guide](CONTRIBUTING.md) for detailed guidelines.","ref":"readme.html#contributing"},{"type":"extras","title":"Testing - Home","doc":"Jido is built with a test-driven mindset and provides comprehensive testing tools for building reliable agent systems. Our testing philosophy emphasizes:\n\n- Thorough test coverage for core functionality\n- Property-based testing for complex behaviors\n- Regression tests for every bug fix\n- Extensive testing helpers and utilities","ref":"readme.html#testing"},{"type":"extras","title":"Testing Utilities - Home","doc":"Jido provides several testing helpers:\n\n- `Jido.TestSupport` - Common testing utilities\n- Property-based testing via StreamData\n- Mocking support through Mimic\n- PubSub testing helpers\n- Signal assertion helpers","ref":"readme.html#testing-utilities"},{"type":"extras","title":"Running Tests - Home","doc":"```bash\n# Run the test suite\nmix test\n\n# Run with coverage reporting\nmix test --cover\n\n# Run the full quality check suite\nmix quality\n```\n\nWhile we strive for 100% test coverage, we prioritize meaningful tests that verify behavior over simple line coverage. Every new feature and bug fix includes corresponding tests to prevent regressions.","ref":"readme.html#running-tests"},{"type":"extras","title":"License - Home","doc":"Apache License 2.0 - See [LICENSE.md](LICENSE.md) for details.","ref":"readme.html#license"},{"type":"extras","title":"Support - Home","doc":"- 📚 [Documentation](https://hexdocs.pm/jido)\n- 💬 [GitHub Discussions](https://github.com/agentjido/jido/discussions)\n- 🐛 [Issue Tracker](https://github.com/agentjido/jido/issues)","ref":"readme.html#support"},{"type":"extras","title":"Getting Started","doc":"# Getting Started with Jido\n\nWelcome to **Jido** (自動), a powerful yet approachable Elixir framework for building autonomous, distributed agent systems. Whether you're building complex business processes, service orchestration, or adaptive workflows, Jido provides a robust foundation for your autonomous agents.","ref":"getting-started.html"},{"type":"extras","title":"Overview - Getting Started","doc":"Jido is built around four core concepts that work together to create intelligent, adaptable systems:\n\n1. **Actions**: Discrete, reusable units of work\n2. **Workflows**: Sequences of Actions that accomplish larger goals\n3. **Agents**: Stateful entities that can plan and execute workflows\n4. **Sensors**: Real-time monitoring and data gathering components\n\nLet's see how these pieces fit together.","ref":"getting-started.html#overview"},{"type":"extras","title":"Installation - Getting Started","doc":"Add Jido to your project's dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:jido, \"~> 1.0.0\"}\n  ]\nend\n```\n\nThen run:\n\n```bash\nmix deps.get\n```","ref":"getting-started.html#installation"},{"type":"extras","title":"Quick Start Example - Getting Started","doc":"Here's a complete example showing all major components working together in a user registration system:\n\n```elixir\n# 1. Define an Action\ndefmodule MyApp.Actions.FormatUser do\n  use Jido.Action,\n    name: \"format_user\",\n    description: \"Formats user data\",\n    schema: [\n      name: [type: :string, required: true],\n      email: [type: :string, required: true]\n    ]\n\n  def run(params, _context) do\n    {:ok, %{\n      formatted_name: String.trim(params.name),\n      email: String.downcase(params.email)\n    }}\n  end\nend\n\n# 2. Create a Sensor to monitor registrations\ndefmodule MyApp.Sensors.RegistrationCounter do\n  use Jido.Sensor,\n    name: \"registration_counter\",\n    description: \"Tracks registration metrics\",\n    schema: [\n      emit_interval: [type: :pos_integer, default: 1000]\n    ]\n\n  def mount(opts) do\n    {:ok, Map.merge(opts, %{successful: 0, failed: 0})}\n  end\n\n  def handle_info({:registration, :success}, state) do\n    new_state = %{state | successful: state.successful + 1}\n    {:noreply, new_state}\n  end\nend\n\n# 3. Define an Agent to manage registrations\ndefmodule MyApp.Agents.RegistrationAgent do\n  use Jido.Agent,\n    name: \"registration_agent\",\n    description: \"Manages user registration process\",\n    actions: [MyApp.Actions.FormatUser],\n    schema: [\n      registrations: [type: :integer, default: 0],\n      last_registration: [type: {:or, [:map, nil]}, default: nil]\n    ]\n\n  def on_after_run(agent, result) do\n    new_count = agent.state.registrations + 1\n    {:ok, %{agent | state: %{agent.state | \n      registrations: new_count,\n      last_registration: result\n    }}}\n  end\nend\n\n# 4. Use it all together\ndefmodule MyApp.Example do\n  def register_user do\n    # Create and configure the agent\n    agent = MyApp.Agents.RegistrationAgent.new()\n    \n    # Execute a registration\n    {:ok, agent} = MyApp.Agents.RegistrationAgent.cmd(\n      agent,\n      MyApp.Actions.FormatUser,\n      %{\n        name: \"John Doe \",\n        email: \"JOHN@EXAMPLE.COM\"\n      }\n    )\n\n    # Results are stored in agent state\n    agent.state.last_registration\n  end\nend\n```","ref":"getting-started.html#quick-start-example"},{"type":"extras","title":"Key Concepts Explained - Getting Started","doc":"","ref":"getting-started.html#key-concepts-explained"},{"type":"extras","title":"Actions: The Building Blocks - Getting Started","doc":"Actions are small, focused pieces of functionality that:\n- Have a clear input schema\n- Perform one specific task\n- Return standardized results\n- Can be composed into workflows\n\n[Learn more about Actions →](actions.md)","ref":"getting-started.html#actions-the-building-blocks"},{"type":"extras","title":"Workflows: Combining Actions - Getting Started","doc":"Workflows chain Actions together to accomplish larger goals:\n- Pass data between Actions automatically\n- Handle errors consistently\n- Support async execution\n- Enable conditional paths\n\nExample workflow:\n```elixir\nalias MyApp.Actions.{FormatUser, EnrichData, NotifyUser}\n\n{:ok, result} = Jido.Workflow.Chain.chain(\n  [FormatUser, EnrichData, NotifyUser],\n  %{name: \"John Doe\", email: \"john@example.com\"}\n)\n```","ref":"getting-started.html#workflows-combining-actions"},{"type":"extras","title":"Agents: Stateful Intelligence - Getting Started","doc":"Agents provide stateful wrappers around Actions with:\n- Schema-validated state management\n- Action planning and execution\n- Runtime adaptation through directives\n- Lifecycle callbacks\n\n[Learn more about Agents →](agents.md)","ref":"getting-started.html#agents-stateful-intelligence"},{"type":"extras","title":"Sensors: Real-time Monitoring - Getting Started","doc":"Sensors gather data and monitor system state:\n- Run as independent processes\n- Emit standardized signals\n- Support real-time metrics\n- Enable adaptive behavior\n\n[Learn more about Sensors →](sensors.md)","ref":"getting-started.html#sensors-real-time-monitoring"},{"type":"extras","title":"Running in Production - Getting Started","doc":"To run Jido in production, start your components under supervision:\n\n```elixir\n# In your application.ex\nchildren = [\n  {Registry, keys: :unique, name: Jido.AgentRegistry},\n  {Phoenix.PubSub, name: MyApp.PubSub},\n  {Jido.Agent.Supervisor, pubsub: MyApp.PubSub},\n  \n  # Start your agent server\n  {Jido.Agent.Server, \n    agent: MyApp.Agents.RegistrationAgent.new(),\n    name: \"registration_1\"\n  },\n  \n  # Start your sensor\n  {MyApp.Sensors.RegistrationCounter,\n    name: \"counter_1\",\n    pubsub: MyApp.PubSub,\n    topic: \"registration_metrics\"\n  }\n]\n\nSupervisor.start_link(children, strategy: :one_for_one)\n```","ref":"getting-started.html#running-in-production"},{"type":"extras","title":"Next Steps - Getting Started","doc":"Now that you understand the basics, you can explore:\n\n1. [Building Complex Actions and Workflows](actions.md)\n   - Action composition patterns\n   - Error handling\n   - Async execution\n   - Testing strategies\n\n2. [Creating Smart Agents](agents.md)\n   - State management\n   - Planning and execution\n   - Lifecycle hooks\n   - Directive system\n\n3. [Implementing Sensors](sensors.md)\n   - Signal design\n   - Real-time monitoring\n   - Metric collection\n   - Event processing\n\n4. [Using Agent Directives](directives.md)\n   - Runtime adaptation\n   - Dynamic capabilities\n   - Self-modification patterns","ref":"getting-started.html#next-steps"},{"type":"extras","title":"Additional Resources - Getting Started","doc":"- [Complete Documentation](https://hexdocs.pm/jido)\n- [GitHub Repository](https://github.com/agentjido/jido)\n- [Examples and Tutorials](/)\n\nFor AI/LLM integration capabilities, see the separate [`jido_ai`](https://github.com/agentjido/jido_ai) package.","ref":"getting-started.html#additional-resources"},{"type":"extras","title":"Contributing - Getting Started","doc":"We welcome contributions! Please see our [Contributing Guide](CONTRIBUTING.md) for guidelines.","ref":"getting-started.html#contributing"},{"type":"extras","title":"License - Getting Started","doc":"Apache License 2.0 - See [LICENSE.md](LICENSE.md) for details.","ref":"getting-started.html#license"},{"type":"extras","title":"Actions & Workflows","doc":"# Actions & Workflows\n\nThis guide will introduce you to building composable, maintainable workflows using Jido's action-based architecture. Whether you're building autonomous agents, complex business processes, or distributed systems, Jido provides a robust foundation for your workflows.","ref":"actions.html"},{"type":"extras","title":"Understanding Core Concepts - Actions & Workflows","doc":"Before diving into code, let's understand the foundational concepts of Jido and when to use them.","ref":"actions.html#understanding-core-concepts"},{"type":"extras","title":"Why Actions? - Actions & Workflows","doc":"You might wonder: \"Why should I wrap my code in Actions when I could just write regular Elixir functions?\" This is a crucial question. If you're building a standard Elixir application without autonomous agents, you probably shouldn't use Actions – regular Elixir modules and functions would be simpler and more direct.\n\nActions exist specifically to support autonomous agent systems. When building agents that need to make independent decisions about what steps to take, you need a way to package functionality into discrete, composable units that the agent can reason about and combine in novel ways. Think of Actions as LEGO bricks for agents – standardized, well-described building blocks that can be assembled in different combinations to solve problems.\n\nThe Action system provides several critical features for agent-based systems:\n\n- Comprehensive metadata for agent reasoning\n- Schema validation using NimbleOptions\n- Built-in telemetry and observability\n- Enhanced error handling with compensation\n- Improved runtime safety checks\n- Dynamic composition through workflows\n- IO operation safety monitoring\n- Task group management for concurrent operations","ref":"actions.html#why-actions"},{"type":"extras","title":"Core Components - Actions & Workflows","doc":"Jido's action system consists of three main components:\n\n1. **Actions**: Discrete, composable units of work that:\n\n   - Have clear input/output contracts\n   - Provide rich metadata\n   - Support validation and compensation\n   - Can be reasoned about by agents\n\n2. **Workflows**: Sequences of Actions that:\n\n   - Handle complex orchestration\n   - Manage state transitions\n   - Support conditional branching\n   - Provide error recovery\n\n3. **Chain Runner**: Runtime engine that:\n   - Executes action sequences\n   - Manages data flow\n   - Handles errors and compensation\n   - Provides telemetry and monitoring","ref":"actions.html#core-components"},{"type":"extras","title":"Creating Your First Action - Actions & Workflows","doc":"Let's create a simple Action to understand the features:\n\n```elixir\ndefmodule MyApp.Actions.FormatUser do\n  use Jido.Action,\n    name: \"format_user\",\n    description: \"Formats and validates user data\",\n    category: \"user_management\",\n    tags: [\"user\", \"formatting\"],\n    compensation: [\n      enabled: true,\n      max_retries: 3,\n      timeout: 5000\n    ],\n    schema: [\n      name: [\n        type: :string,\n        required: true,\n        doc: \"User's full name - will be trimmed\"\n      ],\n      email: [\n        type: :string,\n        required: true,\n        doc: \"Email address - will be normalized\"\n      ],\n      age: [\n        type: :integer,\n        required: true,\n        doc: \"User's age in years\"\n      ]\n    ]\n\n  @impl true\n  def run(params, context) do\n    with {:ok, formatted} <- format_data(params) do\n      Logger.warning(\"User data formatted #{inspect(formatted)}\")\n\n      {:ok, formatted}\n    end\n  end\n\n  @impl true\n  def on_error(failed_params, error, _context, _opts) do\n    # Compensation logic for failures\n    Logger.warning(\"Compensating for format failure\",\n      error: inspect(error),\n      params: failed_params\n    )\n\n    {:ok, %{compensated: true, original_error: error}}\n  end\n\n  defp format_data(%{name: name, email: email, age: age}) do\n    {:ok, %{\n      formatted_name: String.trim(name),\n      email: String.downcase(email),\n      age: age,\n      is_adult: age >= 18\n    }}\n  end\nend\n```\n\nLet's break down the key features:\n\n1. **Rich Metadata**:\n\n   - `name`, `description`, `category`, and `tags` for better organization\n   - Comprehensive schema documentation\n   - Clear action description\n\n2. **Compensation Support**:\n\n   - Enable/disable compensation\n   - Configure retries and timeouts\n   - Custom compensation logic\n\n3. **Structured Logging**:\n\n   - Using `Logger.warning/2` for better observability\n   - Context-aware logging\n   - Error tracking\n\n4. **Improved Error Handling**:\n   - Clear error paths with `with` statements\n   - Type-safe error returns\n   - Compensation strategies","ref":"actions.html#creating-your-first-action"},{"type":"extras","title":"Running Actions - Actions & Workflows","doc":"Jido provides several ways to execute Actions, each with specific benefits:","ref":"actions.html#running-actions"},{"type":"extras","title":"1. Direct Execution - Actions & Workflows","doc":"Best for testing and development - this does not provide any of the features of the workflow runtime.\n\n```elixir\n{:ok, result} = FormatUser.run(\n  %{\n    name: \"John Doe \",\n    email: \"JOHN@EXAMPLE.COM\",\n    age: 30\n  },\n  %{request_id: \"req_123\"}\n)\n```","ref":"actions.html#1-direct-execution"},{"type":"extras","title":"2. Workflow Runtime - Actions & Workflows","doc":"For production use with full features:\n\n```elixir\n{:ok, result} = Jido.Workflow.run(\n  FormatUser, # Action Module to run\n  %{name: \"John Doe\", email: \"john@example.com\", age: 30}, # Input parameters\n  %{request_id: \"req_123\"}, # Context\n  timeout: 5000, # Runtime opts, such as timeout & retries\n  max_retries: 2\n)\n```\n\nThe workflow runtime provides:\n\n- Enhanced telemetry events\n- IO operation safety checks\n- Task group management\n- Improved timeout handling\n- Configurable retries\n- Context propagation","ref":"actions.html#2-workflow-runtime"},{"type":"extras","title":"3. Async Execution - Actions & Workflows","doc":"For long-running operations:\n\n```elixir\n# Start async execution\nasync_ref = Jido.Workflow.run_async(\n  FormatUser,\n  params,\n  context,\n  timeout: 10_000\n)\n\n# Do other work...\n\n# Get result with timeout\n{:ok, result} = Jido.Workflow.await(async_ref, 5000)\n```","ref":"actions.html#3-async-execution"},{"type":"extras","title":"Building Complex Workflows - Actions & Workflows","doc":"Let's create a complete user registration workflow:\n\n```elixir\ndefmodule MyApp.Actions.EnrichUserData do\n  use Jido.Action,\n    name: \"enrich_user_data\",\n    description: \"Adds profile data to user record\",\n    category: \"user_management\",\n    tags: [\"user\", \"profile\"],\n    compensation: [enabled: true],\n    schema: [\n      formatted_name: [type: :string, required: true],\n      email: [type: :string, required: true]\n    ]\n\n  def run(%{formatted_name: name, email: email}, context) do\n    # Access task group from context if needed\n    task_group = context[:__task_group__]\n\n    with {:ok, username} <- generate_username(name, task_group),\n         {:ok, avatar_url} <- get_avatar_url(email, task_group) do\n      {:ok, %{\n        username: username,\n        avatar_url: avatar_url,\n        profile_created_at: DateTime.utc_now()\n      }}\n    end\n  end\n\n  def on_error(params, error, _context, _opts) do\n    Logger.warning(\"Compensating enrichment failure\",\n      error: inspect(error),\n      params: params\n    )\n    {:ok, %{compensated: true}}\n  end\nend\n\ndefmodule MyApp.Actions.NotifyUser do\n  use Jido.Action,\n    name: \"notify_user\",\n    description: \"Sends welcome notification\",\n    category: \"notifications\",\n    tags: [\"user\", \"email\"],\n    compensation: [enabled: true],\n    schema: [\n      email: [type: :string, required: true],\n      username: [type: :string, required: true]\n    ]\n\n  def run(params, context) do\n    :telemetry.execute(\n      [:myapp, :notification, :start],\n      %{system_time: System.system_time()},\n      %{email: params.email}\n    )\n\n    # Simulated notification\n    Process.sleep(100)\n\n    {:ok, %{\n      notification_sent: true,\n      notification_type: \"welcome_email\",\n      sent_at: DateTime.utc_now()\n    }}\n  end\nend\n```","ref":"actions.html#building-complex-workflows"},{"type":"extras","title":"Chaining Actions - Actions & Workflows","doc":"Jido provides sophisticated chain functionality:\n\n```elixir\nalias Jido.Workflow.Chain\n\n{:ok, result} = Chain.chain(\n  [\n    FormatUser,\n    {EnrichUserData, max_retries: 3},\n    {NotifyUser, timeout: 10_000}\n  ],\n  %{\n    name: \"John Doe\",\n    email: \"john@example.com\",\n    age: 30\n  },\n  context: %{\n    request_id: \"req_123\",\n    tenant_id: \"tenant_456\"\n  }\n)\n```\n\nKey chain features:\n\n- Per-action configuration\n- Enhanced context propagation\n- Better error handling\n- Task group management\n- IO safety monitoring","ref":"actions.html#chaining-actions"},{"type":"extras","title":"Testing Strategies - Actions & Workflows","doc":"Comprehensive testing strategies:\n\n```elixir\ndefmodule MyApp.Actions.UserRegistrationTest do\n  use ExUnit.Case, async: true\n  import Mimic\n\n  setup :verify_on_exit!\n\n  @valid_user_data %{\n    name: \"John Doe \",\n    email: \"JOHN@EXAMPLE.COM\",\n    age: 30\n  }\n\n  describe \"format user\" do\n    test \"formats and validates user data\" do\n      {:ok, result} = FormatUser.run(@valid_user_data, %{})\n\n      assert result.formatted_name == \"John Doe\"\n      assert result.email == \"john@example.com\"\n      assert result.is_adult == true\n    end\n\n    test \"handles invalid data\" do\n      {:error, error} = FormatUser.run(%{}, %{})\n      assert error.type == :validation_error\n    end\n  end\n\n  describe \"user registration workflow\" do\n    test \"executes complete workflow\" do\n      expect(NotifyUser, :run, fn params, _context ->\n        assert params.email == \"john@example.com\"\n        {:ok, %{notification_sent: true}}\n      end)\n\n      {:ok, result} = Chain.chain(\n        [FormatUser, EnrichUserData, NotifyUser],\n        @valid_user_data\n      )\n\n      assert result.notification_sent == true\n    end\n\n    test \"handles compensation on failure\" do\n      expect(EnrichUserData, :run, fn _params, _context ->\n        {:error, \"enrichment failed\"}\n      end)\n\n      {:error, error} = Chain.chain(\n        [FormatUser, EnrichUserData, NotifyUser],\n        @valid_user_data\n      )\n\n      assert error.type == :execution_error\n      assert error.details.compensated == true\n    end\n  end\nend\n```\n\nTesting features:\n\n- Enhanced Mimic integration\n- Task group testing\n- Compensation testing\n- Telemetry testing\n- IO safety verification","ref":"actions.html#testing-strategies"},{"type":"extras","title":"Advanced Features - Actions & Workflows","doc":"","ref":"actions.html#advanced-features"},{"type":"extras","title":"1. Task Groups - Actions & Workflows","doc":"Task groups provide better concurrency management:\n\n```elixir\ndefmodule MyApp.Actions.ParallelProcessor do\n  use Jido.Action,\n    name: \"parallel_processor\",\n    schema: [items: [type: {:list, :map}, required: true]]\n\n  def run(%{items: items}, %{__task_group__: group}) do\n    results = Task.Supervisor.async_stream_nolink(\n      group,\n      items,\n      &process_item/1,\n      ordered: false\n    )\n    |> Enum.to_list()\n\n    {:ok, %{results: results}}\n  end\nend\n```","ref":"actions.html#1-task-groups"},{"type":"extras","title":"2. Telemetry Integration - Actions & Workflows","doc":"Comprehensive telemetry support for monitoring:\n\n```elixir\n:telemetry.attach(\n  \"action-handler\",\n  [:jido, :workflow, :start],\n  fn name, measurements, metadata, _config ->\n    Logger.info(\"Action started\",\n      action: metadata.action,\n      duration_us: measurements.system_time\n    )\n  end,\n  nil\n)\n```","ref":"actions.html#2-telemetry-integration"},{"type":"extras","title":"3. IO Safety - Actions & Workflows","doc":"IO safety features prevent timeouts:\n\n```elixir\ndefmodule MyApp.Actions.SafeProcessor do\n  use Jido.Action,\n    name: \"safe_processor\"\n\n  def run(params, _context) do\n    # This would trigger an IO safety warning:\n    # IO.inspect(params)\n\n    # Instead, use Logger:\n    Logger.debug(\"Processing params\", params: params)\n    {:ok, process_data(params)}\n  end\nend\n```","ref":"actions.html#3-io-safety"},{"type":"extras","title":"Best Practices - Actions & Workflows","doc":"1. **Action Design**:\n\n   - Keep actions focused and single-purpose\n   - Use comprehensive metadata\n   - Enable compensation for critical operations\n   - Leverage task groups for concurrency\n   - Use structured logging\n\n2. **Error Handling**:\n\n   - Implement compensation strategies\n   - Use with statements for clarity\n   - Return structured errors\n   - Log errors with context\n   - Handle timeouts appropriately\n\n3. **Testing**:\n\n   - Test compensation paths\n   - Verify telemetry events\n   - Check IO safety\n   - Test task group behavior\n   - Use Mimic for mocking\n\n4. **Performance**:\n\n   - Monitor execution times\n   - Use async operations appropriately\n   - Configure timeouts properly\n   - Manage task group resources\n   - Implement backoff strategies\n\n5. **Observability**:\n   - Use structured logging\n   - Attach telemetry handlers\n   - Monitor compensation events\n   - Track task group usage\n   - Measure timing metrics","ref":"actions.html#best-practices"},{"type":"extras","title":"Next Steps - Actions & Workflows","doc":"Explore these advanced topics:\n\n- Custom runners for specialized workflows\n- Distributed workflow execution\n- Complex compensation strategies\n- Advanced telemetry patterns\n- Performance optimization\n\nThe test suite provides comprehensive examples of these patterns in action. For more details on specific features, consult the full API documentation.","ref":"actions.html#next-steps"},{"type":"extras","title":"Action Instructions","doc":"# Instructions Guide","ref":"instructions.html"},{"type":"extras","title":"Overview - Action Instructions","doc":"Instructions are the fundamental unit of execution in the Jido system. An Instruction wraps an action module with its parameters and execution context, allowing Runners to execute them in a standardized way. This guide covers instruction formats, normalization, and best practices.","ref":"instructions.html#overview"},{"type":"extras","title":"Instruction Structure - Action Instructions","doc":"A full instruction struct contains:\n\n```elixir\n%Jido.Instruction{\n  action: module(),      # The action module to execute (required)\n  params: map(),        # Parameters to pass to the action (default: %{})\n  context: map(),       # Execution context data (default: %{})\n  result: term()        # Execution result (default: nil)\n}\n```","ref":"instructions.html#instruction-structure"},{"type":"extras","title":"Instruction Formats - Action Instructions","doc":"Jido supports several shorthand formats for convenience, all of which are normalized to the full instruction struct during processing. Here are the supported formats:","ref":"instructions.html#instruction-formats"},{"type":"extras","title":"1. Action Module Only - Action Instructions","doc":"```elixir\n# Shorthand\nMyApp.Actions.ProcessFile\n\n# Normalizes to\n%Instruction{\n  action: MyApp.Actions.ProcessFile,\n  params: %{},\n  context: %{}\n}\n```","ref":"instructions.html#1-action-module-only"},{"type":"extras","title":"2. Action With Parameters (Tuple Form) - Action Instructions","doc":"```elixir\n# Shorthand\n{MyApp.Actions.ProcessFile, %{path: \"/tmp/data.csv\"}}\n\n# Normalizes to\n%Instruction{\n  action: MyApp.Actions.ProcessFile,\n  params: %{path: \"/tmp/data.csv\"},\n  context: %{}\n}\n```","ref":"instructions.html#2-action-with-parameters-tuple-form"},{"type":"extras","title":"3. Full Instruction Struct - Action Instructions","doc":"```elixir\n# Explicit struct\n%Instruction{\n  action: MyApp.Actions.ProcessFile,\n  params: %{path: \"/tmp/data.csv\"},\n  context: %{user_id: \"123\"}\n}\n```","ref":"instructions.html#3-full-instruction-struct"},{"type":"extras","title":"4. Lists of Mixed Formats - Action Instructions","doc":"```elixir\n# Mixed shorthand list\n[\n  ValidateAction,\n  {ProcessAction, %{file: \"data.csv\"}},\n  %Instruction{action: StoreAction, context: %{store_id: \"456\"}}\n]\n\n# Each element normalizes to a full instruction struct\n```","ref":"instructions.html#4-lists-of-mixed-formats"},{"type":"extras","title":"Normalization Process - Action Instructions","doc":"All instruction formats are normalized when:\n1. Planning actions on an Agent\n2. Directly executing through a Runner\n3. Creating instruction queues\n\nThe normalization ensures:\n- Consistent structure for execution\n- Parameter validation\n- Context preservation\n- Type safety\n- Serialization support","ref":"instructions.html#normalization-process"},{"type":"extras","title":"Best Practices - Action Instructions","doc":"","ref":"instructions.html#best-practices"},{"type":"extras","title":"When to Use Shorthand - Action Instructions","doc":"Use shorthand formats when:\n- Planning simple action sequences\n- Writing tests\n- Demonstrating examples\n- Working with basic workflows\n\n```elixir\n# Good use of shorthand\n{:ok, agent} = MyAgent.plan(agent, [\n  ValidateInput,\n  {ProcessData, %{format: \"csv\"}},\n  StoreResults\n])\n```","ref":"instructions.html#when-to-use-shorthand"},{"type":"extras","title":"When to Use Full Structs - Action Instructions","doc":"Use full instruction structs when:\n- Implementing custom runners\n- Building complex workflows\n- Needing explicit context control\n- Working with instruction queues directly\n\n```elixir\n# Good use of full struct\ninstruction = %Instruction{\n  action: ProcessData,\n  params: %{format: \"csv\"},\n  context: %{\n    request_id: \"abc123\",\n    user_id: \"456\",\n    tenant: \"acme\"\n  }\n}\n```","ref":"instructions.html#when-to-use-full-structs"},{"type":"extras","title":"Context Management - Action Instructions","doc":"The context map is preserved during normalization and is available to:\n- Action implementations\n- Runners\n- Error handlers\n- Telemetry events\n\n```elixir\n# Setting context during planning\n{:ok, agent} = MyAgent.plan(\n  agent,\n  ProcessAction,\n  %{                        # Context map\n    request_id: \"abc123\",\n    user_id: \"456\"\n  }\n)\n```","ref":"instructions.html#context-management"},{"type":"extras","title":"Working with Instructions Programmatically - Action Instructions","doc":"When manipulating instructions in code:\n\n```elixir\n# Pattern match on full struct\ndef process_instruction(%Instruction{action: action, params: params}) do\n  # Work with normalized form\nend\n\n# Transform instructions\ndef add_context(instructions, context) do\n  Enum.map(instructions, fn\n    %Instruction{} = inst -> \n      %{inst | context: Map.merge(inst.context, context)}\n    {action, params} -> \n      %Instruction{action: action, params: params, context: context}\n    action when is_atom(action) -> \n      %Instruction{action: action, context: context}\n  end)\nend\n```","ref":"instructions.html#working-with-instructions-programmatically"},{"type":"extras","title":"Error Handling - Action Instructions","doc":"Instructions provide rich error context:\n\n```elixir\ncase MyAgent.plan(agent, invalid_instruction) do\n  {:ok, agent} -> \n    # Success case\n  {:error, %Error{type: :validation_error, context: context}} ->\n    # Handle validation failure with full context\nend\n```","ref":"instructions.html#error-handling"},{"type":"extras","title":"Testing Instructions - Action Instructions","doc":"Test both shorthand and normalized forms:\n\n```elixir\ntest \"supports shorthand planning\" do\n  {:ok, agent} = MyAgent.plan(agent, SimpleAction)\n  assert [%Instruction{action: SimpleAction}] = \n    :queue.to_list(agent.pending_instructions)\nend\n\ntest \"preserves context in normalization\" do\n  context = %{request_id: \"123\"}\n  {:ok, agent} = MyAgent.plan(agent, SimpleAction, context)\n  \n  [instruction] = :queue.to_list(agent.pending_instructions)\n  assert instruction.context.request_id == \"123\"\nend\n```","ref":"instructions.html#testing-instructions"},{"type":"extras","title":"Summary - Action Instructions","doc":"- Use shorthand formats for convenience in simple cases\n- Work with full instruction structs in implementation code\n- Trust the normalization process to handle all formats consistently\n- Leverage the context map for cross-cutting concerns\n- Test both shorthand and normalized forms\n- Handle errors with full context\n\nRemember: Instructions are always normalized before execution, so choose the format that makes your code most readable and maintainable in each specific situation.","ref":"instructions.html#summary"},{"type":"extras","title":"Signal Bus","doc":"# Signal Bus\n\nIn our previous guides, we explored how Actions provide composable building blocks, how Agents manage state, and how Signals enable real-time monitoring. Now, let's discover how the Signal Bus ties these components together by providing a robust messaging infrastructure.","ref":"bus.html"},{"type":"extras","title":"Understanding the Signal Bus - Signal Bus","doc":"At its core, the Jido Signal Bus is a message routing and persistence system that enables:\n\n- Publishing and subscribing to signal streams\n- Persistent message storage and replay\n- Real-time signal distribution\n- State snapshots for recovery\n- Flexible adapter architecture\n\nThink of the Signal Bus as the nervous system of your Jido application - it ensures signals flow reliably between components while maintaining message ordering and persistence guarantees.","ref":"bus.html#understanding-the-signal-bus"},{"type":"extras","title":"Key Concepts - Signal Bus","doc":"The Signal Bus is built around several key abstractions:\n\n1. **Signals**: Standardized message format following CloudEvents spec\n2. **Streams**: Ordered sequences of signals\n3. **Subscriptions**: Ways to consume signals (transient or persistent)\n4. **Snapshots**: Point-in-time state captures\n5. **Adapters**: Pluggable backend implementations","ref":"bus.html#key-concepts"},{"type":"extras","title":"Getting Started - Signal Bus","doc":"Let's see how to use the Signal Bus in practice:\n\n```elixir\n# Start the bus with an in-memory adapter\nchildren = [\n  {Jido.Bus, name: :my_bus, adapter: :in_memory}\n]\n\n# Or with Phoenix.PubSub for distributed scenarios\nchildren = [\n  {Jido.Bus, name: :my_bus, adapter: :pubsub, pubsub_name: MyApp.PubSub}\n]\n```","ref":"bus.html#getting-started"},{"type":"extras","title":"Publishing Signals - Signal Bus","doc":"Signals can be published to named streams:\n\n```elixir\nsignal = %Jido.Signal{\n  id: Jido.Util.generate_id(),\n  source: \"user_service\",\n  type: \"user.registered\",\n  data: %{user_id: \"123\", email: \"user@example.com\"},\n  metadata: %{client_ip: \"127.0.0.1\"}\n}\n\n:ok = Jido.Bus.publish(:my_bus, \"user_stream\", :any_version, [signal])\n```\n\nThe `:any_version` parameter tells the bus to append without checking versions. You can also use:\n\n- `:no_stream` - Requires stream doesn't exist\n- `:stream_exists` - Requires stream exists\n- Integer version - Requires exact version match","ref":"bus.html#publishing-signals"},{"type":"extras","title":"Subscribing to Signals - Signal Bus","doc":"The bus supports two types of subscriptions:\n\n#### 1. Transient Subscriptions\n\nBest for temporary subscribers that don't need persistence:\n\n```elixir\n# Subscribe to a single stream\n:ok = Jido.Bus.subscribe(:my_bus, \"user_stream\")\n\n# Receive signals\nreceive do\n  {:signals, signals} ->\n    Enum.each(signals, &process_signal/1)\nend\n```\n\n#### 2. Persistent Subscriptions\n\nFor durable subscribers that need guaranteed delivery:\n\n```elixir\n{:ok, subscription} = Jido.Bus.subscribe_persistent(\n  :my_bus,                # Bus name\n  \"user_stream\",          # Stream to subscribe to\n  \"user_processor\",       # Subscription name\n  self(),                # Subscriber pid\n  :origin,               # Start from beginning\n  [                      # Options\n    concurrency_limit: 2,\n    partition_by: &partition_function/1\n  ]\n)\n\n# Receive signals and acknowledge processing\nreceive do\n  {:signals, signals} ->\n    process_signals(signals)\n    Jido.Bus.ack(:my_bus, subscription, List.last(signals))\nend\n```\n\nThe subscription options include:\n\n- `:origin` - Start from beginning\n- `:current` - Start from now\n- Integer position - Start from specific point\n- `concurrency_limit` - Max concurrent subscribers\n- `partition_by` - Function to partition signals","ref":"bus.html#subscribing-to-signals"},{"type":"extras","title":"Replaying Signals - Signal Bus","doc":"The bus supports replaying historical signals:\n\n```elixir\n# Replay all signals from start\nsignals = Jido.Bus.replay(:my_bus, \"user_stream\")\n\n# Replay from specific version\nsignals = Jido.Bus.replay(:my_bus, \"user_stream\", 100)\n\n# Control batch size\nsignals = Jido.Bus.replay(:my_bus, \"user_stream\", 100, 50)\n\nEnum.each(signals, &process_signal/1)\n```","ref":"bus.html#replaying-signals"},{"type":"extras","title":"Working with Snapshots - Signal Bus","doc":"Snapshots provide point-in-time state capture:\n\n```elixir\n# Record a snapshot\nsnapshot = %Jido.Bus.Snapshot{\n  source_id: \"user_123\",\n  source_version: 50,\n  source_type: \"UserAggregate\",\n  data: serialize_state(current_state),\n  created_at: DateTime.utc_now()\n}\n\n:ok = Jido.Bus.record_snapshot(:my_bus, snapshot)\n\n# Read snapshot\n{:ok, snapshot} = Jido.Bus.read_snapshot(:my_bus, \"user_123\")\n\n# Delete snapshot\n:ok = Jido.Bus.delete_snapshot(:my_bus, \"user_123\")\n```","ref":"bus.html#working-with-snapshots"},{"type":"extras","title":"Bus Adapters - Signal Bus","doc":"The Signal Bus supports multiple backend adapters:","ref":"bus.html#bus-adapters"},{"type":"extras","title":"In-Memory Adapter - Signal Bus","doc":"Best for development and testing:\n\n- Full persistence and replay support\n- Fast local operation\n- Memory-bound storage\n- No distribution\n\n```elixir\n{Jido.Bus, name: :my_bus, adapter: :in_memory}\n```","ref":"bus.html#in-memory-adapter"},{"type":"extras","title":"PubSub Adapter - Signal Bus","doc":"Best for distributed scenarios:\n\n- Built on Phoenix.PubSub\n- Supports clustering\n- Real-time message distribution\n- No persistence/replay\n\n```elixir\n{Jido.Bus, name: :my_bus, adapter: :pubsub, pubsub_name: MyApp.PubSub}\n```","ref":"bus.html#pubsub-adapter"},{"type":"extras","title":"Agents","doc":"# Agents\n\nIn our previous guide, we explored how Actions serve as composable building blocks for workflows. Now, let's discover how Agents bring these Actions to life by providing a stateful, intelligent wrapper around them.","ref":"agents.html"},{"type":"extras","title":"What are Agents? - Agents","doc":"At their core, Jido Agents are simply data structures that understand how to maintain and transform their own state. While you might be familiar with Elixir's built-in Agent processes for state management, Jido Agents take a layered approach. The `Jido.Agent` module provides the foundational data structure, and the `Jido.Agent.Server` provides the GenServer implementation that manages the Agent's runtime state within OTP.\n\nThink of a Jido Agent as a specialized map that knows two important things:\n1. What shape its data should have (through its schema)\n2. What transformations it can apply to that data (through its registered Actions)\n\nWhen you create an Agent, you're not starting a process or service - you're creating a data structure that can validate its own contents and understands how to transform itself through Actions. This pure data approach means Agents are predictable, testable, and can be easily serialized or persisted.","ref":"agents.html#what-are-agents"},{"type":"extras","title":"Agents as Data Transformation Engines - Agents","doc":"The fundamental job of an Agent is to transform data in controlled, predictable ways. It does this through a simple pattern:\n\n1. It holds some initial state (input data)\n2. It applies one or more Actions to transform that data\n3. It either returns the transformed data or updates its own state\n\nFor example, imagine an Agent processing user registration data. It might:\n1. Start with raw input: `%{name: \"JOHN DOE \", email: \"JOHN@EXAMPLE.COM\"}`\n2. Apply a formatting Action that normalizes the data\n3. Store or return the result: `%{name: \"John Doe\", email: \"john@example.com\"}`","ref":"agents.html#agents-as-data-transformation-engines"},{"type":"extras","title":"The Role of State - Agents","doc":"An Agent's state is just a schema-validated map. The schema serves several purposes:\n- It documents what fields the Agent can contain\n- It ensures data consistency through type validation\n- It provides default values for fields\n- It makes the Agent's purpose clear to other developers\n\nThe schema isn't just for validation - it's the Agent's contract with the rest of your system about what data it manages and how that data should look.\n\n#### State Validation Modes\n\nBy default, Agents use a permissive validation approach - they validate fields defined in the schema while allowing additional unknown fields. This flexibility supports development workflows and experimental features. For example, you might want to temporarily store debugging information or track experimental metrics without modifying your schema.\n\nHowever, when you need stricter guarantees about your data, you can enable strict validation mode when setting state. In strict mode, the Agent will reject any fields not defined in its schema. This is valuable when:\n\n- You need to guarantee complete state consistency\n- You want to catch typos or mistakes in field names early\n- You're working with sensitive data where extra fields could be problematic\n- You want to maintain a strict contract about what data an Agent manages\n\nYou control this behavior through the `strict_validation` option when setting state, which we'll see in practice shortly.","ref":"agents.html#the-role-of-state"},{"type":"extras","title":"Actions and Transformation - Agents","doc":"While the Agent holds state, Actions do the actual work of transformation. The Agent's job is to:\n- Validate that an Action is registered and can be used\n- Provide its current state as input to the Action\n- Validate and store (or return) the Action's output\n\nThink of it like a pipeline where data flows through one or more transformations, with the Agent ensuring each step maintains data integrity.","ref":"agents.html#actions-and-transformation"},{"type":"extras","title":"Planning and Execution - Agents","doc":"One key feature of Agents is their ability to plan sequences of transformations before executing them. When you plan Actions on an Agent, you're creating a queue of transformations to apply. This separation between planning and execution lets you:\n\n1. Validate the entire transformation sequence before running it\n2. Ensure Actions will receive the right input at each step\n3. Choose whether to apply results to Agent state or just return them\n4. Control how results flow between Actions in the sequence\n\nThe actual execution happens through a Runner, which orchestrates how the transformations occur and how results flow between steps.","ref":"agents.html#planning-and-execution"},{"type":"extras","title":"Creating Your First Agent - Agents","doc":"Let's create a simple User Registration Agent that will help us understand these concepts. We'll start with the same user registration workflow from our Actions guide, but now we'll wrap it in an Agent:\n\n```elixir\ndefmodule MyApp.UserAgent do\n  use Jido.Agent,\n    name: \"user_agent\",\n    description: \"Manages user registration\",\n    # Actions this agent can use\n    actions: [FormatUser, EnrichUserData, NotifyUser],\n    # State schema\n    schema: [\n      # Input fields\n      name: [\n        type: {:or, [:string, nil]}, \n        default: nil,\n        doc: \"User's raw input name\"\n      ],\n      email: [\n        type: {:or, [:string, nil]}, \n        default: nil,\n        doc: \"User's raw input email\"\n      ],\n      age: [\n        type: {:or, [:integer, nil]}, \n        default: nil,\n        doc: \"User's age in years\"\n      ],\n      \n      # Fields that will store action results\n      formatted_name: [\n        type: {:or, [:string, nil]}, \n        default: nil,\n        doc: \"Name after formatting\"\n      ],\n      username: [\n        type: {:or, [:string, nil]}, \n        default: nil,\n        doc: \"Generated username\"\n      ],\n      notification_sent: [\n        type: :boolean, \n        default: false,\n        doc: \"Whether welcome notification was sent\"\n      ]\n    ]\nend\n```\n\nLet's break down this definition:\n\n1. The `name` and `description` help other parts of the system understand what this Agent does\n2. The `actions` list declares which Actions this Agent can use\n3. The `schema` defines what information this Agent can remember, including both input fields and expected action results","ref":"agents.html#creating-your-first-agent"},{"type":"extras","title":"Working with Agents - Agents","doc":"Now that we have our Agent defined, let's see how to use it. Working with Agents follows a natural progression of create → set → plan → run.","ref":"agents.html#working-with-agents"},{"type":"extras","title":"Creating an Agent - Agents","doc":"First, we create a new instance of our Agent:\n\n```elixir\n# Create a new agent\nagent = MyApp.UserAgent.new()\n\n# It starts with default values\nagent.state.name         #=> nil\nagent.state.email       #=> nil\nagent.state.username    #=> nil\n\n# Each agent has a unique ID\nagent.id  #=> \"ag_123...\"\n```","ref":"agents.html#creating-an-agent"},{"type":"extras","title":"Setting State - Agents","doc":"Before we can process anything, we need to give our Agent some data to work with. We can use `set/2` in either permissive or strict mode:\n\n```elixir\n# Default permissive mode allows unknown fields\n{:ok, agent} = MyApp.UserAgent.set(agent, %{\n  name: \"John Doe\",\n  email: \"john@example.com\",\n  debug_info: %{source: \"test\"}  # Not in schema, but allowed\n})\n\n# Strict mode rejects unknown fields\n{:error, error} = MyApp.UserAgent.set(agent, \n  %{\n    name: \"John Doe\",\n    unknown_field: true\n  },\n  strict_validation: true\n)\n\n# The error clearly identifies rejected fields\nassert error.message =~ \"Unknown fields: [:unknown_field]\"\n```\n\nState updates follow these rules:\n- Each update returns a new immutable copy of the agent\n- Schema fields are always validated\n- Unknown fields are allowed by default but can be rejected with strict validation\n- Multiple fields can be updated at once\n- Existing fields are preserved unless explicitly changed","ref":"agents.html#setting-state"},{"type":"extras","title":"Planning Actions - Agents","doc":"Once our Agent has some state, we can plan what Actions it should take:\n\n```elixir\n# Plan a single action using current state\n{:ok, agent} = MyApp.UserAgent.plan(agent, FormatUser)\n\n# Plan multiple actions with explicit parameters\n{:ok, agent} = MyApp.UserAgent.plan(agent, [\n  {FormatUser, agent.state},\n  EnrichUserData,\n  NotifyUser\n])\n\n# Planning doesn't execute - state remains unchanged\nassert agent.state.formatted_name == nil\n```\n\nThe planning phase lets us:\n- Validate that all actions are registered\n- Set up the transformation sequence\n- Prepare parameters for each step\n- Check for obvious problems before execution\n\n> Why doesn't the agent pass it's state to the first action?\n>\n> Great question - and there are two reasons for this:\n> 1. The Runner is the component responsible for passing state to the Action - so in our contrived example, it's important to demonstrate that the state is being explicitly passed\n> 2. The Agent has sensible defaults that try to remove as many assumptions and require explicit programming.  Passing the state by default is a good example of this.  If you want to pass the state to the first action, you can do so explicitly.","ref":"agents.html#planning-actions"},{"type":"extras","title":"Running Actions - Agents","doc":"Once we've planned our Actions, we can execute them:\n\n```elixir\n# Run and apply results to state\n{:ok, agent} = MyApp.UserAgent.run(agent, \n  apply_state: true,\n  runner: Jido.Runner.Chain\n)\n\n# Verify transformations happened\nassert agent.state.formatted_name == \"John Doe\"\nassert agent.state.username == \"john.doe\"\n```\n\nOr we can run without updating state to inspect the results first:\n\n```elixir\n# Run without applying state changes\n{:ok, agent} = MyApp.UserAgent.run(agent, apply_state: false)\n\n# State unchanged but results available\nassert agent.state.formatted_name == nil\nassert agent.result.result_state.formatted_name == \"John Doe\"\n```\n\nThis separation between execution and state updates gives us control over when and how our Agent's state changes.","ref":"agents.html#running-actions"},{"type":"extras","title":"Putting It All Together: Commands - Agents","doc":"While we can use `set`, `plan`, and `run` separately, Jido provides a convenient `cmd/4` function that combines them:\n\n```elixir\n{:ok, agent} = MyApp.UserAgent.cmd(\n  agent,\n  [{FormatUser, agent.state}, EnrichUserData],  # Actions to run\n  %{age: 30},                                   # State to set\n  apply_state: true,                           # Update state with results\n  runner: Jido.Runner.Chain                    # Use chain runner\n)\n\n# Everything happened in one step\nassert agent.state.formatted_name == \"John Doe\"\nassert agent.state.username == \"john.doe\"\nassert agent.state.age == 30\n```","ref":"agents.html#putting-it-all-together-commands"},{"type":"extras","title":"Best Practices - Agents","doc":"As you build with Agents, keep these principles in mind:\n\n1. Use strict validation when data consistency is critical and permissive validation during development or for temporary data.\n\n2. Keep your schema focused on the data your Agent truly needs to manage. Just because you can store additional fields doesn't mean you should.\n\n3. Let your Actions handle complex transformations and use `set/2` primarily for direct state updates.\n\n4. Take advantage of the separation between planning and execution to validate operations before running them.\n\n5. Consider whether to apply results to state based on your use case - sometimes you want to inspect results before committing them.","ref":"agents.html#best-practices"},{"type":"extras","title":"Next Steps - Agents","doc":"Now that you understand the basics of Agents, you can explore:\n- Complex action chains\n- Conditional execution paths\n- Error handling and recovery\n- State persistence\n- Agent supervision and lifecycle\n- Inter-agent communication\n\nThe test suite provides many examples of these advanced patterns. Remember: Agents are most powerful when they maintain focused state and use well-defined Actions to transform that state. Keep your Agents focused, their state clean, and their Actions clear.","ref":"agents.html#next-steps"},{"type":"extras","title":"Agent Directives","doc":"# Agent Directives\n\nIn our previous guides, we explored how Agents provide stateful wrappers around Actions and how Signals enable real-time monitoring. Now, let's discover how Directives enable agents to modify their own behavior and capabilities at runtime.","ref":"directives.html"},{"type":"extras","title":"Understanding Directives - Agent Directives","doc":"Directives are special instructions that allow agents to modify their own state and capabilities. Think of them as meta-actions that let agents:\n\n1. Queue up new actions to execute (Enqueue)\n2. Learn new capabilities (RegisterAction) \n3. Remove capabilities (DeregisterAction)\n\nThis self-modification ability is crucial for building adaptive agents that can:\n- Respond to new situations by queueing appropriate actions\n- Learn new behaviors by registering actions\n- Optimize themselves by removing unused capabilities\n- Build complex workflows dynamically\n\nLet's see how to implement these patterns.","ref":"directives.html#understanding-directives"},{"type":"extras","title":"Creating Your First Directive-Based Agent - Agent Directives","doc":"We'll create an agent that can adapt its behavior by learning new arithmetic operations. This will demonstrate how directives enable runtime evolution of agent capabilities.\n\n```elixir\ndefmodule MyApp.AdaptiveCalculator do\n  use Jido.Agent,\n    name: \"adaptive_calculator\",\n    description: \"A calculator that can learn new operations\",\n    actions: [\n      # Core actions for self-modification\n      Jido.Actions.Directives.EnqueueAction,\n      Jido.Actions.Directives.RegisterAction,\n      Jido.Actions.Directives.DeregisterAction,\n      \n      # Initial arithmetic capabilities\n      MyApp.Actions.Add,\n      MyApp.Actions.Subtract\n    ],\n    schema: [\n      value: [type: :float, default: 0.0],\n      operations_used: [type: {:list, :atom}, default: []],\n      last_operation: [type: {:or, [:atom, nil]}, default: nil]\n    ]\n\n  # Optional callbacks for tracking operations\n  def on_after_run(agent, result) do\n    case result do\n      %{status: :ok, action: action} ->\n        operations = [action | agent.state.operations_used] |> Enum.uniq()\n        {:ok, %{agent | state: Map.put(agent.state, :operations_used, operations)}}\n      _ ->\n        {:ok, agent}\n    end\n  end\nend\n```\n\nNow let's implement some basic arithmetic actions that can work with our calculator:\n\n```elixir\ndefmodule MyApp.Actions.Add do\n  use Jido.Action,\n    name: \"add\",\n    description: \"Adds a number to the current value\",\n    schema: [\n      value: [type: :float, required: true],\n      amount: [type: :float, required: true]\n    ]\n\n  def run(%{value: current, amount: amount}, _context) do\n    {:ok, %{value: current + amount}}\n  end\nend\n\ndefmodule MyApp.Actions.Multiply do\n  use Jido.Action,\n    name: \"multiply\",\n    description: \"Multiplies the current value by an amount\",\n    schema: [\n      value: [type: :float, required: true],\n      amount: [type: :float, required: true]\n    ]\n\n  def run(%{value: current, amount: amount}, _context) do\n    {:ok, %{value: current * amount}}\n  end\nend\n\ndefmodule MyApp.Actions.Power do\n  use Jido.Action,\n    name: \"power\",\n    description: \"Raises the current value to a power\",\n    schema: [\n      value: [type: :float, required: true],\n      exponent: [type: :float, required: true]\n    ]\n\n  def run(%{value: current, exponent: exp}, _context) do\n    {:ok, %{value: :math.pow(current, exp)}}\n  end\nend\n```","ref":"directives.html#creating-your-first-directive-based-agent"},{"type":"extras","title":"Working with Directives - Agent Directives","doc":"Let's explore how our calculator can use directives to evolve its capabilities:","ref":"directives.html#working-with-directives"},{"type":"extras","title":"1. Queueing New Operations (Enqueue) - Agent Directives","doc":"The Enqueue lets an agent add new instructions to its pending queue. This is useful for building dynamic workflows:\n\n```elixir\n# Create our calculator\ncalculator = MyApp.AdaptiveCalculator.new()\n\n# Start with a simple addition\n{:ok, calculator} = MyApp.AdaptiveCalculator.set(calculator, %{value: 5})\n\n# Queue up a sequence of operations using EnqueueAction\n{:ok, calculator} = MyApp.AdaptiveCalculator.cmd(\n  calculator,\n  [\n    # Utilize a directive to queue up an instruction rather than planning it directly\n    {Jido.Actions.Directives.EnqueueAction, %{\n      action: MyApp.Actions.Add,\n      params: %{amount: 10}\n    }},\n    {Jido.Actions.Directives.EnqueueAction, %{\n      action: MyApp.Actions.Add,\n      params: %{amount: 20}\n    }}\n  ]\n)\n\n# Final value should be 35 (5 + 10 + 20)\ncalculator.state.value #=> 35.0\n```","ref":"directives.html#1-queueing-new-operations-enqueue"},{"type":"extras","title":"2. Learning New Operations (RegisterAction) - Agent Directives","doc":"The RegisterAction lets an agent learn new capabilities at runtime:\n\n```elixir\n# Our calculator doesn't know how to multiply yet\n{:error, _} = MyApp.AdaptiveCalculator.plan(calculator, MyApp.Actions.Multiply)\n\n# Teach it multiplication\n{:ok, calculator} = MyApp.AdaptiveCalculator.cmd(\n  calculator,\n  {Jido.Actions.Directives.RegisterAction, %{\n    action_module: MyApp.Actions.Multiply\n  }}\n)\n\n# Now we can multiply!\n{:ok, calculator} = MyApp.AdaptiveCalculator.cmd(\n  calculator,\n  {MyApp.Actions.Multiply, %{amount: 2}}\n)\n\ncalculator.state.value #=> 70.0  # (35 * 2)\n```","ref":"directives.html#2-learning-new-operations-registeraction"},{"type":"extras","title":"3. Removing Operations (DeregisterAction) - Agent Directives","doc":"The DeregisterAction lets an agent remove capabilities it no longer needs:\n\n```elixir\n# Remove multiplication if we don't need it anymore\n{:ok, calculator} = MyApp.AdaptiveCalculator.cmd(\n  calculator,\n  {Jido.Actions.Directives.DeregisterAction, %{\n    action_module: MyApp.Actions.Multiply\n  }}\n)\n\n# Trying to multiply now will fail\n{:error, _} = MyApp.AdaptiveCalculator.plan(calculator, MyApp.Actions.Multiply)\n```","ref":"directives.html#3-removing-operations-deregisteraction"},{"type":"extras","title":"Building Complex Self-Modifying Workflows - Agent Directives","doc":"Directives become really powerful when combined into workflows that let agents adapt their behavior based on conditions. Here's an example of a calculator that learns more advanced operations when needed:\n\n```elixir\ndefmodule MyApp.Actions.LearnAdvancedMath do\n  use Jido.Action,\n    name: \"learn_advanced_math\",\n    description: \"Teaches the calculator advanced operations\",\n    schema: [\n      value: [type: :float, required: true]\n    ]\n\n  def run(%{value: value}, _context) do\n    # First register the power operation\n    power_directive = %Jido.Agent.Directive.RegisterAction{\n      action_module: MyApp.Actions.Power\n    }\n\n    # Then queue up a calculation using it\n    calculate_directive = %Jido.Agent.Directive.Enqueue{\n      action: MyApp.Actions.Power,\n      params: %{value: value, exponent: 2}\n    }\n\n    # Return both directives to be applied in order\n    {:ok, [power_directive, calculate_directive]}\n  end\nend\n\n# Use our advanced learning action\ncalculator = MyApp.AdaptiveCalculator.new()\n{:ok, calculator} = MyApp.AdaptiveCalculator.set(calculator, %{value: 5})\n\n# Learn and apply advanced math\n{:ok, calculator} = MyApp.AdaptiveCalculator.cmd(\n  calculator,\n  [\n    {MyApp.Actions.LearnAdvancedMath, %{}},  # This will register Power and queue its use\n    {Jido.Actions.Directives.EnqueueAction, %{  # Then we'll queue another operation\n      action: MyApp.Actions.Add,\n      params: %{amount: 10}\n    }}\n  ]\n)\n\n# Final value: 5^2 + 10 = 35\ncalculator.state.value #=> 35.0\n```","ref":"directives.html#building-complex-self-modifying-workflows"},{"type":"extras","title":"Best Practices - Agent Directives","doc":"When working with directives, keep these principles in mind:\n\n1. **Capability Management**\n   - Only register actions the agent actually needs\n   - Consider deregistering unused actions to keep the agent focused\n   - Track which operations are most frequently used\n\n2. **Directive Chains**\n   - Order directives carefully - registration must happen before usage\n   - Consider using composite actions for complex directive sequences\n   - Validate directive success before proceeding\n\n3. **State Evolution**\n   - Keep track of how agent capabilities change over time\n   - Consider implementing rollback mechanisms for failed directive chains\n   - Use callbacks to monitor and log capability changes\n\n4. **Testing**\n   - Test both successful and failed directive applications\n   - Verify capability addition and removal\n   - Test complex directive chains\n   - Check state consistency after directive application","ref":"directives.html#best-practices"},{"type":"extras","title":"Next Steps - Agent Directives","doc":"Now that you understand directives, you can explore:\n- More complex self-modification patterns\n- Conditional capability loading\n- Dynamic workflow construction\n- State-based capability management\n- Multi-agent capability sharing\n\nRemember: Directives give agents the power to evolve and adapt. Use them thoughtfully to create agents that can grow and optimize themselves while maintaining stability and predictability.","ref":"directives.html#next-steps"},{"type":"extras","title":"Agent Sensors","doc":"# Signals and Sensors\n\nIn our previous guides, we explored how Actions serve as composable building blocks and how Agents provide stateful wrappers around them. Now, let's discover how Signals and Sensors enable real-time event monitoring and metrics collection in your Jido system.","ref":"sensors.html"},{"type":"extras","title":"Understanding Signals and Sensors - Agent Sensors","doc":"Before we dive into code, let's understand what Signals and Sensors are and why they're important in a distributed system.","ref":"sensors.html#understanding-signals-and-sensors"},{"type":"extras","title":"What are Signals? - Agent Sensors","doc":"At their core, Signals are standardized event messages that flow through your Jido system. They implement the CloudEvents specification (v1.0.2) with Jido-specific extensions, providing a consistent format for all system events. Think of Signals as the nervous system of your application - they carry information about:\n\n- State changes\n- Metric updates\n- System events\n- Process lifecycle events\n- Command execution results\n\nEvery Signal includes standard CloudEvents fields like:\n- `id`: A unique identifier\n- `source`: Where the signal originated\n- `type`: What kind of event it represents\n- `subject`: What entity the signal relates to\n- `data`: The actual event payload\n\nPlus Jido-specific fields for:\n- `jido_instructions`: Instructions for the Agent to execute\n- `jido_opts`: Options for signal processing","ref":"sensors.html#what-are-signals"},{"type":"extras","title":"Why Sensors? - Agent Sensors","doc":"Sensors are independent GenServer processes that gather external information and update agent state independently. This architectural choice is crucial for distributed systems where a single agent might have dozens of sensors feeding it information simultaneously - from API monitoring and metrics tracking to event stream processing.\n\nRunning sensors as standalone processes under OTP supervision provides key benefits:\n- **Fault Isolation**: A failing sensor won't crash the agent or other sensors\n- **Independent Scaling**: Sensors can be distributed across nodes based on load\n- **Resource Management**: Each sensor manages its own memory and process queue\n- **Dynamic Operation**: Sensors can be started, stopped, and supervised independently\n\nWhile this guide covers defining and starting individual sensors, later guides will explain how agents dynamically manage multiple sensors through registration, lifecycle management, and event routing.\n\nSensors serve several purposes:\n- Track metrics and state changes in real-time\n- Process and aggregate event streams\n- Generate alerts for important conditions\n- Maintain recent event history\n\nA key role of Sensors is translating external events into standardized Signals that agents can understand and consume. For example, a sensor might:\n- Listen for HTTP webhook events and emit corresponding agent-ready signals\n- Watch a message queue and transform messages into signals\n- Monitor file changes and generate file-event signals\n- Subscribe to external event streams and normalize them into signals\n\nThink of sensors as specialized translators that convert various external events into a consistent Signal format that agents know how to process, all while operating independently under OTP supervision.","ref":"sensors.html#why-sensors"},{"type":"extras","title":"Creating Your First Sensor - Agent Sensors","doc":"Let's build on our user registration example by creating a Sensor that monitors registration success rates. This will help us understand usage patterns and identify potential issues.\n\n```elixir\ndefmodule MyApp.Sensors.RegistrationCounter do\n  @moduledoc \"\"\"\n  Tracks user registration success and failure metrics.\n  \"\"\"\n  use Jido.Sensor,\n    name: \"registration_counter\",\n    description: \"Monitors registration successes and failures\",\n    category: :metrics,\n    tags: [:registration, :counter],\n    vsn: \"1.0.0\",\n    schema: [\n      emit_interval: [\n        type: :pos_integer,\n        default: 1000,\n        doc: \"Interval between metric emissions in ms\"\n      ]\n    ]\n\n  def mount(opts) do\n    state = Map.merge(opts, %{\n      successful: 0,\n      failed: 0\n    })\n    \n    schedule_emit(state)\n    {:ok, state}\n  end\n\n  def generate_signal(state) do\n    total = state.successful + state.failed\n    success_rate = if total > 0, do: state.successful / total * 100, else: 0\n\n    Jido.Signal.new(%{\n      source: \"#{state.sensor.name}:#{state.id}\",\n      subject: \"registration_counts\",\n      type: \"registration.metrics\",\n      data: %{\n        successful: state.successful,\n        failed: state.failed,\n        total: total,\n        success_rate: success_rate\n      }\n    })\n  end\n\n  def handle_info(:emit, state) do\n    with {:ok, signal} <- generate_signal(state),\n         :ok <- Phoenix.PubSub.broadcast(state.pubsub, state.topic, {:sensor_signal, signal}) do\n      schedule_emit(state)\n      {:noreply, state}\n    else\n      error ->\n        Logger.warning(\"Error generating/publishing signal: #{inspect(error)}\")\n        schedule_emit(state)\n        {:noreply, state}\n    end\n  end\n\n  def handle_info({:registration, :success}, state) do\n    new_state = %{state | successful: state.successful + 1}\n    \n    with {:ok, signal} <- generate_signal(new_state),\n         :ok <- Phoenix.PubSub.broadcast(state.pubsub, state.topic, {:sensor_signal, signal}) do\n      {:noreply, new_state}\n    else\n      error ->\n        Logger.warning(\"Error broadcasting success signal: #{inspect(error)}\")\n        {:noreply, new_state}\n    end\n  end\n\n  def handle_info({:registration, :failure}, state) do\n    new_state = %{state | failed: state.failed + 1}\n    \n    with {:ok, signal} <- generate_signal(new_state),\n         :ok <- Phoenix.PubSub.broadcast(state.pubsub, state.topic, {:sensor_signal, signal}) do\n      {:noreply, new_state}\n    else\n      error ->\n        Logger.warning(\"Error broadcasting failure signal: #{inspect(error)}\")\n        {:noreply, new_state}\n    end\n  end\n\n  defp schedule_emit(state) do\n    Process.send_after(self(), :emit, state.emit_interval)\n  end\nend\n```\n\nLet's break down what's happening in this Sensor:\n\n1. We use `use Jido.Sensor` to define our Sensor, providing metadata like:\n   - `name`: Unique identifier\n   - `description`: What the Sensor monitors\n   - `category`: Classification for grouping\n   - `tags`: Labels for filtering\n   - `schema`: Configuration options\n\n2. The `mount/1` callback initializes our Sensor's state with:\n   - Counters for successes and failures\n   - Configuration from schema\n   - Scheduled metric emissions\n\n3. `generate_signal/1` creates standardized Signal structs containing:\n   - Current metric values\n   - Calculated success rate\n   - Source identification\n   - Event type and subject\n\n4. Message handlers track events and emit signals:\n   - `:emit` for regular metric updates\n   - `{:registration, :success}` for successful registrations\n   - `{:registration, :failure}` for failed registrations","ref":"sensors.html#creating-your-first-sensor"},{"type":"extras","title":"Working with Signals - Agent Sensors","doc":"Signals provide a standardized way to represent events in your system. Let's look at how to create and handle them:","ref":"sensors.html#working-with-signals"},{"type":"extras","title":"Creating Signals - Agent Sensors","doc":"The simplest way to create a Signal is using `Jido.Signal.new/1`:\n\n```elixir\n{:ok, signal} = Jido.Signal.new(%{\n  source: \"my_component\",\n  type: \"user.registered\",\n  subject: \"user_123\",\n  data: %{\n    username: \"john.doe\",\n    timestamp: DateTime.utc_now()\n  }\n})\n```\n\nSignals are validated to ensure they contain required fields and follow the CloudEvents spec.","ref":"sensors.html#creating-signals"},{"type":"extras","title":"Publishing Signals - Agent Sensors","doc":"Signals are typically published through Phoenix.PubSub:\n\n```elixir\nPhoenix.PubSub.broadcast(pubsub, topic, signal)\n```\n\nThe signal will be received directly by any subscribers to that topic.","ref":"sensors.html#publishing-signals"},{"type":"extras","title":"Subscribing to Signals - Agent Sensors","doc":"To receive Signals, subscribe to the relevant PubSub topic:\n\n```elixir\nPhoenix.PubSub.subscribe(pubsub, \"registration_counter\")\n\n# In your process's handle_info:\ndef handle_info(%Signal{} = signal, state) do\n  # Process the signal\n  Logger.info(\"Received signal: #{signal.type} with data: #{inspect(signal.data)}\")\n  {:noreply, state}\nend\n```","ref":"sensors.html#subscribing-to-signals"},{"type":"extras","title":"Testing Sensors and Signals - Agent Sensors","doc":"For a complete example of how to test Sensors and Signals, refer to the registration counter test suite in `test/jido/sensor/examples/user_registration_sensor_test.exs`. This comprehensive test suite demonstrates:\n\n1. Setting up test PubSub environments\n2. Testing event counting and metric calculations\n3. Verifying signal emissions and formats\n4. Handling timing-dependent behavior\n\nThe test patterns shown there can be adapted for testing your own Sensors.","ref":"sensors.html#testing-sensors-and-signals"},{"type":"extras","title":"Best Practices - Agent Sensors","doc":"When working with Signals and Sensors, keep these principles in mind:\n\n1. **Signal Design**\n   - Use clear, consistent signal types\n   - Include enough context in payloads\n   - Follow the CloudEvents spec\n   - Keep payloads focused and minimal\n\n2. **Sensor Implementation**\n   - One responsibility per Sensor\n   - Clear configuration through schema\n   - Efficient state management\n   - Graceful error handling\n\n3. **Testing**\n   - Test both success and failure paths\n   - Verify metric calculations\n   - Check signal formats\n   - Test timing-dependent behavior\n\n4. **Performance**\n   - Use appropriate emission intervals\n   - Batch updates when possible\n   - Monitor memory usage\n   - Clean up old data","ref":"sensors.html#best-practices"},{"type":"extras","title":"Integration with Agents - Agent Sensors","doc":"Signals and Sensors lay the groundwork for advanced agent capabilities by providing:\n\n1. **Real-time Metrics**: Agents can use sensor data to make informed decisions\n2. **Event Streams**: Agents can react to system events through signals\n3. **Status Updates**: Agents can monitor each other's health\n4. **Command Results**: Agents can track the results of their actions\n\nIn the next guide, we'll explore how Server Agents use these signals for coordination and decision-making.","ref":"sensors.html#integration-with-agents"},{"type":"extras","title":"Next Steps - Agent Sensors","doc":"Now that you understand Signals and Sensors, you can explore:\n- Custom signal types\n- Complex metric calculations\n- Event stream processing\n- Multi-sensor coordination\n- Integration with monitoring systems\n- Historical analysis\n\nThe test suite provides many examples of these patterns in action.\n\nRemember: Signals and Sensors are your eyes and ears in the system. Design them thoughtfully to give your agents the information they need to make smart decisions.","ref":"sensors.html#next-steps"},{"type":"extras","title":"Skills","doc":"# Building Skills in JIDO","ref":"skills.html"},{"type":"extras","title":"Overview - Skills","doc":"Skills are the fundamental building blocks of agent capabilities in JIDO. A Skill encapsulates:\n\n- Signal routing and handling patterns\n- State management and isolation\n- Process supervision\n- Configuration management\n- Runtime adaptation\n\nThink of Skills as composable \"feature packs\" that give agents new abilities. Just as a human might learn new skills like \"cooking\" or \"programming\", JIDO agents gain new capabilities by incorporating Skills.","ref":"skills.html#overview"},{"type":"extras","title":"Core Concepts - Skills","doc":"","ref":"skills.html#core-concepts"},{"type":"extras","title":"1. Skill Structure - Skills","doc":"A Skill is defined by several key components:\n\n```elixir\ndefmodule MyApp.WeatherMonitorSkill do\n  use Jido.Skill,\n    name: \"weather_monitor\",\n    description: \"Monitors weather conditions and generates alerts\",\n    category: \"monitoring\",\n    tags: [\"weather\", \"alerts\"],\n    vsn: \"1.0.0\",\n    schema_key: :weather,\n    signals: [\n      input: [\"weather.data.received\", \"weather.alert.*\"],\n      output: [\"weather.alert.generated\"]\n    ],\n    config: [\n      weather_api: [\n        type: :map,\n        required: true,\n        doc: \"Weather API configuration\"\n      ]\n    ]\nend\n```\n\nLet's break down each component:\n\n- `name`: Unique identifier for the skill (required)\n- `description`: Human-readable explanation of the skill's purpose\n- `category`: Broad classification for organization\n- `tags`: List of searchable tags\n- `vsn`: Version string for compatibility checking\n- `schema_key`: Atom key for state namespace isolation\n- `signals`: Input/output signal patterns the skill handles\n- `config`: Configuration schema for validation","ref":"skills.html#1-skill-structure"},{"type":"extras","title":"2. State Management - Skills","doc":"Skills use `schema_key` for state namespace isolation. This prevents different skills from accidentally interfering with each other's state:\n\n```elixir\ndef initial_state do\n  %{\n    current_conditions: nil,\n    alert_history: [],\n    last_update: nil\n  }\nend\n```\n\nThis state will be stored under the skill's `schema_key` in the agent's state map:\n\n```elixir\n%{\n  weather: %{  # Matches schema_key\n    current_conditions: nil,\n    alert_history: [],\n    last_update: nil\n  }\n}\n```","ref":"skills.html#2-state-management"},{"type":"extras","title":"3. Signal Routing - Skills","doc":"Skills define signal routing patterns using a combination of exact matches, wildcards, and pattern matching functions:\n\n```elixir\ndef router do\n  [\n    # High priority alerts\n    %{\n      path: \"weather.alert.**\",\n      instruction: %{\n        action: Actions.GenerateWeatherAlert\n      },\n      priority: 100\n    },\n    \n    # Process incoming data\n    %{\n      path: \"weather.data.received\",\n      instruction: %{\n        action: Actions.ProcessWeatherData\n      }\n    },\n    \n    # Match severe conditions\n    %{\n      path: \"weather.condition.*\",\n      match: fn signal ->\n        get_in(signal.data, [:severity]) >= 3\n      end,\n      instruction: %{\n        action: Actions.GenerateWeatherAlert\n      },\n      priority: 75\n    }\n  ]\nend\n```","ref":"skills.html#3-signal-routing"},{"type":"extras","title":"4. Process Supervision - Skills","doc":"Skills can define child processes that need to run alongside the agent:\n\n```elixir\ndef child_spec(config) do\n  [\n    {WeatherAPI.Sensor,\n     [\n       name: \"weather_api\",\n       config: config.weather_api,\n       interval: :timer.minutes(15)\n     ]},\n    {WeatherAlerts.Monitor,\n     [\n       name: \"weather_alerts\",\n       config: config.alerts\n     ]}\n  ]\nend\n```","ref":"skills.html#4-process-supervision"},{"type":"extras","title":"Building Skills - Skills","doc":"","ref":"skills.html#building-skills"},{"type":"extras","title":"Step 1: Define the Skill Module - Skills","doc":"```elixir\ndefmodule MyApp.DataProcessingSkill do\n  use Jido.Skill,\n    name: \"data_processor\",\n    description: \"Processes and transforms data streams\",\n    schema_key: :processor,\n    signals: [\n      input: [\"data.received.*\", \"data.transform.*\"],\n      output: [\"data.processed.*\"]\n    ],\n    config: [\n      batch_size: [\n        type: :pos_integer,\n        default: 100,\n        doc: \"Number of items to process in each batch\"\n      ]\n    ]\nend\n```","ref":"skills.html#step-1-define-the-skill-module"},{"type":"extras","title":"Step 2: Implement Required Callbacks - Skills","doc":"```elixir\n# Initial state for the skill's namespace\ndef initial_state do\n  %{\n    processed_count: 0,\n    last_batch: nil,\n    error_count: 0\n  }\nend\n\n# Child processes to supervise\ndef child_spec(config) do\n  [\n    {DataProcessor.BatchWorker,\n     [\n       name: \"batch_worker\",\n       batch_size: config.batch_size\n     ]}\n  ]\nend\n\n# Signal routing rules\ndef router do\n  [\n    %{\n      path: \"data.received.*\",\n      instruction: %{\n        action: Actions.ProcessData\n      }\n    }\n  ]\nend\n```","ref":"skills.html#step-2-implement-required-callbacks"},{"type":"extras","title":"Step 3: Define Actions - Skills","doc":"```elixir\ndefmodule MyApp.DataProcessingSkill.Actions do\n  defmodule ProcessData do\n    use Jido.Action,\n      name: \"process_data\",\n      description: \"Processes incoming data batch\",\n      schema: [\n        data: [type: {:list, :map}, required: true]\n      ]\n\n    def run(%{data: data}, context) do\n      # Access skill config from context\n      batch_size = get_in(context, [:config, :batch_size])\n      \n      # Process data...\n      {:ok, %{\n        processed: transformed_data,\n        count: length(transformed_data)\n      }}\n    end\n  end\nend\n```","ref":"skills.html#step-3-define-actions"},{"type":"extras","title":"Testing Skills - Skills","doc":"","ref":"skills.html#testing-skills"},{"type":"extras","title":"1. Unit Testing Core Components - Skills","doc":"```elixir\ndefmodule MyApp.DataProcessingSkillTest do\n  use ExUnit.Case\n  alias MyApp.DataProcessingSkill\n  \n  describe \"skill configuration\" do\n    test \"validates config schema\" do\n      assert {:ok, config} = \n        Jido.Skill.validate_config(\n          DataProcessingSkill,\n          %{batch_size: 50}\n        )\n      \n      assert config.batch_size == 50\n    end\n    \n    test \"rejects invalid config\" do\n      assert {:error, error} = \n        Jido.Skill.validate_config(\n          DataProcessingSkill,\n          %{batch_size: -1}\n        )\n      \n      assert error.type == :validation_error\n    end\n  end\n  \n  describe \"signal validation\" do\n    test \"accepts valid signal patterns\" do\n      signal = %Jido.Signal{\n        type: \"data.received.batch\",\n        data: %{items: [1, 2, 3]}\n      }\n      \n      assert :ok = \n        Jido.Skill.validate_signal(\n          signal,\n          DataProcessingSkill.signals()\n        )\n    end\n  end\nend\n```","ref":"skills.html#1-unit-testing-core-components"},{"type":"extras","title":"2. Integration Testing - Skills","doc":"```elixir\ndefmodule MyApp.DataProcessingSkill.IntegrationTest do\n  use ExUnit.Case\n  \n  setup do\n    # Start necessary processes\n    start_supervised!(DataProcessor.BatchWorker)\n    :ok\n  end\n  \n  test \"processes data through complete flow\" do\n    # Create test signal\n    signal = %Jido.Signal{\n      type: \"data.received.batch\",\n      data: %{items: [1, 2, 3]}\n    }\n    \n    # Find matching route\n    [route] = Enum.filter(\n      DataProcessingSkill.router(),\n      &(&1.path == \"data.received.*\")\n    )\n    \n    # Execute action\n    {:ok, result} = route.instruction.action.run(\n      %{data: signal.data.items},\n      %{config: %{batch_size: 10}}\n    )\n    \n    assert result.count == 3\n    assert length(result.processed) == 3\n  end\nend\n```","ref":"skills.html#2-integration-testing"},{"type":"extras","title":"Best Practices - Skills","doc":"1. **State Isolation**\n   - Use meaningful `schema_key` names\n   - Keep state focused and minimal\n   - Document state structure\n   - Consider persistence needs\n\n2. **Signal Design**\n   - Use consistent naming patterns\n   - Document signal formats\n   - Include necessary context\n   - Consider routing efficiency\n\n3. **Configuration**\n   - Validate thoroughly\n   - Provide good defaults\n   - Document all options\n   - Consider runtime changes\n\n4. **Process Management**\n   - Supervise child processes\n   - Handle crashes gracefully\n   - Monitor resource usage\n   - Consider distribution\n\n5. **Testing**\n   - Test configuration validation\n   - Test signal routing\n   - Test state transitions\n   - Test process lifecycle\n   - Use property-based tests for complex logic","ref":"skills.html#best-practices"},{"type":"extras","title":"Common Patterns - Skills","doc":"","ref":"skills.html#common-patterns"},{"type":"extras","title":"1. Stateful Processing - Skills","doc":"```elixir\ndefmodule StatefulSkill do\n  use Jido.Skill,\n    name: \"stateful_processor\",\n    schema_key: :processor\n    \n  def router do\n    [\n      %{\n        path: \"data.received\",\n        instruction: %{\n          action: Actions.Process\n        }\n      }\n    ]\n  end\n  \n  # State updates are handled by the action itself\n  defmodule Actions.Process do\n    use Jido.Action,\n      name: \"process_data\"\n      \n    def run(params, context) do\n      # Process data and return updates for state\n      {:ok, %{\n        last_result: processed_data,\n        timestamp: DateTime.utc_now()\n      }}\n    end\n  end\nend\n```","ref":"skills.html#1-stateful-processing"},{"type":"extras","title":"2. Conditional Routing - Skills","doc":"```elixir\ndef router do\n  [\n    %{\n      path: \"event.*\",\n      match: fn signal ->\n        signal.data.priority == :high\n      end,\n      instruction: %{\n        action: Actions.HandleHighPriority\n      },\n      priority: 100\n    }\n  ]\nend\n```","ref":"skills.html#2-conditional-routing"},{"type":"extras","title":"3. State Management - Skills","doc":"```elixir\ndefmodule StateManagementSkill do\n  use Jido.Skill,\n    name: \"state_manager\",\n    schema_key: :manager\n\n  def router do\n    [\n      %{\n        path: \"data.update\",\n        instruction: %{\n          action: Actions.UpdateData\n        }\n      }\n    ]\n  end\n  \n  defmodule Actions.UpdateData do\n    use Jido.Action,\n      name: \"update_data\"\n      \n    def run(_params, context) do\n      # Actions can read current state from context\n      current_count = get_in(context, [:state, :manager, :count]) || 0\n      \n      # Return updates to be applied to state\n      {:ok, %{\n        count: current_count + 1,\n        last_update: DateTime.utc_now()\n      }}\n    end\n  end\nend\n```","ref":"skills.html#3-state-management"},{"type":"extras","title":"Troubleshooting - Skills","doc":"Common issues and solutions:\n\n1. **Signal Not Routing**\n   - Check signal type matches patterns\n   - Verify skill is registered with agent\n   - Check priority conflicts\n   - Enable debug logging\n\n2. **State Not Updating**\n   - Verify transform function\n   - Check schema_key path\n   - Validate state structure\n   - Check action results\n\n3. **Process Crashes**\n   - Review supervision strategy\n   - Check resource limits\n   - Monitor error counts\n   - Add detailed logging","ref":"skills.html#troubleshooting"},{"type":"extras","title":"Advanced Topics - Skills","doc":"","ref":"skills.html#advanced-topics"},{"type":"extras","title":"1. Dynamic Configuration - Skills","doc":"Skills can adapt their behavior based on configuration:\n\n```elixir\ndefmodule DynamicSkill do\n  use Jido.Skill,\n    name: \"dynamic_processor\",\n    schema_key: :processor,\n    config: [\n      mode: [\n        type: {:enum, [:fast, :thorough]},\n        default: :fast,\n        doc: \"Processing mode to use\"\n      ]\n    ]\n\n  def router do\n    [\n      %{\n        path: \"data.*\",\n        match: fn signal -> \n          # Can use config to determine routing\n          config = signal.metadata.config\n          should_process?(signal.data, config.mode)\n        end,\n        instruction: %{\n          action: Actions.ProcessData\n        }\n      }\n    ]\n  end\n  \n  defmodule Actions.ProcessData do\n    use Jido.Action,\n      name: \"process_data\"\n      \n    def run(params, context) do\n      # Access config from context to determine behavior\n      mode = get_in(context, [:config, :mode])\n      \n      result = case mode do\n        :fast -> quick_process(params)\n        :thorough -> detailed_process(params)\n      end\n      \n      {:ok, result}\n    end\n  end\nend\n```","ref":"skills.html#1-dynamic-configuration"},{"type":"extras","title":"2. Composable Skills - Skills","doc":"Skills can be composed to build more complex capabilities:\n\n```elixir\ndefmodule CompositeSkill do\n  use Jido.Skill,\n    name: \"composite\",\n    signals: merge_signals([\n      WeatherSkill.signals(),\n      AlertSkill.signals()\n    ])\n    \n  def child_spec(config) do\n    WeatherSkill.child_spec(config) ++\n    AlertSkill.child_spec(config)\n  end\nend\n```","ref":"skills.html#2-composable-skills"},{"type":"extras","title":"3. Distributed Skills - Skills","doc":"Skills can operate across nodes:\n\n```elixir\ndef child_spec(config) do\n  [\n    {DistributedWorker,\n     [\n       name: {:global, \"worker_1\"},\n       nodes: config.cluster_nodes\n     ]}\n  ]\nend\n```","ref":"skills.html#3-distributed-skills"},{"type":"extras","title":"Conclusion - Skills","doc":"Skills are a powerful abstraction for building modular, composable agent capabilities. By following the patterns and practices in this guide, you can create robust, maintainable skills that enhance your agents' abilities while maintaining clean separation of concerns.\n\nRemember:\n- Keep skills focused and single-purpose\n- Design clear signal interfaces\n- Manage state carefully\n- Test thoroughly\n- Document extensively\n\nFor more examples and advanced patterns, refer to the test suite and example implementations in the JIDO codebase.","ref":"skills.html#conclusion"},{"type":"extras","title":"Signal Router","doc":"# Signal Router\n\nIn our previous guides, we explored how Signals provide real-time event streams and how the Signal Bus enables message distribution. Now, let's discover how the Signal Router enables sophisticated message routing using Instructions in your Jido system.","ref":"signal-router.html"},{"type":"extras","title":"Understanding the Router - Signal Router","doc":"The Signal Router is a powerful trie-based message routing system that:\n\n- Routes signals to appropriate Instructions based on path patterns\n- Supports both wildcards and pattern matching functions\n- Manages execution priority and complexity-based ordering\n- Provides efficient trie-based path matching\n- Integrates with Jido's Instruction system\n\nThink of the Router as a smart traffic controller for your signals - it ensures each signal reaches its intended handlers in the correct order, supporting both simple path matching and complex routing rules.","ref":"signal-router.html#understanding-the-router"},{"type":"extras","title":"Core Concepts - Signal Router","doc":"Before diving into code, let's understand the key concepts:","ref":"signal-router.html#core-concepts"},{"type":"extras","title":"Routes - Signal Router","doc":"A route connects a signal path pattern to one or more Instructions. Routes support:\n\n- Exact matches: `\"user.created\"`\n- Single-level wildcards: `\"user.*.updated\"`\n- Multi-level wildcards: `\"order.**.completed\"`","ref":"signal-router.html#routes"},{"type":"extras","title":"Instructions - Signal Router","doc":"Instructions wrapped in routes:\n\n- Define an Action module to execute\n- Can have priorities (-100 to 100)\n- May include pattern matching functions\n- Integrate with Jido's execution system","ref":"signal-router.html#instructions"},{"type":"extras","title":"Priority & Complexity - Signal Router","doc":"Handlers execute based on:\n\n- Priority (higher numbers execute first)\n- Path complexity scoring\n- Registration order for same priority/complexity","ref":"signal-router.html#priority-complexity"},{"type":"extras","title":"Creating Your First Router - Signal Router","doc":"Let's create a router to handle user-related signals:\n\n```elixir\n# Define some actions\ndefmodule MyApp.Actions.AuditUser do\n  use Jido.Action,\n    name: \"audit_user\",\n    description: \"Logs user events for auditing\",\n    schema: [\n      signal: [type: :map, required: true]\n    ]\n\n  def run(%{signal: signal}, _context) do\n    Logger.info(\"User event: #{signal.type}\")\n    {:ok, :audited}\n  end\nend\n\ndefmodule MyApp.Actions.HandlePremiumUser do\n  use Jido.Action,\n    name: \"handle_premium_user\",\n    schema: [\n      signal: [type: :map, required: true]\n    ]\n\n  def run(%{signal: signal}, _context) do\n    if signal.data.premium do\n      Process.send(PremiumWorker, {:process, signal}, [])\n      {:ok, :premium_queued}\n    else\n      {:ok, :ignored}\n    end\n  end\nend\n\n# Create a router with multiple routes\n{:ok, router} = Jido.Signal.Router.new([\n  # High priority audit logging\n  {\"user.**\", %Instruction{\n    action: MyApp.Actions.AuditUser\n  }, 100},\n\n  # Pattern match for premium users\n  {\"user.*.updated\",\n    fn signal -> Map.get(signal.data, :premium, false) end,\n    %Instruction{\n      action: MyApp.Actions.HandlePremiumUser\n    },\n    75},\n\n  # Standard notification handling\n  {\"user.*.notify\", %Instruction{\n    action: MyApp.Actions.NotifyUser\n  }}\n])\n```\n\nLet's break down what's happening:\n\n1. We define Actions that handle different aspects of user signals\n2. We create a router with multiple routes using different patterns\n3. Each route specifies:\n   - A path pattern\n   - An optional pattern matching function\n   - An Instruction to execute\n   - An optional priority","ref":"signal-router.html#creating-your-first-router"},{"type":"extras","title":"Route Types - Signal Router","doc":"The router supports several route specifications:","ref":"signal-router.html#route-types"},{"type":"extras","title":"1. Basic Route with Instruction - Signal Router","doc":"```elixir\n# Simple path and instruction\n{\"user.created\", %Instruction{\n  action: MyApp.Actions.HandleUserCreated\n}}\n\n# With priority (-100 to 100)\n{\"user.created\", %Instruction{\n  action: MyApp.Actions.HandleUserCreated\n}, 90}\n```","ref":"signal-router.html#1-basic-route-with-instruction"},{"type":"extras","title":"2. Pattern Matching Routes - Signal Router","doc":"```elixir\n# With match function\n{\n  \"payment.processed\",\n  fn signal -> signal.data.amount > 1000 end,\n  %Instruction{\n    action: MyApp.Actions.HandleLargePayment\n  }\n}\n\n# With match and priority\n{\n  \"payment.processed\",\n  fn signal -> signal.data.amount > 1000 end,\n  %Instruction{\n    action: MyApp.Actions.HandleLargePayment\n  },\n  90\n}\n```","ref":"signal-router.html#2-pattern-matching-routes"},{"type":"extras","title":"3. Route Structs - Signal Router","doc":"For maximum control:\n\n```elixir\n%Route{\n  path: \"user.created\",\n  instruction: %Instruction{\n    action: MyApp.Actions.HandleUserCreated\n  },\n  priority: 90,\n  match: &MyModule.match_premium/1\n}\n```","ref":"signal-router.html#3-route-structs"},{"type":"extras","title":"Path Matching Rules - Signal Router","doc":"The router implements sophisticated path matching:","ref":"signal-router.html#path-matching-rules"},{"type":"extras","title":"1. Static Segments - Signal Router","doc":"- Must match exactly\n- Can contain alphanumeric characters, underscores, and hyphens\n- Cannot contain consecutive dots","ref":"signal-router.html#1-static-segments"},{"type":"extras","title":"2. Single Wildcards (\\*) - Signal Router","doc":"- Match exactly one path segment\n- Can appear anywhere in the path\n- Multiple wildcards allowed","ref":"signal-router.html#2-single-wildcards"},{"type":"extras","title":"3. Multi-Level Wildcards (\\*\\*) - Signal Router","doc":"- Match zero or more segments\n- Cannot have consecutive multi-wildcards\n- Most expensive match type","ref":"signal-router.html#3-multi-level-wildcards"},{"type":"extras","title":"Complexity Scoring - Signal Router","doc":"The router scores paths to ensure most specific matches take precedence:\n\n1. Base score from segment count\n2. Bonuses for exact matches (higher at start of path)\n3. Penalties for wildcards (higher for \\*_ than _)\n4. Position weighting (earlier segments worth more)","ref":"signal-router.html#complexity-scoring"},{"type":"extras","title":"Managing Routes - Signal Router","doc":"Routes can be dynamically managed:","ref":"signal-router.html#managing-routes"},{"type":"extras","title":"Adding Routes - Signal Router","doc":"```elixir\n{:ok, router} = Router.add(router, [\n  {\"metrics.**\", %Instruction{\n    action: MetricsHandler.Collect\n  }},\n  {\"audit.*\", %Instruction{\n    action: AuditHandler.Log\n  }, 75}\n])\n```","ref":"signal-router.html#adding-routes"},{"type":"extras","title":"Removing Routes - Signal Router","doc":"```elixir\n# Remove a single route\n{:ok, router} = Router.remove(router, \"metrics.**\")\n\n# Remove multiple routes\n{:ok, router} = Router.remove(router, [\n  \"audit.*\",\n  \"user.created\"\n])\n```","ref":"signal-router.html#removing-routes"},{"type":"extras","title":"Listing Routes - Signal Router","doc":"```elixir\n{:ok, routes} = Router.list(router)\n\n# Returns list of Route structs\n[\n  %Route{\n    path: \"user.created\",\n    instruction: %Instruction{...},\n    priority: 0,\n    match: nil\n  }\n]\n```","ref":"signal-router.html#listing-routes"},{"type":"extras","title":"Integration with Agents - Signal Router","doc":"The Router integrates naturally with Agents:\n\n```elixir\ndefmodule MyApp.UserAgent do\n  use Jido.Agent,\n    name: \"user_agent\",\n    actions: [\n      MyApp.Actions.HandleUserCreated,\n      MyApp.Actions.HandleUserUpdated,\n      MyApp.Actions.AuditUser\n    ]\n\n  def init(opts) do\n    {:ok, router} = Router.new([\n      {\"user.created\", %Instruction{\n        action: MyApp.Actions.HandleUserCreated\n      }},\n      {\"user.*.updated\", %Instruction{\n        action: MyApp.Actions.HandleUserUpdated\n      }},\n      {\"user.**\", %Instruction{\n        action: MyApp.Actions.AuditUser\n      }, 100}\n    ])\n\n    {:ok, Map.put(opts, :router, router)}\n  end\n\n  def handle_signal(%Signal{} = signal, state) do\n    case Router.route(state.router, signal) do\n      {:ok, instructions} ->\n        # Execute instructions in priority/complexity order\n        Enum.reduce_while(instructions, {:ok, state}, fn instruction, {:ok, state} ->\n          case run(instruction, state) do\n            {:ok, new_state} -> {:cont, {:ok, new_state}}\n            error -> {:halt, error}\n          end\n        end)\n\n      {:error, _} = error -> error\n    end\n  end\nend\n```","ref":"signal-router.html#integration-with-agents"},{"type":"extras","title":"Best Practices - Signal Router","doc":"When working with the router, keep these principles in mind:","ref":"signal-router.html#best-practices"},{"type":"extras","title":"1. Path Design - Signal Router","doc":"- Use consistent, meaningful path segments\n- Prefer exact matches over wildcards\n- Put more specific routes first\n- Document path patterns\n- Consider future extensibility","ref":"signal-router.html#1-path-design"},{"type":"extras","title":"2. Priority Management - Signal Router","doc":"- Use priority ranges thoughtfully\n- Reserve high/low ends for special cases\n- Document priority meanings\n- Consider complexity scoring\n- Test priority interactions","ref":"signal-router.html#2-priority-management"},{"type":"extras","title":"3. Pattern Matching - Signal Router","doc":"- Keep match functions simple\n- Handle nil/missing data gracefully\n- Document matching conditions\n- Test edge cases thoroughly\n- Consider performance impact","ref":"signal-router.html#3-pattern-matching"},{"type":"extras","title":"4. Route Organization - Signal Router","doc":"- Group related functionality\n- Use consistent naming\n- Document path hierarchies\n- Consider maintenance\n- Plan for scale","ref":"signal-router.html#4-route-organization"},{"type":"extras","title":"5. Error Handling - Signal Router","doc":"- Validate routes early\n- Handle missing routes gracefully\n- Use Error structs consistently\n- Log routing errors\n- Provide clear context","ref":"signal-router.html#5-error-handling"},{"type":"extras","title":"Testing - Signal Router","doc":"Here's how to thoroughly test your routing:\n\n```elixir\ndefmodule MyApp.RouterTest do\n  use ExUnit.Case\n\n  alias Jido.Signal\n  alias Jido.Instruction\n\n  setup do\n    routes = [\n      {\"user.created\", %Instruction{\n        action: MyApp.Actions.HandleUserCreated\n      }},\n      {\"user.*.updated\", %Instruction{\n        action: MyApp.Actions.HandleUserUpdated\n      }},\n      {\"user.**\", %Instruction{\n        action: MyApp.Actions.AuditUser\n      }, 100}\n    ]\n\n    {:ok, router} = Router.new(routes)\n    %{router: router}\n  end\n\n  test \"routes signals correctly\", %{router: router} do\n    signal = %Signal{\n      id: Jido.Util.generate_id(),\n      source: \"/test\",\n      type: \"user.123.updated\",\n      data: %{}\n    }\n\n    {:ok, instructions} = Router.route(router, signal)\n\n    assert length(instructions) == 2  # audit and update handlers\n    assert Enum.any?(instructions, & &1.action == MyApp.Actions.AuditUser)\n    assert Enum.any?(instructions, & &1.action == MyApp.Actions.HandleUserUpdated)\n  end\n\n  test \"respects priority and complexity\", %{router: router} do\n    signal = %Signal{\n      id: Jido.Util.generate_id(),\n      source: \"/test\",\n      type: \"user.created\",\n      data: %{}\n    }\n\n    {:ok, [first | _]} = Router.route(router, signal)\n    assert first.action == MyApp.Actions.AuditUser\n  end\n\n  test \"handles edge cases\" do\n    # Test empty segments\n    assert {:error, _} = Router.new({\"user..created\", instruction})\n\n    # Test consecutive wildcards\n    assert {:error, _} = Router.new({\"user.**.**.created\", instruction})\n\n    # Test priority bounds\n    assert {:error, _} = Router.new({\"test\", instruction, 101})\n\n    # Test pattern matching errors\n    assert {:error, _} = Router.new({\n      \"test\",\n      fn _ -> raise \"boom\" end,\n      instruction\n    })\n  end\nend\n```","ref":"signal-router.html#testing"},{"type":"extras","title":"Next Steps - Signal Router","doc":"Now that you understand the Router, you can explore:\n\n- Complex routing patterns\n- Dynamic route management\n- Custom pattern matching\n- Performance optimization\n- Integration with other Jido components\n\nRemember: The Router is your signal traffic controller. Design your routes thoughtfully to ensure signals flow efficiently to their proper destinations.","ref":"signal-router.html#next-steps"},{"type":"extras","title":"Contributing","doc":"# Contributing to Jido\n\nWelcome to the Jido contributor's guide! We're excited that you're interested in contributing to Jido, our framework for building autonomous, distributed agent systems in Elixir. This guide will help you understand our development process and standards.","ref":"contributing.html"},{"type":"extras","title":"Getting Started - Contributing","doc":"","ref":"contributing.html#getting-started"},{"type":"extras","title":"Development Environment - Contributing","doc":"1. **Elixir Version Requirements**\n   - Jido requires Elixir ~> 1.17\n   - We recommend using asdf or similar version manager\n\n2. **Initial Setup**\n   ```bash\n   # Clone the repository\n   git clone https://github.com/agentjido/jido.git\n   cd jido\n\n   # Install dependencies\n   mix deps.get\n\n   # Run tests to verify your setup\n   mix test\n   ```\n\n3. **Quality Checks**\n   ```bash\n   # Run the full quality check suite\n   mix quality\n\n   # Or individual checks\n   mix format                    # Format code\n   mix compile --warnings-as-errors  # Check compilation\n   mix dialyzer                  # Type checking\n   mix credo --all              # Static analysis\n   ```","ref":"contributing.html#development-environment"},{"type":"extras","title":"Code Organization - Contributing","doc":"","ref":"contributing.html#code-organization"},{"type":"extras","title":"Project Structure - Contributing","doc":"```\n.\n├── lib/\n│   ├── jido/\n│   │   ├── actions/     # Core action implementations\n│   │   ├── agents/      # Agent behaviors and implementations\n│   │   ├── sensors/     # Sensor system components\n│   │   ├── workflows/   # Workflow execution engines\n│   │   └── utils/       # Utility functions\n│   └── jido.ex          # Main entry point\n├── test/\n│   ├── jido/\n│   │   └── ...         # Tests mirroring lib structure\n│   ├── support/        # Test helpers and shared fixtures\n│   └── test_helper.exs\n├── guides/            # Documentation guides\n└── mix.exs\n```","ref":"contributing.html#project-structure"},{"type":"extras","title":"Core Components - Contributing","doc":"- **Actions**: Discrete, composable units of work\n- **Workflows**: Sequences of actions that accomplish larger goals\n- **Agents**: Stateful entities that can plan and execute workflows\n- **Sensors**: Real-time monitoring and data gathering components","ref":"contributing.html#core-components"},{"type":"extras","title":"Development Guidelines - Contributing","doc":"","ref":"contributing.html#development-guidelines"},{"type":"extras","title":"Code Style - Contributing","doc":"1. **Formatting**\n   - Run `mix format` before committing\n   - Follow standard Elixir style guide\n   - Use `snake_case` for functions and variables\n   - Use `PascalCase` for module names\n\n2. **Documentation**\n   - Add `@moduledoc` to every module\n   - Document all public functions with `@doc`\n   - Include examples when helpful\n   - Keep guides up-to-date in the `guides/` directory\n   - Use doctests for simple examples\n\n3. **Type Specifications**\n   ```elixir\n   @type validation_error :: :invalid_name | :invalid_status\n   \n   @spec process(String.t()) :: {:ok, term()} | {:error, validation_error()}\n   def process(input) do\n     # Implementation\n   end\n   ```","ref":"contributing.html#code-style"},{"type":"extras","title":"Testing - Contributing","doc":"1. **Test Organization**\n   ```elixir\n   defmodule Jido.Test.Actions.FormatUserTest do\n     use ExUnit.Case, async: true\n     \n     describe \"run/2\" do\n       test \"formats user data correctly\" do\n         # Test implementation\n       end\n       \n       test \"handles invalid input\" do\n         # Error case testing\n       end\n     end\n   end\n   ```\n\n2. **Coverage Requirements**\n   - Maintain high test coverage\n   - Test both success and error paths\n   - Include property-based tests for complex logic\n   - Test async behavior where applicable\n\n3. **Running Tests**\n   ```bash\n   # Run full test suite\n   mix test\n\n   # Run with coverage\n   mix test --cover\n\n   # Run specific test file\n   mix test test/jido/actions/format_user_test.exs\n   ```","ref":"contributing.html#testing"},{"type":"extras","title":"Error Handling - Contributing","doc":"1. **Use With Patterns**\n   ```elixir\n   def complex_operation(input) do\n     with {:ok, validated} <- validate(input),\n          {:ok, processed} <- process(validated) do\n       {:ok, processed}\n     end\n   end\n   ```\n\n2. **Return Values**\n   - Use tagged tuples: `{:ok, result}` or `{:error, reason}`\n   - Create specific error types for different failures\n   - Never intentionally raise Exceptions, the Jido framework intentionally minimizes the use of exceptions.\n   - Avoid silent failures\n   - Document error conditions","ref":"contributing.html#error-handling"},{"type":"extras","title":"Performance Considerations - Contributing","doc":"1. **Optimization**\n   - Profile before optimizing\n   - Document performance characteristics\n   - Consider resource usage in distributed environments\n   - Implement appropriate timeouts\n\n2. **Resource Management**\n   - Clean up resources properly\n   - Handle large data sets efficiently\n   - Consider memory usage in long-running processes","ref":"contributing.html#performance-considerations"},{"type":"extras","title":"Pull Request Process - Contributing","doc":"1. **Before Submitting**\n   - Run the full quality check suite: `mix quality`\n   - Ensure all tests pass\n   - Update documentation if needed\n   - Add tests for new functionality\n\n2. **PR Guidelines**\n   - Create a feature branch from `main`\n   - Use descriptive commit messages\n   - Reference any related issues\n   - Keep changes focused and atomic\n\n3. **Review Process**\n   - PRs require at least one review\n   - Address all review comments\n   - Maintain a clean commit history\n   - Update your branch if needed","ref":"contributing.html#pull-request-process"},{"type":"extras","title":"Release Process - Contributing","doc":"1. **Version Numbers**\n   - Follow semantic versioning\n   - Update version in `mix.exs`\n   - Update CHANGELOG.md\n\n2. **Documentation**\n   - Update guides if needed\n   - Check all docstrings\n   - Verify README is current","ref":"contributing.html#release-process"},{"type":"extras","title":"Additional Resources - Contributing","doc":"- [Hex Documentation](https://hexdocs.pm/jido)\n- [GitHub Issues](https://github.com/agentjido/jido/issues)\n- [GitHub Discussions](https://github.com/agentjido/jido/discussions)","ref":"contributing.html#additional-resources"},{"type":"extras","title":"Questions or Problems? - Contributing","doc":"If you have questions about contributing:\n- Open a GitHub Discussion\n- Check existing issues\n- Review the guides directory\n\nThank you for contributing to Jido!","ref":"contributing.html#questions-or-problems"},{"type":"extras","title":"Changelog","doc":"# CHANGELOG\n\n<!-- %% CHANGELOG_ENTRIES %% -->","ref":"changelog.html"},{"type":"extras","title":"1.1.0-rc - 2025-02-08 - Changelog","doc":"- Stateful Agents","ref":"changelog.html#1-1-0-rc-2025-02-08"},{"type":"extras","title":"Apache 2.0 License","doc":"Apache License\n                           Version 2.0, January 2004\n                        http://www.apache.org/licenses/\n\n   TERMS AND CONDITIONS FOR USE, REPRODUCTION, AND DISTRIBUTION\n\n   1. Definitions.\n\n      \"License\" shall mean the terms and conditions for use, reproduction,\n      and distribution as defined by Sections 1 through 9 of this document.\n\n      \"Licensor\" shall mean the copyright owner or entity authorized by\n      the copyright owner that is granting the License.\n\n      \"Legal Entity\" shall mean the union of the acting entity and all\n      other entities that control, are controlled by, or are under common\n      control with that entity. For the purposes of this definition,\n      \"control\" means (i) the power, direct or indirect, to cause the\n      direction or management of such entity, whether by contract or\n      otherwise, or (ii) ownership of fifty percent (50%) or more of the\n      outstanding shares, or (iii) beneficial ownership of such entity.\n\n      \"You\" (or \"Your\") shall mean an individual or Legal Entity\n      exercising permissions granted by this License.\n\n      \"Source\" form shall mean the preferred form for making modifications,\n      including but not limited to software source code, documentation\n      source, and configuration files.\n\n      \"Object\" form shall mean any form resulting from mechanical\n      transformation or translation of a Source form, including but\n      not limited to compiled object code, generated documentation,\n      and conversions to other media types.\n\n      \"Work\" shall mean the work of authorship, whether in Source or\n      Object form, made available under the License, as indicated by a\n      copyright notice that is included in or attached to the work\n      (an example is provided in the Appendix below).\n\n      \"Derivative Works\" shall mean any work, whether in Source or Object\n      form, that is based on (or derived from) the Work and for which the\n      editorial revisions, annotations, elaborations, or other modifications\n      represent, as a whole, an original work of authorship. For the purposes\n      of this License, Derivative Works shall not include works that remain\n      separable from, or merely link (or bind by name) to the interfaces of,\n      the Work and Derivative Works thereof.\n\n      \"Contribution\" shall mean any work of authorship, including\n      the original version of the Work and any modifications or additions\n      to that Work or Derivative Works thereof, that is intentionally\n      submitted to Licensor for inclusion in the Work by the copyright owner\n      or by an individual or Legal Entity authorized to submit on behalf of\n      the copyright owner. For the purposes of this definition, \"submitted\"\n      means any form of electronic, verbal, or written communication sent\n      to the Licensor or its representatives, including but not limited to\n      communication on electronic mailing lists, source code control systems,\n      and issue tracking systems that are managed by, or on behalf of, the\n      Licensor for the purpose of discussing and improving the Work, but\n      excluding communication that is conspicuously marked or otherwise\n      designated in writing by the copyright owner as \"Not a Contribution.\"\n\n      \"Contributor\" shall mean Licensor and any individual or Legal Entity\n      on behalf of whom a Contribution has been received by Licensor and\n      subsequently incorporated within the Work.\n\n   2. Grant of Copyright License. Subject to the terms and conditions of\n      this License, each Contributor hereby grants to You a perpetual,\n      worldwide, non-exclusive, no-charge, royalty-free, irrevocable\n      copyright license to reproduce, prepare Derivative Works of,\n      publicly display, publicly perform, sublicense, and distribute the\n      Work and such Derivative Works in Source or Object form.\n\n   3. Grant of Patent License. Subject to the terms and conditions of\n      this License, each Contributor hereby grants to You a perpetual,\n      worldwide, non-exclusive, no-charge, royalty-free, irrevocable\n      (except as stated in this section) patent license to make, have made,\n      use, offer to sell, sell, import, and otherwise transfer the Work,\n      where such license applies only to those patent claims licensable\n      by such Contributor that are necessarily infringed by their\n      Contribution(s) alone or by combination of their Contribution(s)\n      with the Work to which such Contribution(s) was submitted. If You\n      institute patent litigation against any entity (including a\n      cross-claim or counterclaim in a lawsuit) alleging that the Work\n      or a Contribution incorporated within the Work constitutes direct\n      or contributory patent infringement, then any patent licenses\n      granted to You under this License for that Work shall terminate\n      as of the date such litigation is filed.\n\n   4. Redistribution. You may reproduce and distribute copies of the\n      Work or Derivative Works thereof in any medium, with or without\n      modifications, and in Source or Object form, provided that You\n      meet the following conditions:\n\n      (a) You must give any other recipients of the Work or\n          Derivative Works a copy of this License; and\n\n      (b) You must cause any modified files to carry prominent notices\n          stating that You changed the files; and\n\n      (c) You must retain, in the Source form of any Derivative Works\n          that You distribute, all copyright, patent, trademark, and\n          attribution notices from the Source form of the Work,\n          excluding those notices that do not pertain to any part of\n          the Derivative Works; and\n\n      (d) If the Work includes a \"NOTICE\" text file as part of its\n          distribution, then any Derivative Works that You distribute must\n          include a readable copy of the attribution notices contained\n          within such NOTICE file, excluding those notices that do not\n          pertain to any part of the Derivative Works, in at least one\n          of the following places: within a NOTICE text file distributed\n          as part of the Derivative Works; within the Source form or\n          documentation, if provided along with the Derivative Works; or,\n          within a display generated by the Derivative Works, if and\n          wherever such third-party notices normally appear. The contents\n          of the NOTICE file are for informational purposes only and\n          do not modify the License. You may add Your own attribution\n          notices within Derivative Works that You distribute, alongside\n          or as an addendum to the NOTICE text from the Work, provided\n          that such additional attribution notices cannot be construed\n          as modifying the License.\n\n      You may add Your own copyright statement to Your modifications and\n      may provide additional or different license terms and conditions\n      for use, reproduction, or distribution of Your modifications, or\n      for any such Derivative Works as a whole, provided Your use,\n      reproduction, and distribution of the Work otherwise complies with\n      the conditions stated in this License.\n\n   5. Submission of Contributions. Unless You explicitly state otherwise,\n      any Contribution intentionally submitted for inclusion in the Work\n      by You to the Licensor shall be under the terms and conditions of\n      this License, without any additional terms or conditions.\n      Notwithstanding the above, nothing herein shall supersede or modify\n      the terms of any separate license agreement you may have executed\n      with Licensor regarding such Contributions.\n\n   6. Trademarks. This License does not grant permission to use the trade\n      names, trademarks, service marks, or product names of the Licensor,\n      except as required for reasonable and customary use in describing the\n      origin of the Work and reproducing the content of the NOTICE file.\n\n   7. Disclaimer of Warranty. Unless required by applicable law or\n      agreed to in writing, Licensor provides the Work (and each\n      Contributor provides its Contributions) on an \"AS IS\" BASIS,\n      WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or\n      implied, including, without limitation, any warranties or conditions\n      of TITLE, NON-INFRINGEMENT, MERCHANTABILITY, or FITNESS FOR A\n      PARTICULAR PURPOSE. You are solely responsible for determining the\n      appropriateness of using or redistributing the Work and assume any\n      risks associated with Your exercise of permissions under this License.\n\n   8. Limitation of Liability. In no event and under no legal theory,\n      whether in tort (including negligence), contract, or otherwise,\n      unless required by applicable law (such as deliberate and grossly\n      negligent acts) or agreed to in writing, shall any Contributor be\n      liable to You for damages, including any direct, indirect, special,\n      incidental, or consequential damages of any character arising as a\n      result of this License or out of the use or inability to use the\n      Work (including but not limited to damages for loss of goodwill,\n      work stoppage, computer failure or malfunction, or any and all\n      other commercial damages or losses), even if such Contributor\n      has been advised of the possibility of such damages.\n\n   9. Accepting Warranty or Additional Liability. While redistributing\n      the Work or Derivative Works thereof, You may choose to offer,\n      and charge a fee for, acceptance of support, warranty, indemnity,\n      or other liability obligations and/or rights consistent with this\n      License. However, in accepting such obligations, You may act only\n      on Your own behalf and on Your sole responsibility, not on behalf\n      of any other Contributor, and only if You agree to indemnify,\n      defend, and hold each Contributor harmless for any liability\n      incurred by, or claims asserted against, such Contributor by reason\n      of your accepting any such warranty or additional liability.\n\n   END OF TERMS AND CONDITIONS\n\n   APPENDIX: How to apply the Apache License to your work.\n\n      To apply the Apache License to your work, attach the following\n      boilerplate notice, with the fields enclosed by brackets \"[]\"\n      replaced with your own identifying information. (Don't include\n      the brackets!)  The text should be enclosed in the appropriate\n      comment syntax for the file format. We also recommend that a\n      file or class name and description of purpose be included on the\n      same \"printed page\" as the copyright notice for easier\n      identification within third-party archives.\n\n   Copyright 2019 Parker Selbert\n\n   Licensed under the Apache License, Version 2.0 (the \"License\");\n   you may not use this file except in compliance with the License.\n   You may obtain a copy of the License at\n\n       http://www.apache.org/licenses/LICENSE-2.0\n\n   Unless required by applicable law or agreed to in writing, software\n   distributed under the License is distributed on an \"AS IS\" BASIS,\n   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.\n   See the License for the specific language governing permissions and\n   limitations under the License.","ref":"license.html"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.37.0"}}